@@ -1,5 +1,5 @@
-/*  Amplitude  A non-negative PredictionValue.
-    Last modified 1997-11-02
+/*  $Id: Amplitude.cc,v 1.2 2004/10/19 18:01:37 flaterco Exp $
+    Amplitude  A non-negative PredictionValue.
 
     Copyright (C) 1997  David Flater.
 
@@ -25,7 +25,7 @@ PredictionValue (inunits, inval) {
   assert (inval >= 0.0);
 }
 
-Amplitude::Amplitude (): PredictionValue () {};
+Amplitude::Amplitude (): PredictionValue () {}
 
 void Amplitude::val (double newval) {
   assert (newval >= 0.0);
