@@ -1,4 +1,4 @@
-// $Id: Angle.cc,v 1.2 2002/10/04 13:44:00 flaterco Exp $
+// $Id: Angle.cc,v 1.4 2004/10/19 18:02:57 flaterco Exp $
 // Angle:  abstraction to partially alleviate degrees versus radians debate.
 // All angles are "normalized" to the 0-2pi rad (0-360 degree) range.
 
@@ -27,6 +27,11 @@ Angle::normalize() {
   radians = fmod (radians, 2.0 * M_PI);
   if (radians < 0.0)
     radians += 2.0 * M_PI;
+  // This can happen because of floating point roundoff if you add a very
+  // small negative number to 2 PI.
+  if (radians == 2.0 * M_PI)
+    radians = 0.0;
+  assert (radians >= 0.0 && radians < 2.0 * M_PI);
 }
 
 void Angle::setup (units u, qualifier q, double v) {
@@ -68,37 +73,32 @@ double Angle::rad () {
   return radians;
 }
 
-Dstr &
-Angle::ppdeg (unsigned precision) {
-  static Dstr retbuf;
-  char tempbuf[80];
+void Angle::ppdeg (unsigned precision, Dstr &buf) {
+  char retbuf[80];
   int width = precision + 4;
   assert (width < 80);
   if (precision == 0)
     width = 3;
-  sprintf (tempbuf, "%*.*f", width, (int)(precision), deg());
+  sprintf (retbuf, "%*.*f", width, (int)(precision), deg());
 
   // The rounding can result in us seeing 360.00 -- but we don't want that.
-  if (tempbuf[0] == '3' && tempbuf[1] == '6')
-    tempbuf[0] = tempbuf[1] = ' ';
+  if (retbuf[0] == '3' && retbuf[1] == '6')
+    retbuf[0] = retbuf[1] = ' ';
 
-  retbuf = tempbuf;
-  return retbuf;
+  buf = retbuf;
 }
 
-Dstr &Angle::ppqdeg (unsigned precision) {
-  static Dstr retbuf;
-  retbuf = ppdeg (precision);
-  retbuf += '\260';
+void Angle::ppqdeg (unsigned precision, Dstr &buf) {
+  ppdeg (precision, buf);
+  buf += '\260';
   switch (myqual) {
   case Angle::TRU:
-    retbuf += " True";
+    buf += " True";
     break;
   default:
     ;
   }
-  retbuf.trim();
-  return retbuf;
+  buf.trim();
 }
 
 Angle &
@@ -122,8 +122,8 @@ Angle::operator*= (double a) {
   return (*this);
 }
 
-Degrees::Degrees (double v): Angle (Angle::DEGREES, v) {};
-Radians::Radians (double v): Angle (Angle::RADIANS, v) {};
+Degrees::Degrees (double v): Angle (Angle::DEGREES, v) {}
+Radians::Radians (double v): Angle (Angle::RADIANS, v) {}
 
 // Save for a rainy day
 // This gives the whole nine yards:  degrees, minutes, and seconds.
@@ -140,14 +140,6 @@ Radians::Radians (double v): Angle (Angle::RADIANS, v) {};
 //   return out;
 // }
 
-#if 0
-// This just gives degrees with decimals, not degrees/minutes/seconds.
-ostream &operator<< (ostream &out, Angle v) {
-  out << v.ppdeg (2);
-  return out;
-}
-#endif
-
 Angle operator+ (Angle a, Angle b) {
   return Radians (a.rad() + b.rad());
 }
