@@ -1,4 +1,4 @@
-// $Id: Colors.cc,v 1.3 2002/07/10 20:46:40 flaterco Exp $
+// $Id: Colors.cc,v 1.4 2004/04/19 20:16:55 flaterco Exp $
 /*  Colors  Manage XTide colors without X.
 
     Copyright (C) 1998  David Flater.
@@ -21,19 +21,6 @@
 #include "common.hh"
 #include "rgb.hh"
 
-char *colordesc[numcolors] = {
-  "Background color for text windows and location chooser.",
-  "Color of text and other notations.",
-  "Color of mark line in graphs and of stations in the location chooser.",
-  "Background color of buttons.",
-  "Daytime background color in tide graphs.",
-  "Nighttime background color in tide graphs.",
-  "Foreground in tide graphs during incoming tide.",
-  "Foreground in tide graphs during outgoing tide.",
-  "Color of datum line in tide graphs.",
-  "Color of Mean Tide Level line in tide graphs."
-};
-
 char *colorarg[numcolors] = {
   "bg", "fg", "mc", "bc", "dc", "nc", "fc", "ec", "Dc", "Mc"
 };
@@ -41,7 +28,8 @@ char *colorarg[numcolors] = {
 Colors::Colors (const Settings &settings) {
   int a;
   for (a=0; a<numcolors; a++)
-    parseColor (settings.colors[a], cmap[a][0], cmap[a][1], cmap[a][2]);
+    // This is not const: settings[colorarg[a]].s
+    parseColor (settings.find(colorarg[a])->second.s, cmap[a][0], cmap[a][1], cmap[a][2]);
 }
 
 int
