@@ -1,4 +1,4 @@
-// $Id: Graph.cc,v 1.5 2003/02/20 15:19:56 flaterco Exp $
+// $Id: Graph.cc,v 1.14 2004/11/15 16:30:22 flaterco Exp $
 /*  Graph  Abstract superclass for all graphs.
 
     Copyright (C) 1998  David Flater.
@@ -34,18 +34,19 @@ unsigned Graph::ysize() {
   return myysize;
 }
 
-struct Graph::tideline *Graph::findnextsunevent (struct tideline *first,
-Timestamp now, Timestamp endt, Timestamp &nextsunevent) {
-  while (first) {
-    if (first->t_out > now &&
-    (first->etype == Station::sunrise || first->etype == Station::sunset)) {
-      nextsunevent = first->t_out;
-      return first;
+void Graph::findnextsunevent (TideEventsIterator &it,
+TideEventsOrganizer &organizer, Timestamp now, Timestamp endt,
+Timestamp &nextsunevent) {
+  while (it != organizer.end()) {
+    TideEvent &te = it->second;
+    if (te.tm > now &&
+    (te.etype == TideEvent::sunrise || te.etype == TideEvent::sunset)) {
+      nextsunevent = te.tm;
+      return;
     }
-    first = first->next;
+    it++;
   }
   nextsunevent = endt + Interval(DAYSECONDS);
-  return NULL;
 }
 
 double Graph::linterp (double lo, double hi, double saturation) {
@@ -100,6 +101,7 @@ unsigned linestep, int &mindepth, int &maxdepth, unsigned labelwidth) {
 void Graph::drawFunkyLine (double prevytide, double ytide, double nextytide,
 Settings *settings, int x, Colors::colorchoice c) {
   double dy, yleft, yright;
+  double slw = (*settings)["lw"].d;
 
   // The fix for line slope breaks down when the slope gets nasty,
   // so switch to a more conservative strategy when that happens.
@@ -108,9 +110,9 @@ Settings *settings, int x, Colors::colorchoice c) {
 #define dohalfline(yy) {                                          \
   double lw;                                                      \
   if (fabs(dy) < slopelimit)                                      \
-    lw = (1.0 + (M_SQRT2 - 1.0) * fabs(dy)) * settings->lw / 2.0; \
+    lw = (1.0 + (M_SQRT2 - 1.0) * fabs(dy)) * slw / 2.0; \
   else                                                            \
-    lw = (fabs(dy) + settings->lw) / 2.0;                         \
+    lw = (fabs(dy) + slw) / 2.0;                         \
   if (dy < 0.0)                                                   \
     lw = -lw;                                                     \
   yy = ytide - lw;                                                \
@@ -124,41 +126,41 @@ Settings *settings, int x, Colors::colorchoice c) {
   // Fix degenerate cases.
   if (ytide > yleft && ytide > yright) {
     if (yleft > yright)
-      yleft = ytide + settings->lw / 2.0;
+      yleft = ytide + slw / 2.0;
     else
-      yright = ytide + settings->lw / 2.0;
+      yright = ytide + slw / 2.0;
   } else if (ytide < yleft && ytide < yright) {
     if (yleft < yright)
-      yleft = ytide - settings->lw / 2.0;
+      yleft = ytide - slw / 2.0;
     else
-      yright = ytide - settings->lw / 2.0;
+      yright = ytide - slw / 2.0;
   }
   drawVerticalLine (x, yleft, yright, c);
 }
 
 void Graph::clearGraph (Timestamp start, Timestamp endt, Interval increment,
-Station *sr, struct Graph::tideline *first) {
+Station *sr, TideEventsOrganizer &organizer) {
   assert (sr);
-  Settings *settings = sr->context->settings;
+  char ns = (*(sr->context->settings))["ns"].c;
   // Clear the graph by laying down a background of days and nights.
   int sunisup = 1;
-  if (!(sr->coordinates.isNull()) && settings->ns == "n")
+  if (!(sr->coordinates.isNull()) && ns == 'n')
     sunisup = sun_is_up (start, sr->coordinates);
-  int x;
-  struct tideline *t;
+  unsigned x;
   Timestamp loopt (start);
   Timestamp nextsunevent;
-  findnextsunevent (first, loopt, endt, nextsunevent);
+  TideEventsIterator it = organizer.begin();
+  findnextsunevent (it, organizer, loopt, endt, nextsunevent);
   for (x=0; x<myxsize; x++, loopt += increment) {
-    while (loopt >= nextsunevent) {
-      t = findnextsunevent (first, nextsunevent,
-        endt, nextsunevent);
-      if (t) {
-        switch (t->etype) {
-        case Station::sunrise:
+    if (loopt >= nextsunevent) {
+      findnextsunevent (it, organizer, loopt, endt, nextsunevent);
+      assert (loopt < nextsunevent);
+      if (it != organizer.end()) {
+        switch (it->second.etype) {
+        case TideEvent::sunrise:
           sunisup = 0;
           break;
-        case Station::sunset:
+        case TideEvent::sunset:
           sunisup = 1;
           break;
         default:
@@ -181,65 +183,95 @@ void Graph::drawTides (Station *sr, Timestamp now, Angle *angle) {
   assert (sr);
   Settings *settings = sr->context->settings;
   assert (sr->aspect > 0.0);
+  char tl = (*settings)["tl"].c;
+  char nf = (*settings)["nf"].c;
+  char el = (*settings)["el"].c;
 
   Interval increment = (max (1, (long)(aspmagnum / (double)myysize /
     (aspectfudge() * sr->aspect) + 0.5)));
   Timestamp start = now - increment * startpos();
-
   Timestamp endt = start + increment * xsize(); // end is a keyword
-  struct tideline *t;
 
   // now is the "now" of the graph (the time to show).
   // rightnow is the real "now" (the actual wallclock time).
+  // On tide clocks they ought to be identical.
   Timestamp rightnow ((time_t)(time(NULL)));
 
-  // First get a list of the relevant tide events.  We need to go back
-  // to synchronize internal and real timestamps.  We'll also draw one
-  // extra timestamp on either side, and make sure to stack up at least
-  // one max event and one min for clock mode to use.
-  // Furthermore, we make sure to get a previous event that is a max or
-  // min event for the sake of the tide clock icon (see kludge below).
-  struct tideline *first = new tideline;
-  first->tm = start;
-  do
-    sr->predictExactTideEvent (first->tm, Station::backward,
-      first->t_out, first->etype, first->etype_desc_long,
-      first->etype_desc_short, first->pv);
-  while (first->t_out >= start ||
-        ((angle) && (!(isMaxMinEvent (first->etype)))));
-  struct tideline *prev = first;
-  nextmax = nextmin = NULL;
-  while (prev->t_out <= endt || (!nextmax) || (!nextmin)) {
-    struct tideline *next = new tideline;
-    next->tm = prev->tm;
-    sr->predictExactTideEvent (next->tm, Station::forward,
-      next->t_out, next->etype, next->etype_desc_long,
-      next->etype_desc_short, next->pv);
-    if (next->etype == prev->etype)
-      sr->predictExactTideEvent (next->tm, Station::forward,
-        next->t_out, next->etype, next->etype_desc_long,
-        next->etype_desc_short, next->pv);
-    // Stack up for clock mode.
-    if (next->t_out > now) {
-      switch (next->etype) {
-      case Station::max:
-        if (!nextmax)
-          nextmax = next;
-        break;
-      case Station::min:
-        if (!nextmin)
-          nextmin = next;
-        break;
-      default:
-        ;
+  // First get a list of the relevant tide events.  Need some extra on
+  // either side since text pertaining to events occurring beyond the
+  // margins can still be visible.  We also need to make sure
+  // *something* shows up so that extendRange can work below.
+
+  TideEventsOrganizer organizer;
+  Interval delta;
+  for (delta = Interval(DAYSECONDS); organizer.empty(); delta *= 2U)
+    sr->predictTideEvents (start - delta, endt + delta, organizer);
+
+  // Need at least one following max and min for clock mode.
+  TideEvent nextmax, nextmin;
+  if (clockmode) {
+    int donemax = 0, donemin=0;
+    delta = Interval(DAYSECONDS);
+    while (!(donemax && donemin)) {
+      TideEventsIterator it = organizer.upper_bound(rightnow.timet());
+      while (it != organizer.end()) {
+        TideEvent &te = it->second;
+        if (!donemax && te.etype == TideEvent::max) {
+          donemax = 1;
+          nextmax = te;
+        } else if (!donemin && te.etype == TideEvent::min) {
+          donemin = 1;
+          nextmin = te;
+        }
+        it++;
+      }
+      if (!(donemax && donemin)) {
+        sr->extendRange (organizer, forward, delta);
+        delta *= 2U;
       }
     }
-    prev->next = next;
-    prev = next;
-  }
-  prev->next = NULL;
 
-  clearGraph (start, endt, increment, sr, first);
+    // Need a max/min pair bracketing current time for tide clock icon
+    // angle kludge.
+    if (angle) {
+      TideEvent nextmaxormin;
+      if (nextmax.tm < nextmin.tm)
+	nextmaxormin = nextmax;
+      else
+	nextmaxormin = nextmin;
+      TideEvent prevmaxormin;
+      {
+	int done = 0;
+        delta = Interval(DAYSECONDS);
+	while (!done) {
+	  TideEventsIterator it = organizer.upper_bound(rightnow.timet());
+	  assert (it != organizer.end());
+	  while (it != organizer.begin())
+	    if ((--it)->second.isMaxMinEvent()) {
+	      done = 1;
+	      prevmaxormin = it->second;
+	      break;
+	    }
+	  if (!done) {
+	    sr->extendRange (organizer, backward, delta);
+            delta *= 2U;
+          }
+	}
+      }
+      assert (prevmaxormin.etype != nextmaxormin.etype);
+      assert (prevmaxormin.tm <= rightnow && nextmaxormin.tm > rightnow);
+      assert (prevmaxormin.isMaxMinEvent());
+      assert (nextmaxormin.isMaxMinEvent());
+      double temp = (rightnow - prevmaxormin.tm) /
+                    (nextmaxormin.tm - prevmaxormin.tm);
+      temp *= 180.0;
+      if (prevmaxormin.etype == TideEvent::min)
+        temp += 180.0;
+      (*angle) = Angle (Angle::DEGREES, temp);
+    }
+  }
+
+  clearGraph (start, endt, increment, sr, organizer);
 
   int x;
   double ytide;
@@ -286,36 +318,36 @@ void Graph::drawTides (Station *sr, Timestamp now, Angle *angle) {
     depthLabelRightMargin();
 
   // Draw depth lines now?
-  if (settings->tl == "n")
+  if (tl == 'n')
     drawDepth (ymax, ymin, valmax, valmin, linestep, mindepth, maxdepth,
     labelwidth);
 
   // Draw the actual tides.
   double prevval, prevytide;
-  double val = sr->predictApproximate(start-increment).val();
+  double val = sr->predictTideLevel(start-increment).val();
   ytide = xlate(val);
-  double nextval = sr->predictApproximate(start).val();
+  double nextval = sr->predictTideLevel(start).val();
   double nextytide = xlate (nextval);
   // loopt is actually 1 step ahead of x
-  for (x=0, loopt=start+increment; x<myxsize; x++, loopt += increment) {
+  for (x=0, loopt=start+increment; x<(int)myxsize; x++, loopt += increment) {
     prevval = val;
     prevytide = ytide;
     val = nextval;
     ytide = nextytide;
-    nextval = sr->predictApproximate(loopt).val();
+    nextval = sr->predictTideLevel(loopt).val();
     nextytide = xlate(nextval);
 
     if (sr->isCurrent) {
       Colors::colorchoice c = (val > 0.0 ?
         Colors::flood : Colors::ebb);
-      if (settings->nf == "n")
+      if (nf == 'n')
         drawVerticalLine (x, midwl, ytide, c);
       else
         drawFunkyLine (prevytide, ytide, nextytide, settings, x, c);
     } else {
       Colors::colorchoice c = (prevval < val ?
         Colors::flood : Colors::ebb);
-      if (settings->nf == "n")
+      if (nf == 'n')
         drawVerticalLine (x, myysize, ytide, c);
       else
         drawFunkyLine (prevytide, ytide, nextytide, settings, x, c);
@@ -323,7 +355,7 @@ void Graph::drawTides (Station *sr, Timestamp now, Angle *angle) {
   }
 
   // Draw depth lines later?
-  if (settings->tl != "n")
+  if (tl != 'n')
     drawDepth (ymax, ymin, valmax, valmin, linestep, mindepth, maxdepth,
     labelwidth);
 
@@ -388,13 +420,15 @@ void Graph::drawTides (Station *sr, Timestamp now, Angle *angle) {
     centerStringOnLine (myxsize/2, 0, ts);
 
     // Write next max
-    centerStringOnLine (myxsize/2, 1, nextmax->etype_desc_long);
-    nextmax->t_out.printtime (ts, sr->timeZone, settings);
+    Dstr desc (nextmax.longdesc());
+    centerStringOnLine (myxsize/2, 1, desc);
+    nextmax.tm.printtime (ts, sr->timeZone, settings);
     centerStringOnLine (myxsize/2, 2, ts);
 
     // Write next min
-    centerStringOnLine (myxsize/2, -3, nextmin->etype_desc_long);
-    nextmin->t_out.printtime (ts, sr->timeZone, settings);
+    desc = nextmin.longdesc();
+    centerStringOnLine (myxsize/2, -3, desc);
+    nextmin.tm.printtime (ts, sr->timeZone, settings);
     centerStringOnLine (myxsize/2, -2, ts);
 
   } else {
@@ -402,51 +436,55 @@ void Graph::drawTides (Station *sr, Timestamp now, Angle *angle) {
     drawTitleLine (sr->name);
 
     // Put timestamps for timestampable events.
-    t = first;
-    while (t) {
+    TideEventsIterator it = organizer.begin();
+    while (it != organizer.end()) {
+      TideEvent &te = it->second;
       Dstr ts;
-      x = (int)((t->t_out - start) / increment + 0.5);
-      switch (t->etype) {
-      case Station::max:
-      case Station::min:
-	t->t_out.printdate (ts, sr->timeZone, settings);
+      x = (int)((te.tm - start) / increment + 0.5);
+      switch (te.etype) {
+      case TideEvent::max:
+      case TideEvent::min:
+	te.tm.printdate (ts, sr->timeZone, settings);
 	drawTimestampLabel (x, 1, ts);
-	t->t_out.printtime (ts, sr->timeZone, settings);
+	te.tm.printtime (ts, sr->timeZone, settings);
 	drawTimestampLabel (x, 2, ts);
 	break;
 
-      case Station::moonrise:
-      case Station::moonset:
-      case Station::slackrise:
-      case Station::slackfall:
-      case Station::markrise:
-      case Station::markfall:
-      case Station::newmoon:
-      case Station::firstquarter:
-      case Station::fullmoon:
-      case Station::lastquarter:
+      case TideEvent::moonrise:
+      case TideEvent::moonset:
+      case TideEvent::slackrise:
+      case TideEvent::slackfall:
+      case TideEvent::markrise:
+      case TideEvent::markfall:
+      case TideEvent::newmoon:
+      case TideEvent::firstquarter:
+      case TideEvent::fullmoon:
+      case TideEvent::lastquarter:
         drawHourTick (x, Colors::mark);
-	t->t_out.printtime (ts, sr->timeZone, settings);
+	te.tm.printtime (ts, sr->timeZone, settings);
 	drawTimestampLabel (x, -2, ts);
 
         // Pre-2.6:  only draw mark/slack crossings at the bottom;
         // include date.
-	// t->t_out.printdate (ts, sr->timeZone, settings);
+	// te.tm.printdate (ts, sr->timeZone, settings);
 	// drawTimestampLabel (x, -3, ts);
 
         // 2.6:  replace the date with the description and draw more
         // sun/moon events.  Try to be smart about when to use long
         // descriptions or short.
-        if (!is_banner() && (sr->isCurrent || sr->markLevel))
-	  drawTimestampLabel (x, -3, t->etype_desc_short);
-        else
-	  drawTimestampLabel (x, -3, t->etype_desc_long);
-
+        {
+          Dstr desc;
+          if (!is_banner() && (sr->isCurrent || sr->markLevel))
+            desc = te.shortdesc();
+          else
+	    desc = te.longdesc();
+	  drawTimestampLabel (x, -3, desc);
+        }
 	break;
       default:
 	;
       }
-      t = t->next;
+      it++;
     }
   }
 
@@ -455,7 +493,7 @@ void Graph::drawTides (Station *sr, Timestamp now, Angle *angle) {
     ytide = xlate(sr->markLevel->val());
     drawHorizontalLine (labelwidth, myxsize-1, ytide, Colors::mark);
   }
-  if (settings->el != "n") {
+  if (el != 'n') {
     drawHorizontalLine (labelwidth, myxsize-1, midwl, Colors::datum);
     ytide = (ymax + ymin) / 2.0;
     drawHorizontalLine (labelwidth, myxsize-1, ytide, Colors::msl);
@@ -464,43 +502,8 @@ void Graph::drawTides (Station *sr, Timestamp now, Angle *angle) {
   // X marks the current time.
   if (rightnow >= start && rightnow < endt) {
     x = (int)((rightnow - start) / increment + 0.5);
-    ytide = xlate(sr->predictApproximate(rightnow).val());
+    ytide = xlate(sr->predictTideLevel(rightnow).val());
     drawX (x, ytide);
-
-    if (angle) {
-      // Kludge for tide clock -- estimate angle of hand on analog tide
-      // clock.
-      t = first;
-      while (!(isMaxMinEvent (t->etype))) {
-	t = t->next;
-	assert (t);
-      }
-      struct tideline *prev = NULL;
-      while (t->t_out <= rightnow) {
-        prev = t;
-        t = t->next;
-        while (!(isMaxMinEvent (t->etype))) {
-          t = t->next;
-          assert (t);
-        }
-        assert (t);
-      }
-      assert (prev);
-      assert (prev->t_out <= rightnow && t->t_out > rightnow &&
-              isMaxMinEvent (prev->etype) && isMaxMinEvent (t->etype));
-      double temp = (rightnow - prev->t_out) / (t->t_out - prev->t_out);
-      temp *= 180.0;
-      if (prev->etype == Station::min)
-        temp += 180.0;
-      (*angle) = Angle (Angle::DEGREES, temp);
-    }
-  }
-
-  // Free up temp storage.
-  while (first) {
-    t = first->next;
-    free (first);
-    first = t;
   }
 }
 
