@@ -1,5 +1,5 @@
+// $Id: HarmonicsPath.cc,v 1.2 2002/10/04 13:44:00 flaterco Exp $
 /*  HarmonicsPath  Storage of harmonics file names
-    Last modified 1997-08-17
 
     Copyright (C) 1997  David Flater.
 
@@ -40,6 +40,7 @@ Dstr &HarmonicsPath::operator[] (unsigned index) {
   return storage[index];
 }
 
+#if 0
 ostream &operator<< (ostream &out, HarmonicsPath h) {
   out << "Contents of HarmonicsPath:" << endl;
   unsigned i;
@@ -47,3 +48,4 @@ ostream &operator<< (ostream &out, HarmonicsPath h) {
     out << "  " << h[i] << endl;
   return out;
 }
+#endif
