@@ -1,4 +1,4 @@
-// $Id: Interval.cc,v 1.3 2002/12/19 18:40:45 flaterco Exp $
+// $Id: Interval.cc,v 1.4 2004/10/21 19:58:13 flaterco Exp $
 // Interval:  what you get if you subtract two timestamps.
 
 /*
@@ -123,12 +123,16 @@ Interval operator- (Interval a, Interval b) {
   return Interval (a.in_seconds() - b.in_seconds());
 }
 
-void
-Interval::operator-= (Interval a) {
+void Interval::operator-= (Interval a) {
   // Potential overflow/underflow
   seconds -= a.in_seconds();
 }
 
+void Interval::operator*= (unsigned a) {
+  // Potential overflow
+  seconds *= a;
+}
+
 #if 0
 // This is not used anymore so it's not worth fixing the overflow.
 void
