@@ -1,4 +1,4 @@
-// $Id: Offsets.cc,v 1.2 2002/10/04 13:44:00 flaterco Exp $
+// $Id: Offsets.cc,v 1.5 2004/11/16 22:27:29 flaterco Exp $
 // Offsets:  storage for tide offsets.
 
 /*
@@ -21,6 +21,10 @@
 
 #include "common.hh"
 
+Offsets::~Offsets() {
+  // This space intentionally left blank.
+}
+
 void LevelOffset::levelMultiply (double in_mult) {
   assert (in_mult > 0.0);
   mylevelMultiply = in_mult;
@@ -28,7 +32,7 @@ void LevelOffset::levelMultiply (double in_mult) {
 
 double LevelOffset::levelMultiply () const {
   return mylevelMultiply;
-};
+}
 
 LevelOffset::LevelOffset () {
   mylevelMultiply = 1.0;
@@ -46,7 +50,7 @@ HairyOffsets::is_simple() {
 
 SimpleOffsets::SimpleOffsets () {
   mylevelMultiply = 1.0;
-};
+}
 
 SimpleOffsets::SimpleOffsets (const CombinedOffset &co_in) {
   levelAdd = co_in.levelAdd;
@@ -62,37 +66,11 @@ int operator== (CombinedOffset a, CombinedOffset b) {
   return 0;
 }
 
-#if 0
-ostream &operator<< (ostream &out, Offsets *o) {
-  if (!o) {
-    out << "{null offsets}";
-  } else {
-    if (o->is_simple()) {
-      SimpleOffsets *s = (SimpleOffsets *)o;
-      out << "{" << s->timeAdd << "," << s->levelMultiply()
-	   << "," << s->levelAdd << "}";
-    } else {
-      HairyOffsets *h = (HairyOffsets *)o;
-      out << "{max";
-      out << "{" << h->max.timeAdd << "," << h->max.levelMultiply()
-	   << "," << h->max.levelAdd << "}";
-      out << ",min";
-      out << "{" << h->min.timeAdd << "," << h->min.levelMultiply()
-	   << "," << h->min.levelAdd << "}";
-      out << ",flood " << h->floodbegins.timeAdd;
-      out << ", ebb " << h->ebbbegins.timeAdd;
-      out << "}";
-    }
-  }
-  return out;
-}
-#endif
-
 HairyOffsets::HairyOffsets () {
   // This space intentionally left blank.
 }
 
 HairyOffsets::HairyOffsets (const SimpleOffsets &s_in) {
   max = min = s_in;
-  floodbegins.timeAdd = ebbbegins.timeAdd = s_in.timeAdd;
+  floodbegins = ebbbegins = s_in.timeAdd;
 }
