@@ -1,4 +1,4 @@
-// $Id: PredictionValue.cc,v 1.2 2002/10/04 13:44:00 flaterco Exp $
+// $Id: PredictionValue.cc,v 1.6 2004/11/15 16:30:22 flaterco Exp $
 /*  PredictionValue  feet, meters, knots, et cetera.
 
     Copyright (C) 1998  David Flater.
@@ -45,11 +45,11 @@ PredictionValue::Unit::Unit (Units units_in) {
   mytype = units_in;
 }
 
-void PredictionValue::Unit::shortname (Dstr &name_out) {
+void PredictionValue::Unit::shortname (Dstr &name_out) const {
   name_out = shortnames[mytype];
 }
 
-void PredictionValue::Unit::longname (Dstr &name_out) {
+void PredictionValue::Unit::longname (Dstr &name_out) const {
   name_out = longnames[mytype];
 }
 
@@ -91,47 +91,53 @@ void PredictionValue::Units (PredictionValue::Unit::Units tounits) {
 }
 
 void PredictionValue::Units (Unit tounits) {
-  if (myvalue != 0.0) {
-    switch (myunits.mytype) {
-    case Unit::Feet:
-      if (tounits.mytype == Unit::Meters)
-	myvalue *= 0.3048;
-      else
-	convbarf;
-      break;
-    case Unit::Meters:
-      if (tounits.mytype == Unit::Feet)
-	myvalue /= 0.3048;
-      else
-	convbarf;
-      break;
-    case Unit::KnotsSquared:
-      if (tounits.mytype == Unit::Knots) {
-	// This is not mathematically correct, but it is tidematically correct.
-	if (myvalue < 0)
-	  myvalue = -sqrt(fabs(myvalue));
+  if (myunits == tounits)
+    barf (NO_CONVERSION, 0);
+  else {
+    // You are allowed to convert nulls from meters to knots because
+    // otherwise it just complicates initialization.
+    if (myvalue != 0.0) {
+      switch (myunits.mytype) {
+      case Unit::Feet:
+	if (tounits.mytype == Unit::Meters)
+	  myvalue *= 0.3048;
 	else
-	  myvalue = sqrt(myvalue);
-      } else
-	convbarf;
-      break;
-    case Unit::Knots:
-      if (tounits.mytype == Unit::KnotsSquared) {
-        // This is used only in Station::predictExactTideEvent to set up
-        // the mark level.
-	// This is not mathematically correct, but it is tidematically correct.
-	if (myvalue < 0)
-	  myvalue = -(myvalue*myvalue);
+	  convbarf;
+	break;
+      case Unit::Meters:
+	if (tounits.mytype == Unit::Feet)
+	  myvalue /= 0.3048;
 	else
-	  myvalue *= myvalue;
-      } else
-        convbarf;
-      break;
-    default:
-      convbarf;
+	  convbarf;
+	break;
+      case Unit::KnotsSquared:
+	if (tounits.mytype == Unit::Knots) {
+	  // This is not mathematically correct, but it is tidematically correct.
+	  if (myvalue < 0)
+	    myvalue = -sqrt(fabs(myvalue));
+	  else
+	    myvalue = sqrt(myvalue);
+	} else
+	  convbarf;
+	break;
+      case Unit::Knots:
+	if (tounits.mytype == Unit::KnotsSquared) {
+	  // This is used only in Station::predictTideEvents to set up
+	  // the mark level.
+	  // This is not mathematically correct, but it is tidematically correct.
+	  if (myvalue < 0)
+	    myvalue = -(myvalue*myvalue);
+	  else
+	    myvalue *= myvalue;
+	} else
+	  convbarf;
+	break;
+      default:
+	convbarf;
+      }
     }
+    myunits = tounits;
   }
-  myunits = tounits;
 }
 
 // This insists that both values must have exactly the same units.
@@ -285,7 +291,7 @@ int operator== (PredictionValue::Unit a, PredictionValue::Unit b) {
 }
 
 // Print in the form -XX.YY units (padding as needed)
-void PredictionValue::print (Dstr &print_out) {
+void PredictionValue::print (Dstr &print_out) const {
   char temp[80];
   Dstr unitname;
   myunits.longname (unitname);
@@ -295,7 +301,7 @@ void PredictionValue::print (Dstr &print_out) {
   print_out += unitname;
 }
 
-void PredictionValue::printnp (Dstr &print_out) {
+void PredictionValue::printnp (Dstr &print_out) const {
   char temp[80];
   Dstr unitname;
   myunits.shortname (unitname);
