@@ -1,4 +1,4 @@
-// $Id: ReferenceStation.cc,v 1.2 2003/02/04 15:53:46 flaterco Exp $
+// $Id: ReferenceStation.cc,v 1.9 2004/11/17 16:27:18 flaterco Exp $
 /*  ReferenceStation  Implementation for regular tide station.
 
     Copyright (C) 1998  David Flater.
@@ -22,6 +22,7 @@
 
 ReferenceStation::ReferenceStation (TideContext *in_tidecontext,
 ConstantSetWrapper *in_constants): Station (in_tidecontext) {
+  // Other fields are filled in in HarmonicsFile::getStation.
   assert (in_constants);
   constants = in_constants;
 }
@@ -33,20 +34,23 @@ ReferenceStation::~ReferenceStation() {
     delete markLevel;
 }
 
-// tm is "uncorrected" or "internal" timestamp
-// t_out is "corrected" timestamp (with offsets applied)
-// For reference station or simple offsets, they are identical.
-// pv_out is undefined if the event is a sun or moon event.
-void ReferenceStation::predictExactTideEvent (Timestamp &tm, Direction d,
-Timestamp &t_out, EventType &etype_out, Dstr &etype_desc_long,
-Dstr &etype_desc_short, PredictionValue &pv_out) {
-  _predictExactTideEvent (tm, d, etype_out);
-  t_out = tm;
-  if (!isSunMoonEvent (etype_out))
-    pv_out = Station::predictApproximate (tm);
-  etypedesc (etype_out, pv_out, etype_desc_long, etype_desc_short);
-}
-
 int ReferenceStation::is_reference_station() {
   return 1;
 }
+
+int ReferenceStation::have_residuals() {
+  return 0;
+}
+
+int ReferenceStation::have_floodbegins() {
+  return 1;
+}
+
+int ReferenceStation::have_ebbbegins() {
+  return 1;
+}
+
+void ReferenceStation::finishTideEvent (TideEvent &te) {
+  if (!te.isSunMoonEvent())
+    te.pv = predictTideLevel (te.tm);
+}
