@@ -1,4 +1,4 @@
-// $Id: TabulatedConstituent.cc,v 1.4 2002/12/23 18:53:56 flaterco Exp $
+// $Id: TabulatedConstituent.cc,v 1.7 2004/10/04 13:57:50 flaterco Exp $
 // TabulatedConstituent:  definition of a constituent that uses tabulated
 // equilibrium arguments and node factors.
 
@@ -30,25 +30,18 @@ TabulatedConstituent::Argfac::Argfac () {
 TabulatedConstituent::Argfac::Argfac (Angle in_arg, double in_nod) {
   arg = in_arg;
   nod = in_nod;
+  assert (in_nod > 0.0);
 }
 
 TabulatedConstituent::TabulatedConstituent () {
-  myname = "Anonymous";
-  myspeed = DegreesPerHour (0.0);
-  myargfacs = NULL;
-}
-
-TabulatedConstituent::TabulatedConstituent (Dstr &in_name) {
-  myname = in_name;
   myspeed = DegreesPerHour (0.0);
   myargfacs = NULL;
 }
 
 TabulatedConstituent &
-TabulatedConstituent::operator= (Constituent &in_con) {
+TabulatedConstituent::operator= (TabulatedConstituent &in_con) {
   if (myargfacs)
     delete [] myargfacs;
-  myname = in_con.name();
   myspeed = in_con.speed();
   myfirstvalidyear = in_con.firstvalidyear();
   mylastvalidyear = in_con.lastvalidyear();
@@ -62,13 +55,6 @@ TabulatedConstituent::operator= (Constituent &in_con) {
   return (*this);
 }
 
-// Workaround for funky overload resolution problem.
-TabulatedConstituent &
-TabulatedConstituent::operator= (TabulatedConstituent &in_con) {
-  (*this) = *((Constituent*)&in_con);  // You forth me to use forth!
-  return (*this);
-}
-
 void
 TabulatedConstituent::speed (Speed in_speed) {
   myspeed = in_speed;
@@ -126,8 +112,11 @@ TabulatedConstituent::nod (Year in_year) {
   return myargfacs[in_year.val()-myfirstvalidyear.val()].nod;
 }
 
-void TabulatedConstituent::readboth (int i, Year first_year, Year last_year)
-{
+// Install equilibrium arguments and node factors.  N.B. single-precision
+// floats on input to match libtcd; otherwise never used in XTide.
+// args are in degrees.
+void TabulatedConstituent::installargsandnodes (Year first_year,
+Year last_year, float *args, float *nodes) {
   assert (!myargfacs);
   assert (last_year >= first_year);
   nargs = last_year.val() - first_year.val() + 1;
@@ -135,9 +124,8 @@ void TabulatedConstituent::readboth (int i, Year first_year, Year last_year)
   myfirstvalidyear = first_year;
   mylastvalidyear = last_year;
   for (unsigned looper=0; looper<nargs; looper++) {
-    myargfacs[looper].arg = 
-      Degrees(get_equilibrium (i, looper));
-    myargfacs[looper].nod = get_node_factor (i, looper);
+    myargfacs[looper].arg = Degrees(args[looper]);
+    myargfacs[looper].nod = nodes[looper];
   }
 }
 
