@@ -1,4 +1,4 @@
-// $Id: Timestamp.cc,v 1.7 2003/02/20 15:19:56 flaterco Exp $
+// $Id: Timestamp.cc,v 1.13 2004/09/16 15:31:47 flaterco Exp $
 // Timestamp:  encapsulation for all manner of time-related services needed
 // by XTide.
 
@@ -196,7 +196,7 @@ Timestamp::Timestamp (double jd) {
 }
 
 // Convert to Julian date.
-double Timestamp::jd() {
+double Timestamp::jd() const {
   return ((double)posixtime / (double)DAYSECONDS) + beginning_of_time;
 }
 
@@ -222,8 +222,7 @@ Timestamp::Timestamp (Year in_year, double fraction) {
   posixtime = y1 + (time_t) (fraction * (y2 - y1));
 }
 
-int
-Timestamp::isNull() {
+int Timestamp::isNull() const {
   return is_null;
 }
 
@@ -234,7 +233,7 @@ Timestamp::isNull() {
 // but that became a liability when we started going outside of that
 // epoch.  So we are back to the slow but simple way.
 
-Year Timestamp::year() {
+Year Timestamp::year() const {
   assert (!is_null);
   return Year (((gmtime (&posixtime))->tm_year) + 1900);
 }
@@ -340,18 +339,6 @@ Timestamp::tm2utc (struct tm ht)
   return guess;
 }
 
-#if 0
-// Output timestamp in ISO 8601 format (UTC).
-ostream &operator<< (ostream &out, Timestamp &in_time) {
-  assert (!in_time.is_null);
-  char tempbuf[80];
-  assert (strftime (tempbuf, 79, "%Y-%m-%d %TZ", gmtime(&(in_time.posixtime)))
-    < 79);
-  out << tempbuf;
-  return out;
-}
-#endif
-
 // Subtract b from a.
 Interval operator- (Timestamp a, Timestamp b) {
   assert (!a.is_null);
@@ -430,21 +417,39 @@ void Timestamp::make_null() {
   is_null = 1;
 }
 
-// Output timestamp in local time zone, no seconds
-void Timestamp::print (Dstr &out_time, const Dstr &timezone, Settings *settings) {
+// Output timestamp in format complying with RFC 2445 (iCalendar)
+// if zero_secs <> 0:  YYYYMMDDTHHMM00Z
+//              else:  YYYYMMDDTHHMMSSZ
+//
+// RFC 2445 p. 140, Recommended Practices:
+// 7.  If seconds of the minute are not supported by an implementation,
+//     then a value of "00" SHOULD be specified for the seconds
+//     component in a time value.
+// We have seconds, but they aren't significant.
+void Timestamp::print_iCalendar (Dstr &out_time, int zero_secs) const {
   char tempbuf[80];
-  Dstr fmt (settings->df);
+  if (zero_secs)
+    assert (strftime (tempbuf, 79, "%Y%m%dT%H%M00Z", gmtime(&posixtime)) < 79);
+  else
+    assert (strftime (tempbuf, 79, "%Y%m%dT%H%M%SZ", gmtime(&posixtime)) < 79);
+  tempbuf[79] = '\0';
+  out_time = tempbuf;
+}
+
+// Output timestamp in local time zone, no seconds
+void Timestamp::print (Dstr &out_time, const Dstr &timezone, Settings *settings) const {
+  char tempbuf[80];
+  Dstr fmt ((*settings)["df"].s);
   fmt += " ";
-  fmt += settings->tf;
+  fmt += (*settings)["tf"].s;
   assert (strftime (tempbuf, 79, fmt.aschar(), get_tm(timezone,settings)) < 79);
   tempbuf[79] = '\0';
   out_time = tempbuf;
 }
 
-void
-Timestamp::printhour (Dstr &out_time, const Dstr &timezone, Settings *settings) {
+void Timestamp::printhour (Dstr &out_time, const Dstr &timezone, Settings *settings) const {
   char tempbuf[80];
-  assert (strftime (tempbuf, 79, settings->hf.aschar(), get_tm(timezone,settings)) < 79);
+  assert (strftime (tempbuf, 79, (*settings)["hf"].s.aschar(), get_tm(timezone,settings)) < 79);
   tempbuf[79] = '\0';
   out_time = tempbuf;
   // Kludge to get rid of leading space on hours
@@ -452,27 +457,27 @@ Timestamp::printhour (Dstr &out_time, const Dstr &timezone, Settings *settings)
     out_time /= 1;
 }
 
-struct tm *Timestamp::get_tm (const Dstr &timezone, Settings *settings) {
+struct tm *Timestamp::get_tm (const Dstr &timezone, Settings *settings) const {
   assert (!is_null);
   install_time_zone (timezone, settings);
   return localtime(&posixtime);
 }
 
 unsigned
-Timestamp::gethour (const Dstr &timezone, Settings *settings) {
+Timestamp::gethour (const Dstr &timezone, Settings *settings) const {
   return (unsigned)((get_tm(timezone,settings))->tm_hour);
 }
 
-void Timestamp::printdate (Dstr &out_time, const Dstr &timezone, Settings *settings) {
+void Timestamp::printdate (Dstr &out_time, const Dstr &timezone, Settings *settings) const {
   char tempbuf[80];
-  assert (strftime (tempbuf, 79, settings->df.aschar(), get_tm(timezone,settings)) < 79);
+  assert (strftime (tempbuf, 79, (*settings)["df"].s.aschar(), get_tm(timezone,settings)) < 79);
   tempbuf[79] = '\0';
   out_time = tempbuf;
 }
 
-void Timestamp::printtime (Dstr &out_time, const Dstr &timezone, Settings *settings) {
+void Timestamp::printtime (Dstr &out_time, const Dstr &timezone, Settings *settings) const {
   char tempbuf[80];
-  assert (strftime (tempbuf, 79, settings->tf.aschar(), get_tm(timezone,settings)) < 79);
+  assert (strftime (tempbuf, 79, (*settings)["tf"].s.aschar(), get_tm(timezone,settings)) < 79);
   tempbuf[79] = '\0';
   out_time = tempbuf;
   // Kludge to get rid of leading space on hours
@@ -481,7 +486,7 @@ void Timestamp::printtime (Dstr &out_time, const Dstr &timezone, Settings *setti
 }
 
 void Timestamp::printcalheading (Dstr &out_heading, const Dstr &timezone,
-				 Settings *settings) {
+				 Settings *settings) const {
   char tempbuf[80];
   assert (strftime (tempbuf, 79, "%B %Y", get_tm(timezone,settings)) < 79);
   tempbuf[79] = '\0';
@@ -489,7 +494,7 @@ void Timestamp::printcalheading (Dstr &out_heading, const Dstr &timezone,
 }
 
 void Timestamp::printdayheading (Dstr &out_heading, const Dstr &timezone,
-				 Settings *settings) {
+				 Settings *settings) const {
   char tempbuf[80];
   assert (strftime (tempbuf, 79, "%a %d", get_tm(timezone,settings)) < 79);
   tempbuf[79] = '\0';
@@ -530,7 +535,7 @@ Settings *settings) {
 
 #ifdef TIME_WORKAROUND
 
-void Timestamp::install_time_zone (const Dstr &timeZone_in, Settings *settings)
+void Timestamp::install_time_zone (const Dstr &timeZone_in, Settings *settings) const
 {
   // The only use of time zones is by strftime for %Z
   static char env_string[80];
@@ -583,6 +588,10 @@ static tz_support tzlevel = TZNULL;
 
    It's unfortunate that these zones will be broken on platforms
    having intermediate versions of the "new" zoneinfo.
+
+   2004-09-16
+   Added :America/Juneau, :America/Nome, :America/Yakutat and
+   :Pacific/Johnston.
 */
 
 /* In many cases, these substitutions will either break DST adjustments
@@ -679,6 +688,7 @@ static char *subs[][4] = {
 {":America/Hermosillo",         "MST7", "MST7", "MST7"},
 {":America/Iqaluit",            ":Canada/Eastern", "EST5EDT", "EST5EDT"},
 {":America/Jamaica",            ":Jamaica", "EST5EDT", "EST5EDT"},
+{":America/Juneau",             ":US/Alaska", "AKST9AKDT", "YST9YDT"},
 {":America/La_Paz",             "AST4", "AST4", "AST4"},
 {":America/Lima",               "EST5", "EST5", "EST5"},
 {":America/Los_Angeles",        ":US/Pacific", "PST8PDT", "PST8PDT"},
@@ -694,6 +704,7 @@ static char *subs[][4] = {
 {":America/Montserrat",         "AST4", "AST4", "AST4"},
 {":America/Nassau",             ":US/Eastern", "EST5EDT", "EST5EDT"},
 {":America/New_York",           ":US/Eastern", "EST5EDT", "EST5EDT"},
+{":America/Nome",               ":US/Alaska", "AKST9AKDT", "YST9YDT"},
 {":America/Noronha",            ":Brazil/DeNoronha", "FST2", "FST2"},
 {":America/Panama",             "EST5", "EST5", "EST5"},
 {":America/Paramaribo",         "EST3", "EST3", "EST3"},
@@ -718,6 +729,7 @@ static char *subs[][4] = {
 {":America/Vancouver",          ":Canada/Pacific", "PST8PDT", "PST8PDT"},
 {":America/Whitehorse",         ":Canada/Yukon", "PST8PDT", "PST8PDT"},
 {":America/Winnipeg",           ":Canada/Central", "CST6CDT", "CST6CDT"},
+{":America/Yakutat",            ":US/Alaska", "AKST9AKDT", "YST9YDT"},
 
 {":Antarctica/Casey",           "WST-8", "WST-8", "WST-8"},
 {":Antarctica/Davis",           "DAVT-7", "DAVT-7", "DAVT-7"},
@@ -931,6 +943,7 @@ static char *subs[][4] = {
 {":Pacific/Guadalcanal",        "NCST-11", "NCST-11", "NST-11"},
 {":Pacific/Guam",               "GST-10", "GST-10", "GST-10"},
 {":Pacific/Honolulu",           ":US/Hawaii", "HST10", "HST10"},
+{":Pacific/Johnston",           ":US/Hawaii", "HST10", "HST10"},
 {":Pacific/Kiritimati",         "KRT-14", "KRT-14", "KRT-14"},
 {":Pacific/Kosrae",             "NZST-12", "NZST-12", "NST-12"},
 {":Pacific/Kwajalein",          "NZST-12", "NZST-12", "NST-12"},
@@ -1029,10 +1042,10 @@ static char *subs[][4] = {
 static Dstr current_time_zone;
 
 // Unfortunately, time zones are a session default.
-void Timestamp::install_time_zone (const Dstr &timeZone_in, Settings *settings)
+void Timestamp::install_time_zone (const Dstr &timeZone_in, Settings *settings) const
 {
   Dstr timeZone ("UTC0");
-  if (settings->z == "n")
+  if ((*settings)["z"].c == 'n')
     timeZone = timeZone_in;
   if (current_time_zone == timeZone)
     return;
@@ -1112,8 +1125,7 @@ Savings Time) adjustments will not be done for some locations.", LOG_WARNING);
   tzset ();
 }
 
-int
-Timestamp::zoneinfo_doesnt_suck(Settings *settings) {
+int Timestamp::zoneinfo_doesnt_suck(Settings *settings) {
   if (tzlevel == TZNULL) {
     Dstr temptz ("UTC0");
     install_time_zone (temptz, settings);
@@ -1260,7 +1272,15 @@ Timestamp::inc_day (const Dstr &timezone, Settings *settings) {
   /* That will work, even though we are not starting on a day transition. */
 }
 
-time_t
-Timestamp::timet() {
+time_t Timestamp::timet() const {
   return posixtime;
 }
+
+// The moonrise and moonset logic blows up if you go before 1900 or
+// after 2099.  This is just a range check for that.
+int Timestamp::moonriseset_inrange() const {
+  double danger_y = 1900. + (jd() - 2415019.5) / 365.25;
+  if (danger_y > 1900.1 && danger_y < 2099.9)
+    return 1;
+  return 0;
+}
