@@ -1,4 +1,4 @@
-// 	$Id: ZoneIndex.cc,v 1.2 2002/10/04 13:44:00 flaterco Exp $	
+// 	$Id: ZoneIndex.cc,v 1.4 2004/10/26 15:54:53 flaterco Exp $	
 /*  ZoneIndex  Index stations by zone for xttpd.
 
     Copyright (C) 1998  David Flater.
@@ -141,35 +141,8 @@ ZoneIndex::dnode *ZoneIndex::makezone (const Dstr &zone) {
 void ZoneIndex::add (StationIndex *si, unsigned long i) {
   assert (si);
   StationRef *sr = (*si)[i];
-  Dstr zone = sr->timezone;
-  if (zone.isNull()) {
-    if (sr->is_reference_station())
-      return;
-    SubordinateStationRef *ssr = (SubordinateStationRef *)sr;
-    StationRef *rsr = si->reffirstlookup (ssr->source);
-    if (!rsr) {
-      Dstr details ("Could not find reference station ");
-      details += ssr->source;
-      details += "\nneeded by subordinate station ";
-      details += sr->name;
-      details += ".";
-      barf (STATION_NOT_FOUND, details);
-    }
-    if (!(rsr->is_reference_station())) {
-      Dstr details ("The subordinate station ");
-      details += sr->name;
-      details +="\ndepends on ";
-      details += ssr->source;
-      details += "\nwhich matched the subordinate station named ";
-      details += rsr->name;
-      details += ".";
-      barf (SUB_SUBORDINATE, details);
-    }
-    zone = rsr->timezone;
-    if (zone.isNull())
-      return;
-  }
-
+  Dstr &zone = sr->timezone;
+  assert (!zone.isNull());
   dnode *tryit = (*this)[zone];
   if (!tryit)
     tryit = makezone (zone);
