@@ -1,4 +1,4 @@
-/* $Id: tide_db_default.h,v 1.3 2002/10/01 18:37:26 flaterco Exp $ */
+/* $Id: tide_db_default.h,v 1.6 2004/10/14 21:31:08 flaterco Exp $ */
 
 #include "tcd.h"
 
@@ -19,19 +19,18 @@
 
 #define DEFAULT_HEADER_SIZE                         4096
 #define DEFAULT_NUMBER_OF_RECORDS                   0
-#define DEFAULT_NUMBER_OF_OBSOLETE_RECORDS          0
 #define DEFAULT_LEVEL_UNIT_TYPES                    5
 #define DEFAULT_DIR_UNIT_TYPES                      3
 #define DEFAULT_RESTRICTION_TYPES                   2
 #define DEFAULT_RESTRICTION_BITS                    4
-#define DEFAULT_PEDIGREE_TYPES                      13
-#define DEFAULT_PEDIGREE_BITS                       7
 #define DEFAULT_TZFILES                             406
 #define DEFAULT_TZFILE_BITS                         10
 #define DEFAULT_COUNTRIES                           240
 #define DEFAULT_COUNTRY_BITS                        9
 #define DEFAULT_DATUM_TYPES                         61
 #define DEFAULT_DATUM_BITS                          7
+#define DEFAULT_LEGALESES                           1
+#define DEFAULT_LEGALESE_BITS                       4
 #define DEFAULT_SPEED_SCALE                         10000000
 #define DEFAULT_EQUILIBRIUM_SCALE                   100
 #define DEFAULT_NODE_SCALE                          10000
@@ -44,7 +43,7 @@
 #define DEFAULT_LATITUDE_SCALE                      100000
 #define DEFAULT_LONGITUDE_BITS                      26
 #define DEFAULT_LONGITUDE_SCALE                     100000
-#define DEFAULT_RECORD_SIZE_BITS                    12
+#define DEFAULT_RECORD_SIZE_BITS                    16
 #define DEFAULT_STATION_BITS                        18
 #define DEFAULT_DATUM_OFFSET_BITS                   28
 #define DEFAULT_DATUM_OFFSET_SCALE                  10000
@@ -62,14 +61,12 @@
 #define DEFAULT_LEVEL_UNIT_SIZE                     15
 #define DEFAULT_DIR_UNIT_SIZE                       15
 #define DEFAULT_RESTRICTION_SIZE                    30
-#define DEFAULT_PEDIGREE_SIZE                       70
 #define DEFAULT_DATUM_SIZE                          70
+#define DEFAULT_LEGALESE_SIZE                       70
 #define DEFAULT_TZFILE_SIZE                         30
 #define DEFAULT_COUNTRY_SIZE                        50
 
 
-
-
 /*  Stuff for inferring constituents (NAVO short duration tide stations).  */
 
 #define INFERRED_SEMI_DIURNAL_COUNT                 10
@@ -89,7 +86,6 @@ NV_FLOAT32 diurnal_coeff[INFERRED_DIURNAL_COUNT] = {
 NV_FLOAT32 coeff[2] = {.9085, .3771};
 
 
-
 /* The following lookup tables are only used for initialization
    purposes and in the pull-down menus in TideEditor.  It should be
    possible to change them without breaking existing TCD files.  TCD
@@ -97,33 +93,28 @@ NV_FLOAT32 coeff[2] = {.9085, .3771};
 */
 
 
-/*  # Level unit names  */
+/*  Level unit names  */
 
 NV_CHAR level_unit[DEFAULT_LEVEL_UNIT_TYPES][DEFAULT_LEVEL_UNIT_SIZE] = {
   "Unknown", "feet", "meters", "knots", "knots^2"};
 
 
-/*  # Direction unit names  */
+/*  Direction unit names  */
 
 NV_CHAR dir_unit[DEFAULT_DIR_UNIT_TYPES][DEFAULT_DIR_UNIT_SIZE] = {
   "Unknown", "degrees true", "degrees"};
 
 
-/*  # Restriction types  */
+/*  Restriction types  */
 
 NV_CHAR restriction[DEFAULT_RESTRICTION_TYPES][DEFAULT_RESTRICTION_SIZE] = {
   "Public Domain", "DoD/DoD Contractors Only"};
 
 
-/*  # Pedigree types  */
+/*  Legaleses  */
 
-NV_CHAR pedigree[DEFAULT_PEDIGREE_TYPES][DEFAULT_PEDIGREE_SIZE] = {"Unknown",
-  "-2000000000 (Traceable to authority)", "-1500000000 (TD4/CR4)",
-  "-1000000000 (TID/CUR)", "-500000000 (Old NOAA/USC&GS Form 444)",
-  "0 (NOAA web site)", "250000000 (St. Johns River Water Management District)",
-  "500000000 (Harmgen)", "1000000000 (Approximated or converted data)",
-  "1125000000 (Via Chalmers)", "1250000000 (Via LaPointe)",
-  "1500000000 (Via Pieper)", "2000000000 (Hearsay)"};
+NV_CHAR legalese[DEFAULT_LEGALESES][DEFAULT_LEGALESE_SIZE] = {
+  "NULL"};
 
 
 /*  # Datum names  */
@@ -131,98 +122,98 @@ NV_CHAR pedigree[DEFAULT_PEDIGREE_TYPES][DEFAULT_PEDIGREE_SIZE] = {"Unknown",
 NV_CHAR datum[DEFAULT_DATUM_TYPES][DEFAULT_DATUM_SIZE] = {
   "Unknown", "Mean Sea Level", "Mean Low Water", "Mean Lower Low Water",
   "Mean High Water", "Mean Higher High Water", "Mean Lower High Water",
-  "Mean Higher Low Water", "Mean Low Water Springs", 
-  "Mean Lower Low Water Springs", "Mean Low Water Neaps", 
-  "Mean High Water Neaps", "Mean High Water Springs", 
+  "Mean Higher Low Water", "Mean Low Water Springs",
+  "Mean Lower Low Water Springs", "Mean Low Water Neaps",
+  "Mean High Water Neaps", "Mean High Water Springs",
   "Mean Higher High Water Springs", "Indian Spring Low Water",
   "Equatorial Spring Low Water", "Lowest Normal Low Water", "Lowest Low Water",
-  "Lowest Possible Low Water", "Lowest Astronomical Tide ", 
+  "Lowest Possible Low Water", "Lowest Astronomical Tide",
   "International Great Lakes Datum(1955)", "Lower Low Water, Large Tide",
   "Lowest Normal Tide", "Higher High Water, Large Tide", "Mean Water Level",
-  "Higher High Water, Mean Tide", "Lower Low Water, Mean Tide", 
+  "Higher High Water, Mean Tide", "Lower Low Water, Mean Tide",
   "Mean Tide Level", "World Geodetic System (1984)",
   "National Geodetic Vertical Datum", "Gulf Coast Low Water Datum",
-  "Approximate Level of Mean Sea Level", 
-  "Approximate Level of Mean Low Water", 
+  "Approximate Level of Mean Sea Level",
+  "Approximate Level of Mean Low Water",
   "Approximate Level of Mean Lower Low Water",
-  "Approximate Level of Mean High Water", 
-  "Approximate Level of Mean Higher High Water", 
+  "Approximate Level of Mean High Water",
+  "Approximate Level of Mean Higher High Water",
   "Approximate Level of Mean Lower High Water",
-  "Approximate Level of Mean Higher Low Water", 
-  "Approximate Level of Mean Low Water Springs", 
-  "Approximate Level of Mean Lower Low Water Springs", 
-  "Approximate Level of Mean Low Water Neaps", 
-  "Approximate Level of Mean High Water Neaps", 
-  "Approximate Level of Mean High Water Springs", 
-  "Approximate Level of Mean Higher High Water Springs", 
+  "Approximate Level of Mean Higher Low Water",
+  "Approximate Level of Mean Low Water Springs",
+  "Approximate Level of Mean Lower Low Water Springs",
+  "Approximate Level of Mean Low Water Neaps",
+  "Approximate Level of Mean High Water Neaps",
+  "Approximate Level of Mean High Water Springs",
+  "Approximate Level of Mean Higher High Water Springs",
   "Approximate Level of Indian Spring Low Water",
-  "Approximate Level of Equatorial Spring Low Water", 
-  "Approximate Level of Lowest Normal Low Water", 
+  "Approximate Level of Equatorial Spring Low Water",
+  "Approximate Level of Lowest Normal Low Water",
   "Approximate Level of Lowest Low Water",
-  "Approximate Level of Lowest Possible Low Water", 
+  "Approximate Level of Lowest Possible Low Water",
   "Approximate Level of Lowest Astronomical Tide",
-  "Approximate Level of International Great Lakes Datum (1955)", 
+  "Approximate Level of International Great Lakes Datum (1955)",
   "Approximate Level of Lower Low Water, Large Tide",
-  "Approximate Level of Lowest Normal Tide", 
-  "Approximate Level of Higher High Water, Large Tide", 
+  "Approximate Level of Lowest Normal Tide",
+  "Approximate Level of Higher High Water, Large Tide",
   "Approximate Level of Mean Water Level",
-  "Approximate Level of Higher High Water, Mean Tide", 
-  "Approximate Level of Lower Low Water, Mean Tide", 
-  "Approximate Level of Mean Tide Level", 
+  "Approximate Level of Higher High Water, Mean Tide",
+  "Approximate Level of Lower Low Water, Mean Tide",
+  "Approximate Level of Mean Tide Level",
   "Approximate Level of World Geodetic System (1984)",
-  "Approximate Level of National Geodetic Vertical Datum", 
+  "Approximate Level of National Geodetic Vertical Datum",
   "Approximate Level of Gulf Coast Low Water Datum"};
 
- 
+
 /*  # Country names from ISO 3166-1:1999 2-character country code list  */
 
 NV_CHAR country[DEFAULT_COUNTRIES][DEFAULT_COUNTRY_SIZE] = {"Unknown",
-  "Afghanistan", "Albania", "Algeria", "Andorra", "Angola", "Anguilla", 
-  "Antarctica", "Antigua & Barbuda", "Argentina", "Armenia", "Aruba", 
-  "Australia", "Austria", "Azerbaijan", "Bahamas", "Bahrain", "Bangladesh", 
-  "Barbados", "Belarus", "Belgium", "Belize", "Benin", "Bermuda", "Bhutan", 
-  "Bolivia", "Bosnia & Herzegovina", "Botswana", "Bouvet Island", "Brazil", 
-  "Britain (UK)", "British Indian Ocean Territory", "Brunei", "Bulgaria", 
-  "Burkina Faso", "Burundi", "Cambodia", "Cameroon", "Canada", "Cape Verde", 
-  "Cayman Islands", "Central African Rep.", "Chad", "Chile", "China", 
-  "Christmas Island", "Cocos (Keeling) Islands", "Colombia", "Comoros", 
-  "Congo (Dem. Rep.)", "Congo (Rep.)", "Cook Islands", "Costa Rica", 
-  "Cote d'Ivoire", "Croatia", "Cuba", "Cyprus", "Czech Republic", "Denmark", 
-  "Djibouti", "Dominica", "Dominican Republic", "East Timor", "Ecuador", 
-  "Egypt", "El Salvador", "Equatorial Guinea", "Eritrea", "Estonia", 
-  "Ethiopia", "Faeroe Islands", "Falkland Islands", "Fiji", "Finland", 
-  "France", "French Guiana", "French Polynesia", 
+  "Afghanistan", "Albania", "Algeria", "Andorra", "Angola", "Anguilla",
+  "Antarctica", "Antigua & Barbuda", "Argentina", "Armenia", "Aruba",
+  "Australia", "Austria", "Azerbaijan", "Bahamas", "Bahrain", "Bangladesh",
+  "Barbados", "Belarus", "Belgium", "Belize", "Benin", "Bermuda", "Bhutan",
+  "Bolivia", "Bosnia & Herzegovina", "Botswana", "Bouvet Island", "Brazil",
+  "Britain (UK)", "British Indian Ocean Territory", "Brunei", "Bulgaria",
+  "Burkina Faso", "Burundi", "Cambodia", "Cameroon", "Canada", "Cape Verde",
+  "Cayman Islands", "Central African Rep.", "Chad", "Chile", "China",
+  "Christmas Island", "Cocos (Keeling) Islands", "Colombia", "Comoros",
+  "Congo (Dem. Rep.)", "Congo (Rep.)", "Cook Islands", "Costa Rica",
+  "Cote d'Ivoire", "Croatia", "Cuba", "Cyprus", "Czech Republic", "Denmark",
+  "Djibouti", "Dominica", "Dominican Republic", "East Timor", "Ecuador",
+  "Egypt", "El Salvador", "Equatorial Guinea", "Eritrea", "Estonia",
+  "Ethiopia", "Faeroe Islands", "Falkland Islands", "Fiji", "Finland",
+  "France", "French Guiana", "French Polynesia",
   "French Southern & Antarctic Lands", "Gabon", "Gambia", "Georgia", "Germany",
   "Ghana", "Gibraltar", "Greece", "Greenland", "Grenada", "Guadeloupe", "Guam",
-  "Guatemala", "Guinea", "Guinea-Bissau", "Guyana", "Haiti", 
-  "Heard Island & McDonald Islands", "Honduras", "Hong Kong", "Hungary", 
-  "Iceland", "India", "Indonesia", "Iran", "Iraq", "Ireland", "Israel", 
-  "Italy", "Jamaica", "Japan", "Jordan", "Kazakhstan", "Kenya", "Kiribati", 
-  "Korea (North)", "Korea (South)", "Kuwait", "Kyrgyzstan", "Laos", "Latvia", 
-  "Lebanon", "Lesotho", "Liberia", "Libya", "Liechtenstein", "Lithuania", 
-  "Luxembourg", "Macau", "Macedonia", "Madagascar", "Malawi", "Malaysia", 
+  "Guatemala", "Guinea", "Guinea-Bissau", "Guyana", "Haiti",
+  "Heard Island & McDonald Islands", "Honduras", "Hong Kong", "Hungary",
+  "Iceland", "India", "Indonesia", "Iran", "Iraq", "Ireland", "Israel",
+  "Italy", "Jamaica", "Japan", "Jordan", "Kazakhstan", "Kenya", "Kiribati",
+  "Korea (North)", "Korea (South)", "Kuwait", "Kyrgyzstan", "Laos", "Latvia",
+  "Lebanon", "Lesotho", "Liberia", "Libya", "Liechtenstein", "Lithuania",
+  "Luxembourg", "Macau", "Macedonia", "Madagascar", "Malawi", "Malaysia",
   "Maldives", "Mali", "Malta", "Marshall Islands", "Martinique", "Mauritania",
-  "Mauritius", "Mayotte", "Mexico", "Micronesia", "Moldova", "Monaco", 
-  "Mongolia", "Montserrat", "Morocco", "Mozambique", "Myanmar (Burma)", 
-  "Namibia", "Nauru", "Nepal", "Netherlands", "Netherlands Antilles", 
-  "New Caledonia", "New Zealand", "Nicaragua", "Niger", "Nigeria", "Niue", 
-  "Norfolk Island", "Northern Mariana Islands", "Norway", "Oman", "Pakistan", 
-  "Palau", "Palestine", "Panama", "Papua New Guinea", "Paraguay", "Peru", 
-  "Philippines", "Pitcairn", "Poland", "Portugal", "Puerto Rico", "Qatar", 
-  "Reunion", "Romania", "Russia", "Rwanda", "Samoa (American)", 
-  "Samoa (Western)", "San Marino", "Sao Tome & Principe", "Saudi Arabia", 
+  "Mauritius", "Mayotte", "Mexico", "Micronesia", "Moldova", "Monaco",
+  "Mongolia", "Montserrat", "Morocco", "Mozambique", "Myanmar (Burma)",
+  "Namibia", "Nauru", "Nepal", "Netherlands", "Netherlands Antilles",
+  "New Caledonia", "New Zealand", "Nicaragua", "Niger", "Nigeria", "Niue",
+  "Norfolk Island", "Northern Mariana Islands", "Norway", "Oman", "Pakistan",
+  "Palau", "Palestine", "Panama", "Papua New Guinea", "Paraguay", "Peru",
+  "Philippines", "Pitcairn", "Poland", "Portugal", "Puerto Rico", "Qatar",
+  "Reunion", "Romania", "Russia", "Rwanda", "Samoa (American)",
+  "Samoa (Western)", "San Marino", "Sao Tome & Principe", "Saudi Arabia",
   "Senegal", "Seychelles", "Sierra Leone", "Singapore", "Slovakia", "Slovenia",
-  "Solomon Islands", "Somalia", "South Africa", 
-  "South Georgia & the South Sandwich Islands", "Spain", "Sri Lanka", 
-  "St Helena", "St Kitts & Nevis", "St Lucia", "St Pierre & Miquelon", 
-  "St Vincent", "Sudan", "Suriname", "Svalbard & Jan Mayen", "Swaziland", 
-  "Sweden", "Switzerland", "Syria", "Taiwan", "Tajikistan", "Tanzania", 
-  "Thailand", "Togo", "Tokelau", "Tonga", "Trinidad & Tobago", "Tunisia", 
+  "Solomon Islands", "Somalia", "South Africa",
+  "South Georgia & the South Sandwich Islands", "Spain", "Sri Lanka",
+  "St Helena", "St Kitts & Nevis", "St Lucia", "St Pierre & Miquelon",
+  "St Vincent", "Sudan", "Suriname", "Svalbard & Jan Mayen", "Swaziland",
+  "Sweden", "Switzerland", "Syria", "Taiwan", "Tajikistan", "Tanzania",
+  "Thailand", "Togo", "Tokelau", "Tonga", "Trinidad & Tobago", "Tunisia",
   "Turkey", "Turkmenistan", "Turks & Caicos Is", "Tuvalu", "Uganda", "Ukraine",
-  "United Arab Emirates", "United States", "Uruguay", 
-  "US minor outlying islands", "Uzbekistan", "Vanuatu", "Vatican City", 
-  "Venezuela", "Vietnam", "Virgin Islands (UK)", "Virgin Islands (US)", 
-  "Wallis & Futuna", "Western Sahara", "Yemen", "Yugoslavia", "Zambia", 
+  "United Arab Emirates", "United States", "Uruguay",
+  "US minor outlying islands", "Uzbekistan", "Vanuatu", "Vatican City",
+  "Venezuela", "Vietnam", "Virgin Islands (UK)", "Virgin Islands (US)",
+  "Wallis & Futuna", "Western Sahara", "Yemen", "Yugoslavia", "Zambia",
   "Zimbabwe"};
 
 
