@@ -1,4 +1,4 @@
-// $Id: skycal.cc,v 1.4 2003/02/20 15:19:56 flaterco Exp $
+// $Id: skycal.cc,v 1.6 2004/10/19 18:31:03 flaterco Exp $
 // skycal.cc -- Functions for sun and moon events.
 
 // This source file began its life as skycalendar.c and skycalc.c in
@@ -225,7 +225,6 @@ lpsun(double jd, double *ra, double *dec)
 
 // Fwd Decl
 static void accumoon (double jd, double geolat, double lst, double elevsea,
-double *geora, double *geodec, double *geodist,
 double *topora, double *topodec, double *topodist);
 
 // Not in Skycal.
@@ -233,9 +232,9 @@ double *topora, double *topodec, double *topodist);
 // find the altitude of the sun or moon.
 static double altitude (double jd, double lat, double longit, int lunar) {
   if (lunar) {
-    double geora, geodec, geodist, ra, dec, dist, sid;
+    double ra, dec, dist, sid;
     sid = lst(jd,longit);
-    accumoon (jd, lat, sid, 0, &geora, &geodec, &geodist, &ra, &dec, &dist);
+    accumoon (jd, lat, sid, 0, &ra, &dec, &dist);
     return altit(dec, sid-ra, lat);
   } else {
     double ra, dec;
@@ -534,7 +533,6 @@ static double etcorr (double jd) {
 	
 // Added 2003-02-04 from Skycal 5 for moonrise/moonset
 static void accumoon (double jd, double geolat, double lst, double elevsea,
-double *geora, double *geodec, double *geodist,
 double *topora, double *topodec, double *topodist)
 
 // double jd,geolat,lst,elevsea;  /* jd, dec. degr., dec. hrs., meters */
@@ -853,11 +851,11 @@ find_next_moon_phase (double &jd, int &phase, int direction) {
 
 // Front end with XTide data types
 void
-find_next_moon_phase (Timestamp &t, Station::EventType &etype_out,
-Station::Direction d) {
+find_next_moon_phase (Timestamp &t, TideEvent::EventType &etype_out,
+Direction d) {
   int dir, phase;
   double jd = t.jd();
-  if (d == Station::forward)
+  if (d == forward)
     dir = 1;
   else
     dir = -1;
@@ -865,16 +863,16 @@ Station::Direction d) {
   t = Timestamp (jd);
   switch (phase) {
   case 0:
-    etype_out = Station::newmoon;
+    etype_out = TideEvent::newmoon;
     break;
   case 1:
-    etype_out = Station::firstquarter;
+    etype_out = TideEvent::firstquarter;
     break;
   case 2:
-    etype_out = Station::fullmoon;
+    etype_out = TideEvent::fullmoon;
     break;
   case 3:
-    etype_out = Station::lastquarter;
+    etype_out = TideEvent::lastquarter;
     break;
   default:
     assert (0);
@@ -949,12 +947,12 @@ int lunar, int &is_rise) {
 // Front end with XTide data types
 void
 find_next_rise_or_set (Timestamp &t, Coordinates c,
-Station::EventType &etype_out, Station::Direction d, int lunar) {
+TideEvent::EventType &etype_out, Direction d, int lunar) {
   assert (!(c.isNull()));
   int dir;
   int isrise;
   double jd = t.jd();
-  if (d == Station::forward)
+  if (d == forward)
     dir = 1;
   else
     dir = -1;
@@ -963,9 +961,9 @@ Station::EventType &etype_out, Station::Direction d, int lunar) {
   find_next_rise_or_set (jd, c.lat(), -(c.lng())/15.0, dir, lunar, isrise);
   t = Timestamp (jd);
   if (isrise)
-    etype_out = (lunar ? Station::moonrise : Station::sunrise);
+    etype_out = (lunar ? TideEvent::moonrise : TideEvent::sunrise);
   else
-    etype_out = (lunar ? Station::moonset : Station::sunset);
+    etype_out = (lunar ? TideEvent::moonset : TideEvent::sunset);
 }
 
 // Simple question deserving a simple answer...
