@@ -1,4 +1,4 @@
-/* $Id: build_tide_db.c,v 1.8 2003/04/03 15:11:01 flaterco Exp $ */
+/* $Id: build_tide_db.c,v 1.25 2004/10/19 18:37:20 flaterco Exp $ */
 
 #include <stdio.h>
 #include <errno.h>
@@ -42,17 +42,17 @@
      Author          Jan C. Depner
      Date            08/01/02
 
-     Modified        August Hahn
-     Date            02/26/03
-
-     Modified        David Flater
-     Date            2003-03-19
+     Modified        David Flater / August Hahn
+     See README for details.
 
 \*****************************************************************************/
 
 NV_BOOL parse_xml_station (FILE *fp, TIDE_RECORD *rec);
 NV_BOOL parse_line (NV_CHAR *in, NV_CHAR *search, NV_CHAR *result);
 
+/* This never really worked.  Use the new hot comment or XML field,
+   or put up with Unknown. */
+#if 0
 NV_INT32 locate_country(NV_CHAR *in)
 {
     /* DWF fixed buffer overflow s[4] 2003-03-19  */
@@ -142,6 +142,15 @@ NV_INT32 locate_country(NV_CHAR *in)
     if (strstr(country, "F.S.M."))
 	strcpy(country, "Micronesia");
 
+    if (strstr(country, "UK ") || strstr(country, "U.K.")
+	|| strstr(country, "Wales")
+	|| strstr(country, "Scotland")
+	|| strstr(country, "England")
+	|| strstr(country, "Northern Ireland")
+	|| strstr(country, "Channel Islands")
+	|| strstr(country, "Isle of Man"))
+      strcpy(country, "Britain (UK)");
+
     result = find_country(country);
 
     if (result == -1) {
@@ -172,14 +181,15 @@ NV_INT32 locate_country(NV_CHAR *in)
 
     return result;
 }
+#endif
 
-NV_INT32 ProcessHarmonicsFile(FILE* fp, DB_HEADER_PUBLIC* db)
+NV_INT32 ProcessHarmonicsFile(FILE* fp, NV_U_INT32 num_constituents)
 {
     NV_CHAR                    string[256], tname[100];
-    NV_INT32                   i, j, num, comment_pos, cnt = 0;
-    NV_BOOL                    done;
+    NV_INT32                   i, j, num, comment_pos = 0, cnt = 0;
+    NV_BOOL                    got_units;
     TIDE_RECORD                rec;
-    comment_pos = 0;
+    int startover = 1;
 
     /*  XTide format ASCII constituent file.  */
 
@@ -196,43 +206,107 @@ NV_INT32 ProcessHarmonicsFile(FILE* fp, DB_HEADER_PUBLIC* db)
 	beginning of the constituents.  This is not part of the
 	comments for the first record and I need to grab those
 	comments.  */
-
-    for (i = 0 ; i < 27 ; i++)
-	fgets (string, sizeof (string), fp);
-
-    done = NVTrue;
-
-    memset (&rec, 0, sizeof (TIDE_RECORD));
+    /* DWF 2004-08-09 changed from counting 27 lines to looking for
+       MERCHANTABILITY. */
+    /* DWF 2004-08-15 if not found, it's a warning, not an error. */
+    do {
+      if (!fgets (string, sizeof (string), fp)) {
+        fprintf (stderr, "Warning:  no tide records found.\n");
+        return 0;
+      }
+      assert (string[0] == '#');
+    } while (!strstr (string, "MERCHANTABILITY"));
+    /* Lose extra blank */
+    assert (fgets (string, sizeof (string), fp));
+    assert (!strcmp (string, "#\n"));
 
     while (fgets (string, sizeof (string), fp))
     {
+        if (startover) {
+          startover = comment_pos = 0;
+          got_units = NVFalse;
+          memset (&rec, 0, sizeof (TIDE_RECORD));
+          rec.header.record_type = 1;
+          rec.header.reference_station = -1;
+          rec.confidence = 10;
+	  rec.restriction = (NV_U_BYTE) find_or_add_restriction ("Non-commercial use only", NULL);
+  	  rec.datum = find_datum ("Mean Lower Low Water");
+          rec.max_direction = rec.min_direction = 361;
+	  {
+	    int r = find_dir_units ("degrees true");
+	    assert (r > 0);
+	    rec.direction_units = (NV_U_BYTE)r;
+	  }
+        }
+
 	/*  Comments.  */
 	if (string[0] == '#' || strlen (string) < 2)
 	{
-	    if (!strncmp (string, "# Source:", 9))
+	    /* FIXME fixup hot comments protocol after X-fields added */
+	    if (strstr (string, "BEGIN HOT COMMENTS"))
+	      ;
+            else if (!strncmp (string, "# country:", 10))
+	      rec.country = find_or_add_country (string+10, NULL);
+	    else if (!strncmp (string, "# Source:", 9) ||
+                     !strncmp (string, "# source:", 9))
 	    {
-		rec.source[0] = 0;
-		if ((strlen (&string[9])) < SOURCE_LENGTH)
+		if ((strlen (&string[9])) < ONELINER_LENGTH)
 		{
+                  if (string[9] == ' ')
+		    strcpy (rec.source, &string[10]);
+                  else
 		    strcpy (rec.source, &string[9]);
 		} else {
-		    fprintf (stderr, "libtcd fatal error:  Source '%s' exceeded SOURCE_LENGTH compiled-in limit %d.  Change limit and recompile.\n", string+9, SOURCE_LENGTH);
+		    fprintf (stderr, "libtcd fatal error:  Source '%s' exceeded ONELINER_LENGTH compiled-in limit %d.  Change limit and recompile.\n", string+9, ONELINER_LENGTH);
 		    exit (-1);
 		}
 	    }
-	    else if (!strncmp (string, "# Datum:", 8))
-	    {
-		rec.datum = find_datum (&string[8]);
-		if (rec.datum < 0)
-		    rec.datum = add_datum (&string[15], db);
-	    }
-	    else if (!strncmp (string, "# rel_pedigree:", 15))
-	    {
-		rec.pedigree = find_pedigree (&string[15]);
-		if (rec.pedigree < 0)
-		    rec.pedigree = add_pedigree (&string[15], db);
-	    }
-	    else if (!strncmp (string, "# !units:", 9))
+            else if (!strncmp (string, "# restriction:", 14))
+              rec.restriction = find_or_add_restriction (string+14, NULL);
+            /* Space intentionally included */
+            else if (!strncmp (string, "# note: ", 8))
+              strcat (rec.notes, string+8);
+            else if (!strncmp (string, "# station_id_context:", 21)) {
+              if ((strlen (&string[21])) < ONELINER_LENGTH)
+                strcpy (rec.station_id_context, string+21);
+              else {
+		    fprintf (stderr, "libtcd fatal error:  Station ID context '%s' exceeded ONELINER_LENGTH compiled-in limit %d.  Change limit and recompile.\n", string+21, ONELINER_LENGTH);
+		    exit (-1);
+	      }
+            } else if (!strncmp (string, "# station_id:", 13)) {
+              if ((strlen (&string[13])) < ONELINER_LENGTH)
+                strcpy (rec.station_id, string+13);
+              else {
+		    fprintf (stderr, "libtcd fatal error:  Station ID '%s' exceeded ONELINER_LENGTH compiled-in limit %d.  Change limit and recompile.\n", string+13, ONELINER_LENGTH);
+		    exit (-1);
+	      }
+	    } else if (!strncmp (string, "# date_imported:", 16))
+              assert (sscanf (string+16, "%u", &(rec.date_imported)) == 1);
+            else if (!strncmp (string, "# max_direction:", 16))
+              assert (sscanf (string+16, "%d", &(rec.max_direction)) == 1);
+            else if (!strncmp (string, "# min_direction:", 16))
+              assert (sscanf (string+16, "%d", &(rec.min_direction)) == 1);
+            else if (!strncmp (string, "# Datum:", 8) ||
+                     !strncmp (string, "# datum:", 8))
+	      rec.datum = find_or_add_datum (&string[8], NULL);
+            else if (!strncmp (string, "# expiration:", 13))
+              assert (sscanf (string+13, "%u", &(rec.expiration_date)) == 1);
+            else if (!strncmp (string, "# used_until:", 13)) {
+              unsigned y, m, d;
+              assert (sscanf (string+13, "%u-%u-%u", &y, &m, &d) == 3);
+              rec.expiration_date = y*10000 + m*100 + d;
+            } else if (!strncmp (string, "# months_on_station:", 20)) {
+              unsigned m;
+              assert (sscanf (string+20, "%u", &m) == 1);
+              rec.months_on_station = (NV_U_INT16)m;
+            } else if (!strncmp (string, "# last_date_on_station:", 23))
+              assert (sscanf (string+23, "%u", &(rec.last_date_on_station)) == 1);
+            else if (!strncmp (string, "# confidence:", 13)) {
+              unsigned i;
+              assert (sscanf (string+13, "%u", &i) == 1);
+              rec.confidence = (NV_U_BYTE)i;
+            } else if (!strncmp (string, "# !units:", 9) ||
+                     !strncmp (string, "#!units:", 8))
 	    {
 		/*  Do nothing.  This is superceded by the datum units.  */
 	    }
@@ -248,18 +322,19 @@ NV_INT32 ProcessHarmonicsFile(FILE* fp, DB_HEADER_PUBLIC* db)
 	    }
 	    else
 	    {
+              /* FIXME unknown hot comments will become X-fields */
 		if (strlen(string) < 3) {
 		    /* Blank line */
-		    if (comment_pos + 1 >= COMMENTS_LENGTH) {
-			fprintf (stderr, "libtcd fatal error:  Comments exceeded COMMENTS_LENGTH compiled-in limit %d.  Change limit and recompile.\n", COMMENTS_LENGTH);
+		    if (comment_pos + 1 >= MONOLOGUE_LENGTH) {
+			fprintf (stderr, "libtcd fatal error:  Comments exceeded MONOLOGUE_LENGTH compiled-in limit %d.  Change limit and recompile.\n", MONOLOGUE_LENGTH);
                         exit (-1);
 		    }
 		    rec.comments[comment_pos++] = 10;
 		} else {
 		    num = strlen (string+1);
 		    assert (num > 0);
-		    if (comment_pos + num >= COMMENTS_LENGTH) {
-			fprintf (stderr, "libtcd fatal error:  Comments containing '%s' exceeded COMMENTS_LENGTH compiled-in limit %d.  Change limit and recompile.\n", string+1, COMMENTS_LENGTH);
+		    if (comment_pos + num >= MONOLOGUE_LENGTH) {
+			fprintf (stderr, "libtcd fatal error:  Comments containing '%s' exceeded MONOLOGUE_LENGTH compiled-in limit %d.  Change limit and recompile.\n", string+1, MONOLOGUE_LENGTH);
                         exit (-1);
 		    } else {
                         /* Compress out leading blank, if present */
@@ -280,7 +355,7 @@ NV_INT32 ProcessHarmonicsFile(FILE* fp, DB_HEADER_PUBLIC* db)
 
 	else
 	{
-	    if ((strlen (string)) < NAME_LENGTH)
+	    if ((strlen (string)) < ONELINER_LENGTH)
 	    {
 		if (string[strlen(string) - 1] == '\n')
 		    string[strlen(string) - 1] = '\0';
@@ -288,7 +363,7 @@ NV_INT32 ProcessHarmonicsFile(FILE* fp, DB_HEADER_PUBLIC* db)
 	    }
 	    else
 	    {
-		fprintf (stderr, "libtcd fatal error:  Name '%s' exceeded NAME_LENGTH compiled-in limit %d.  Change limit and recompile.\n", string, NAME_LENGTH);
+		fprintf (stderr, "libtcd fatal error:  Name '%s' exceeded ONELINER_LENGTH compiled-in limit %d.  Change limit and recompile.\n", string, ONELINER_LENGTH);
 		exit (-1);
 	    }
 
@@ -296,26 +371,20 @@ NV_INT32 ProcessHarmonicsFile(FILE* fp, DB_HEADER_PUBLIC* db)
 	    sscanf (string, "%d:%d %s", &i, &j, tname);
 	    rec.zone_offset = get_time (string);
 
-	    rec.header.tzfile = find_tzfile (tname);
-	    if (rec.header.tzfile < 0)
-		rec.header.tzfile = add_tzfile (tname, db);
+	    rec.header.tzfile = find_or_add_tzfile (tname, NULL);
 
-	    done = NVTrue;
 	    fgets (string, sizeof (string), fp);
 	    sscanf (string, "%f %s", &rec.datum_offset, tname);
 	    i = find_level_units (tname);
 
 	    if (i < 0)
-	    {
-		done = NVFalse;
-	    }
-	    else
-	    {
-		rec.units = i;
+	      got_units = NVFalse;
+	    else {
+ 	      got_units = NVTrue;
+	      rec.level_units = i;
 	    }
 
-
-	    for (i = 0 ; i < db->constituents ; i++)
+	    for (i = 0 ; i < (int)num_constituents ; i++)
 	    {
 		fgets (string, sizeof (string), fp);
 
@@ -336,25 +405,11 @@ NV_INT32 ProcessHarmonicsFile(FILE* fp, DB_HEADER_PUBLIC* db)
 	    /*  Add the final 0 to the comments record since we stripped
 		it on input.  */
 
-	    rec.comments[comment_pos - 1] = 0;
-	    comment_pos = 0;
+	    rec.comments[comment_pos - 1] = (char)0;
 
-	    /*  Defaults (for now).  */
-
-	    rec.country = locate_country(rec.header.name);
-	    rec.header.record_type = 1;
-	    rec.header.reference_station = -1;
-	    rec.restriction = 0;
-	    rec.expiration_date = 0;
-	    rec.months_on_station = 0;
-	    rec.last_date_on_station = 0;
-	    rec.confidence = 10;
-	    if (!rec.datum)
-		rec.datum = find_datum ("Mean Lower Low Water");
-
-	    if (done)
+	    if (got_units)
 	    {
-		add_tide_record (&rec, db);
+		add_tide_record (&rec, NULL);
 		cnt++;
 	    }
 	    else
@@ -362,16 +417,14 @@ NV_INT32 ProcessHarmonicsFile(FILE* fp, DB_HEADER_PUBLIC* db)
 		fprintf (stderr, "Invalid level units for record - %s\n", rec.header.name);
 	    }
 
-	    memset (&rec, 0, sizeof (TIDE_RECORD));
-
-	    done = NVTrue;
+	    startover = 1;
 	}
     }
 
     return cnt;
 }
 
-NV_INT32 ProcessNavoFile(FILE* fp, DB_HEADER_PUBLIC* db)
+NV_INT32 ProcessNavoFile(FILE* fp)
 {
     NV_CHAR                    string[256], dum1[3], dum2[5], name[4][12];
     NV_FLOAT32                 amp[4], epo[4], dummy;
@@ -456,14 +509,12 @@ NV_INT32 ProcessNavoFile(FILE* fp, DB_HEADER_PUBLIC* db)
 	rec.confidence = 10;
 	rec.zone_offset = 0;
         rec.country = 0;
-	rec.units = find_level_units ("meters");
-	rec.header.tzfile = find_tzfile ("GMT0");
-        if (rec.header.tzfile < 0)
-            rec.header.tzfile = add_tzfile ("GMT0", db);
+	rec.level_units = find_level_units ("meters");
+	rec.header.tzfile = find_or_add_tzfile ("GMT0", NULL);
 	rec.datum = find_datum ("Mean Lower Low Water");
 	strcpy (rec.source, "IHO/NAVO ihbtides file");
 
-	add_tide_record (&rec, db);
+	add_tide_record (&rec, NULL);
 	cnt++;
 	memset (&rec, 0, sizeof (TIDE_RECORD));
     }
@@ -471,12 +522,13 @@ NV_INT32 ProcessNavoFile(FILE* fp, DB_HEADER_PUBLIC* db)
     return cnt;
 }
 
-NV_INT32 ProcessXMLFile(FILE* fp, DB_HEADER_PUBLIC* db)
+NV_INT32 ProcessXMLFile(FILE* fp)
 {
-    NV_CHAR                    string[256];
+    NV_CHAR                    string[256], comments[MONOLOGUE_LENGTH];
     NV_BOOL                    done;
     TIDE_RECORD                rec;
     NV_INT32	cnt = 0; /* DWF added initialization */
+    int incomment=0;
 
     done = NVFalse;
 
@@ -487,23 +539,33 @@ NV_INT32 ProcessXMLFile(FILE* fp, DB_HEADER_PUBLIC* db)
 	if (!strncmp (string, "<document>", 10)) done = NVTrue;
     }
 
-    memset (&rec, 0, sizeof (TIDE_RECORD));
-
+    comments[0] = '\0';
     while (fgets (string, sizeof (string), fp))
     {
-	if (parse_line (string, "<subordinatestation name=", rec.header.name))
+      if (!strncmp (string, "<!-- Comments on ", 17))
+        incomment=1;
+      else if (incomment) {
+        if (!strncmp (string, "-->", 3))
+          incomment=0;
+        else {
+          assert (strlen(string) + strlen(comments) + 1 < MONOLOGUE_LENGTH);
+          strcat (comments, string);
+        }
+      } else if (parse_line (string, "<subordinatestation name=", rec.header.name))
 	{
-	    if (parse_xml_station (fp, &rec))
+	  if (parse_xml_station (fp, &rec))
 	    {
-		/*  Defaults (for now).  */
-		rec.header.record_type = 2;
-		rec.restriction = 0;
-		rec.country = locate_country(rec.header.name);
-
-		add_tide_record (&rec, db);
+	        /* Evil */
+  	        /* rec.country = locate_country(rec.header.name); */
+                /* Stupid notes workaround */
+                if (strlen (rec.comments)) {
+                  strcat (comments, "\n");
+                  strcat (comments, rec.comments);
+                }
+                strcpy (rec.comments, comments);
+		add_tide_record (&rec, NULL);
 		cnt++;
-
-		memset (&rec, 0, sizeof (TIDE_RECORD));
+                comments[0] = '\0';
 	    }
 	}
     }
@@ -724,12 +786,26 @@ void CreateTCDFromHarmonics(NV_CHAR* dbname, FILE* fp)
 	comments for the first record and I need to grab those
 	comments.  */
 
+    /* DWF 2004-08-12: Discovered this second occurrence of the
+       hard-coded line count *after* committing the change to
+       duplicate code in ProcessHarmonicsFile on 2004-08-09.  However,
+       the only place this function is used is in main, which closes
+       the file immediately afterward.  No harm, no foul.  Since I
+       don't know why we are bothering to skip forward, I am now
+       commenting out this code.  That way, it's possible to convert
+       congen output and get a TCD file containing only the
+       constituents (zero tide records).  fp officially left in open
+       but otherwise unknown state. */
+#if 0
     for (i = 0 ; i < 27 ; i++)
 	fgets (string, sizeof (string), fp);
+#endif
 
     /*  Free the memory (free the Chicago seven, free Huey [and Dewey
 	and Louie]).  */
 
+    /* FIXME bogus null checking */
+
     for (i = 0 ; i < constituents ; i++)
     {
 	if (constituent[i] != NULL) free (constituent[i]);
@@ -751,7 +827,6 @@ void CreateTCDFromHarmonics(NV_CHAR* dbname, FILE* fp)
 	if (node_factor[i] != NULL) free (node_factor[i]);
     }
     if (node_factor != NULL) free (node_factor);
-
 }
 
 void CreateTCDFromNavo(NV_CHAR* dbname)
@@ -789,6 +864,8 @@ void CreateTCDFromNavo(NV_CHAR* dbname)
 	exit (-1);
     }
 
+    /* FIXME bogus null checking */
+
     for (i = 0 ; i < constituents ; i++)
     {
 	if (equilibrium[i] != NULL) free (equilibrium[i]);
@@ -911,19 +988,19 @@ NV_INT32 main (NV_INT32 argc, char *argv[])
 	/*  XTide format XML offset file (subordinate stations).  */
 	if (xml)
 	{
-	    cnt += ProcessXMLFile(fp, &db);
+	    cnt += ProcessXMLFile(fp);
 	}
 
 	/*  NAVO format constituent file.  */
 	else if (navo)
 	{
-	    cnt += ProcessNavoFile(fp, &db);
+	    cnt += ProcessNavoFile(fp);
 	}
 
 	/*  XTide constituent file.  */
 	else
 	{
-	    cnt += ProcessHarmonicsFile(fp, &db);
+	    cnt += ProcessHarmonicsFile(fp, db.constituents);
 	}
 	fclose(fp);
     }
