@@ -30,16 +30,16 @@
 static int compare_names (const void *a, const void *b)
 {
     char low_left[15], low_right[15];
-
+    unsigned i;
 
     CONSTITUENT left = *(CONSTITUENT *)(a);
     CONSTITUENT right = *(CONSTITUENT *)(b);
 
-    for (int i = 0 ; i < strlen (left.name) ; i++)
+    for (i = 0 ; i < strlen (left.name) ; i++)
         low_left[i] = tolower (left.name[i]);
     low_left[strlen (left.name)] = 0;
 
-    for (int i = 0 ; i < strlen (right.name) ; i++)
+    for (i = 0 ; i < strlen (right.name) ; i++)
         low_right[i] = tolower (right.name[i]);
     low_right[strlen (right.name)] = 0;
 
@@ -57,7 +57,7 @@ TideDialog::TideDialog (QWidget *parent, const char *name, TIDE_RECORD lrec,
                         TideEditor *te):
   QTabDialog (parent, name)
 {
-    int       i;
+    unsigned       i;
 
 
     db = get_tide_db_header ();
@@ -144,7 +144,7 @@ void
 TideDialog::generalTab ()
 {
     char hem;
-    int i;
+    unsigned i;
     double deg, min, sec;
 
 
@@ -233,21 +233,6 @@ TideDialog::generalTab ()
     country->setCurrentItem (rec.country);
 
 
-    QLabel *pedigrees = new QLabel ("Pedigree:", generalTab);
-    pedigrees->setMaximumHeight (pedigrees->sizeHint().height() * 2);
-    pedigrees->setFrameStyle (QFrame::Panel | QFrame::Sunken);
-    pedigree = new QComboBox (FALSE, generalTab);
-    QWhatsThis::add (pedigree, pedigreeText);
-    QWhatsThis::add (pedigrees, pedigreeText);
-
-    for (i = 0 ; i < db.pedigree_types ; i++) 
-    {
-	QString str = QString (get_pedigree (i));
-        pedigree->insertItem (str);
-    }
-    pedigree->setCurrentItem (rec.pedigree);
-
-
     QLabel *restrictions = new QLabel ("Restriction:", generalTab);
     restrictions->setMaximumHeight (restrictions->sizeHint().height() * 2);
     restrictions->setFrameStyle (QFrame::Panel | QFrame::Sunken);
@@ -283,7 +268,7 @@ TideDialog::generalTab ()
 void
 TideDialog::constituentsTab ()
 {
-    int i;
+    unsigned i;
 
 
     //  Don't display this if this is a Subordinate record.
@@ -295,20 +280,20 @@ TideDialog::constituentsTab ()
     constituentsTab->setMargin (5);
     constituentsTab->setSpacing (5);
 
-    QLabel *unitss = new QLabel ("Units:", constituentsTab);
-    unitss->setMaximumHeight (unitss->sizeHint().height() * 2);
-    unitss->setFrameStyle (QFrame::Panel | QFrame::Sunken);
-    units = new QComboBox (FALSE, constituentsTab);
-    QWhatsThis::add (units, unitsText);
-    QWhatsThis::add (unitss, unitsText);
+    QLabel *level_unitss = new QLabel ("Level Units:", constituentsTab);
+    level_unitss->setMaximumHeight (level_unitss->sizeHint().height() * 2);
+    level_unitss->setFrameStyle (QFrame::Panel | QFrame::Sunken);
+    level_units = new QComboBox (FALSE, constituentsTab);
+    QWhatsThis::add (level_units, level_unitsText);
+    QWhatsThis::add (level_unitss, level_unitsText);
 
 
     for (i = 0 ; i < db.level_unit_types ; i++) 
     {
 	QString str = QString (get_level_units (i));
-        units->insertItem (str);
+        level_units->insertItem (str);
     }
-    units->setCurrentItem (rec.units);
+    level_units->setCurrentItem (rec.level_units);
 
 
     QLabel *datums = new QLabel ("Datum:", constituentsTab);
@@ -448,7 +433,7 @@ TideDialog::constituentsTab ()
     if (rec.header.record_number >= 0 && rec.header.reference_station == -1)
     {
         TIDE_RECORD temp_rec;
-        int i, subs = 0;
+        unsigned i, subs = 0;
         
         QVBox *subTab = new QVBox (this);
         subTab->setMargin (5);
@@ -527,7 +512,7 @@ TideDialog::constituentsTab ()
 void
 TideDialog::offsetsTab ()
 {
-    int i;
+    unsigned i;
 
 
     //  If this is a Reference station, don't display this tab.
@@ -600,16 +585,6 @@ TideDialog::offsetsTab ()
     QWhatsThis::add (min_level_mult, min_level_multText);
 
 
-    QString min_avg_levels = QString ("%1").arg (rec.min_avg_level, 0, 'f', 
-        2);
-    if (min_avg_levels == "0.00") min_avg_levels = "";
-    QLabel *min_avg_levellab =  
-        new QLabel ("Minimum Average Level:", offsetsTab);
-    min_avg_level = new QLineEdit (min_avg_levels, offsetsTab);
-    QWhatsThis::add (min_avg_levellab, min_avg_levelText);
-    QWhatsThis::add (min_avg_level, min_avg_levelText);
-
-
     QString min_directions = QString ("%1").arg (rec.min_direction);
     if (min_directions == "361") min_directions = "";
     QLabel *min_directionlab = new QLabel ("Minimum Direction:", offsetsTab);
@@ -645,16 +620,6 @@ TideDialog::offsetsTab ()
     QWhatsThis::add (max_level_mult, max_level_multText);
 
 
-    QString max_avg_levels = QString ("%1").arg (rec.max_avg_level, 0, 'f', 
-        2);
-    if (max_avg_levels == "0.00") max_avg_levels = "";
-    QLabel *max_avg_levellab = 
-        new QLabel ("Maximum Average Level:", offsetsTab);
-    max_avg_level = new QLineEdit (max_avg_levels, offsetsTab);
-    QWhatsThis::add (max_avg_levellab, max_avg_levelText);
-    QWhatsThis::add (max_avg_level, max_avg_levelText);
-
-
     QString max_directions = QString ("%1").arg (rec.max_direction);
     if (max_directions == "361") max_directions = "";
     QLabel *max_directionlab = new QLabel ("Maximum Direction:", offsetsTab);
@@ -693,7 +658,7 @@ TideDialog::offsetsTab ()
 void
 TideDialog::slotUpdateStation ()
 {
-    int i;
+    unsigned i;
     double inlat, inlon;
     char string[50];
 
@@ -718,7 +683,6 @@ TideDialog::slotUpdateStation ()
 
     rec.header.tzfile = tzfile->currentItem ();
     rec.country = country->currentItem ();
-    rec.pedigree = pedigree->currentItem ();
     rec.restriction = restriction->currentItem ();
     strcpy (rec.comments, comments->text ());
 
@@ -727,29 +691,29 @@ TideDialog::slotUpdateStation ()
 
     if (rec.header.record_type == 1)
     {
-        rec.units = units->currentItem ();
+        rec.level_units = level_units->currentItem ();
         rec.datum = datum->currentItem ();
         sscanf (datum_offset->text (), "%f", &rec.datum_offset);
         sscanf (zone_offset->text (), "%d", &rec.zone_offset);
-        if (sscanf (expiration_date->text (), "%d", &rec.expiration_date) <= 0)
+        if (sscanf (expiration_date->text (), "%u", &rec.expiration_date) <= 0)
             rec.expiration_date = 0;
         // DWF: fixed warning: int format, different type arg
         {
           int tempm;
           if (sscanf (months->text (), "%d", &tempm) <= 0)
-              rec.months_on_station = (NV_U_BYTE)0;
+              rec.months_on_station = (NV_U_INT16)0;
           else
-	      rec.months_on_station = (NV_U_BYTE)tempm;
+	      rec.months_on_station = (NV_U_INT16)tempm;
         }
-        if (sscanf (ldos->text (), "%d", &rec.last_date_on_station) <= 0)
+        if (sscanf (ldos->text (), "%u", &rec.last_date_on_station) <= 0)
             rec.last_date_on_station = 0;
 
         for (i = 0 ; i < db.constituents ; i++)
         {
             float amp, epo;
-            if (sscanf (constituents->text (i, 0), "%f", &amp) == EOF) 
+            if (sscanf (constituents->text (i, 0), "%f", &amp) != 1) 
                 amp = 0.0;
-            if (sscanf (constituents->text (i, 1), "%f", &epo) == EOF) 
+            if (sscanf (constituents->text (i, 1), "%f", &epo) != 1) 
                 epo = 0.0;
 
             int j = cnst[i].num;
@@ -778,8 +742,6 @@ TideDialog::slotUpdateStation ()
             rec.min_level_add = 0;
         if (sscanf (min_level_mult->text (), "%f", &rec.min_level_multiply) <=
             0) rec.min_level_multiply = 0;
-        if (sscanf (min_avg_level->text (), "%f", &rec.min_avg_level) <= 0)
-            rec.min_avg_level = 0;
         if (sscanf (min_direction->text (), "%d", &rec.min_direction) <= 0)
             rec.min_direction = 361;
         if (sscanf (max_time_add->text (), "%d", &rec.max_time_add) <= 0)
@@ -788,8 +750,6 @@ TideDialog::slotUpdateStation ()
             rec.max_level_add = 0;
         if (sscanf (max_level_mult->text (), "%f", &rec.max_level_multiply) <=
             0) rec.max_level_multiply = 0;
-        if (sscanf (max_avg_level->text (), "%f", &rec.max_avg_level) <= 0)
-            rec.max_avg_level = 0;
         if (sscanf (max_direction->text (), "%d", &rec.max_direction) <= 0)
             rec.max_direction = 361;
         /* DWF updated per slack offsets fix 2003-03-18 */
@@ -811,7 +771,7 @@ TideDialog::slotUpdateStation ()
 
     if (rec.header.record_number >= 0)
     {
-        update_tide_record (rec.header.record_number, &rec);
+      update_tide_record (rec.header.record_number, &rec, &db);
     }
     else
     {
@@ -833,7 +793,7 @@ TideDialog::slotCancelStation ()
 {
     if (rec.header.record_type == 1)
     {
-        delete (units);
+        delete (level_units);
         delete (datum);
         delete (datum_offset);
         delete (zone_offset);
@@ -849,12 +809,10 @@ TideDialog::slotCancelStation ()
         delete (min_time_add);
         delete (min_level_add);
         delete (min_level_mult);
-        delete (min_avg_level);
         delete (min_direction);
         delete (max_time_add);
         delete (max_level_add);
         delete (max_level_mult);
-        delete (max_avg_level);
         delete (max_direction);
         delete (flood_begins);
         delete (ebb_begins);
@@ -867,7 +825,6 @@ TideDialog::slotCancelStation ()
     delete (lon);
     delete (tzfile);
     delete (country);
-    delete (pedigree);
     delete (restriction);
     delete (comments);
     delete (tideDialog);
@@ -901,7 +858,7 @@ TideDialog::slotDeleteStation ()
     if (rec.header.record_type == 1 && rec.header.reference_station == -1)
     {
         TIDE_RECORD temp_rec;
-        int i, subs = 0;
+        unsigned i, subs = 0;
         
         for (i = 0 ; i < db.number_of_records ; i++)
         {
@@ -969,8 +926,6 @@ TideDialog::slotPrintStation ()
         s->append (string);
         sprintf (string, "Country: %s\n", get_country (rec.country));
         s->append (string);
-        sprintf (string, "Pedigree: %s\n", get_pedigree (rec.pedigree));
-        s->append (string);
         sprintf (string, "Restriction: %s\n", 
                  get_restriction (rec.restriction));
         s->append (string);
@@ -980,9 +935,9 @@ TideDialog::slotPrintStation ()
 
         if (rec.header.record_type == 1)
           {
-            int i;
+            unsigned i;
 
-            sprintf (string, "Units: %s\n", get_level_units (rec.units));
+            sprintf (string, "Level Units: %s\n", get_level_units (rec.level_units));
             s->append (string);
             sprintf (string, "Datum: %s\n", get_datum (rec.datum));
             s->append (string);
@@ -997,7 +952,7 @@ TideDialog::slotPrintStation ()
               }
             if (rec.months_on_station)
               {
-                sprintf (string, "Months On Station: %d\n", 
+                sprintf (string, "Months On Station: %u\n",
                          rec.months_on_station);
                 s->append (string);
               }
@@ -1086,12 +1041,6 @@ TideDialog::slotPrintStation ()
                          rec.min_level_multiply);
                 s->append (string);
               }
-            if (rec.min_avg_level != 0.00)
-              {
-                sprintf (string, "Minimum Average Level: %f\n", 
-                         rec.min_avg_level);
-                s->append (string);
-              }
             if (rec.min_direction != 361)
               {
                 sprintf (string, "Minimum Direction: %d\n", rec.min_direction);
@@ -1113,12 +1062,6 @@ TideDialog::slotPrintStation ()
                          rec.max_level_multiply);
                 s->append (string);
               }
-            if (rec.max_avg_level != 0.00)
-              {
-                sprintf (string, "Maximum Average Level: %f\n", 
-                         rec.max_avg_level);
-                s->append (string);
-              }
             if (rec.max_direction != 361)
               {
                 sprintf (string, "Maximum Direction: %d\n", rec.max_direction);
@@ -1213,17 +1156,17 @@ TideDialog::slotChangeRefStation ()
 
         if (ok)
         {
-            char station[NAME_LENGTH];
+            char station[ONELINER_LENGTH];
             strcpy (station, res);
 
             int num = find_station (station);
-            for (int i = 0 ; i < subList->count () ; i++)
+            for (unsigned i = 0 ; i < subList->count () ; i++)
             {
                 if (subList->isSelected (i))
                 {
                     read_tide_record (sub_station[i], &temp_rec);
                     temp_rec.header.reference_station = num;
-                    update_tide_record (sub_station[i], &temp_rec);
+                    update_tide_record (sub_station[i], &temp_rec, &db);
                 }
             }
         }
