@@ -31,17 +31,17 @@
 
 static int compare_names (const void *a, const void *b)
 {
-    char low_left[NAME_LENGTH], low_right[NAME_LENGTH];
-
+    char low_left[ONELINER_LENGTH], low_right[ONELINER_LENGTH];
+    unsigned i;
 
     STATION_LIST left = *(STATION_LIST *)(a);
     STATION_LIST right = *(STATION_LIST *)(b);
 
-    for (int i = 0 ; i < strlen (left.name) ; i++)
+    for (i = 0 ; i < strlen (left.name) ; i++)
         low_left[i] = tolower (left.name[i]);
     low_left[strlen (left.name)] = 0;
 
-    for (int i = 0 ; i < strlen (right.name) ; i++)
+    for (i = 0 ; i < strlen (right.name) ; i++)
         low_right[i] = tolower (right.name[i]);
     low_right[strlen (right.name)] = 0;
 
@@ -238,7 +238,7 @@ TideEditor::TideEditor (QWidget * parent, const char *name):
     //  Read and write the whole file.
 
     int size;
-    while (size = fread (string, 1, sizeof (string), fp))
+    while ((size = fread (string, 1, sizeof (string), fp)))
         fwrite (string, 1, size, fp1);
 
     fclose (fp);
@@ -507,8 +507,7 @@ TideEditor::slotMidMouse (QMouseEvent * e)
     read_tide_record (rec.header.record_number, &rec);
 
 
-    TideDialog *tideDialog = new TideDialog (this, "Tide Constituent Editor", 
-        rec, this);
+    (void) new TideDialog (this, "Tide Constituent Editor", rec, this);
 }
 
 
@@ -546,11 +545,16 @@ TideEditor::slotRightMouse (QMouseEvent * e)
     rec.header.tzfile = 0;
     rec.header.name[0] = 0;
     rec.country = 0;
-    rec.pedigree = 0;
     rec.source[0] = 0;
     rec.restriction = 0;
     rec.comments[0] = 0;
-    rec.units = 0;
+    rec.notes[0] = 0;
+    rec.legalese = 0;
+    rec.xfields[0] = 0;
+    rec.station_id_context[0] = 0;
+    rec.station_id[0] = 0;
+    rec.date_imported = 0;
+    rec.level_units = 0;
     rec.datum_offset = 0.0;
     rec.datum = 0;
     rec.zone_offset = 0;
@@ -563,18 +567,14 @@ TideEditor::slotRightMouse (QMouseEvent * e)
         rec.amplitude[i] = 0.0;
         rec.epoch[i] = 0.0;
     }
-    rec.level_units = 0;
-    rec.avg_level_units = 0;
     rec.direction_units = 0;
     rec.min_time_add = 0;
     rec.min_level_add = 0.0;
     rec.min_level_multiply = 0.0;
-    rec.min_avg_level = 0.0;
     rec.min_direction = 361;
     rec.max_time_add = 0;
     rec.max_level_add = 0.0;
     rec.max_level_multiply = 0.0;
-    rec.max_avg_level = 0.0;
     rec.max_direction = 361;
     rec.flood_begins = rec.ebb_begins = NULLSLACKOFFSET; /* DWF 2003-06-27 */
 
@@ -612,7 +612,7 @@ TideEditor::slotRightMouse (QMouseEvent * e)
 
         if (ok) 
         {
-            char station[NAME_LENGTH];
+            char station[ONELINER_LENGTH];
             strcpy (station, res);
 
             int num = find_station (station);
@@ -630,8 +630,7 @@ TideEditor::slotRightMouse (QMouseEvent * e)
     }
 
 
-    TideDialog *tideDialog = new TideDialog (this, "Add Tide Record", 
-        rec, this);
+    (void) new TideDialog (this, "Add Tide Record", rec, this);
 }
 
 
@@ -711,7 +710,7 @@ TideEditor::plotStations ()
 
     //  Plot the stations.
 
-    for (int i = 0 ; i < db.number_of_records ; i++)
+    for (int i = 0 ; i < (int)db.number_of_records ; i++)
     {
         get_partial_tide_record (i, &rec.header);
 
@@ -1028,7 +1027,7 @@ TideEditor::slotSearch ()
 void 
 TideEditor::slotTextChanged ()
 {
-    char name[NAME_LENGTH];
+    char name[ONELINER_LENGTH];
 
 
     searchList->clear ();
@@ -1069,7 +1068,7 @@ TideEditor::slotTextChanged ()
 void 
 TideEditor::slotSelected ()
 {
-    char name[NAME_LENGTH], item[256];
+    char name[ONELINER_LENGTH], item[256];
     strcpy (item, searchList->currentText ());
 
     strcpy (name, (strchr (item, ':') + 2));
@@ -1079,8 +1078,7 @@ TideEditor::slotSelected ()
     TIDE_RECORD lrec;
     read_tide_record (record_number, &lrec);
 
-    TideDialog *tideDialog = new TideDialog (this, "Tide Constituent Editor", 
-        lrec, this);
+    (void) new TideDialog (this, "Tide Constituent Editor", lrec, this);
 }
 
 
@@ -1095,7 +1093,7 @@ TideEditor::slotHighlighted ()
     highlightStation (temp_rec.header.latitude, temp_rec.header.longitude, 
         temp_rec.header.record_type);
 
-    char name[NAME_LENGTH], item[256];
+    char name[ONELINER_LENGTH], item[256];
     strcpy (item, searchList->currentText ());
 
     strcpy (name, (strchr (item, ':') + 2));
