@@ -1,4 +1,4 @@
-// $Id: xttpd.cc,v 1.5 2003/02/20 15:19:56 flaterco Exp $
+// $Id: xttpd.cc,v 1.22 2004/10/26 15:54:53 flaterco Exp $
 /*  xttpd  XTide web server.
 
     Copyright (C) 1998  David Flater.
@@ -31,15 +31,43 @@ int zoneinfo_doesnt_suck;
 
 // time control modes
 
-enum TC {	TC_ALL,		// year, month, day
+enum TC {	TC_DAY,		// year, month, day
 		TC_YEAR,	// year
 		TC_MONTH	// year, month
 };
 
 #include "purec++.hh"
 
+static int
+parse_addr (unsigned short &port_number, unsigned long &addr, char *arg) {
+  assert (arg);
+  // The : character is reserved for future use to distinguish
+  // IPv4 from IPv6 addresses.
+  if (strchr (arg, '/') || strchr (arg, '.')) {
+    char *c = strrchr (arg, '/');
+    if (c) {
+      *c = '\0';
+      if (sscanf (c+1, "%hu", &port_number) != 1)
+        port_number = 80;
+    }
+    if (!inet_pton (AF_INET, arg, &addr)) {
+      Dstr details ("The offending address was ");
+      details += arg;
+      barf (BAD_IP_ADDRESS, details);
+    }
+    return 1;
+  } else {
+    addr = INADDR_ANY;
+    if (sscanf (arg, "%hu", &port_number) != 1) {
+      port_number = 80;
+      return 0;
+    }
+    return 1;
+  }
+}
+
 static void
-setup_socket (unsigned short port_number, int &s)
+setup_socket (unsigned short port_number, unsigned long addr, int &s)
 {
   struct sockaddr_in sin;
   struct hostent *hp;
@@ -54,7 +82,7 @@ setup_socket (unsigned short port_number, int &s)
   memcpy ((char *) &sin.sin_addr, hp->h_addr, hp->h_length);
   sin.sin_port = htons (port_number);
   sin.sin_family = hp->h_addrtype;
-  sin.sin_addr.s_addr = INADDR_ANY;
+  sin.sin_addr.s_addr = addr;
   if ((s = socket (hp->h_addrtype, SOCK_STREAM, 0)) == -1) {
     Dstr details ("socket: ");
     details += strerror (errno);
@@ -294,7 +322,7 @@ list_location (Dstr &d, StationIndex *si, unsigned long i) {
   d += ".html\">";
   d += sr->name;
   d += "</a></td><td>";
-  if (sr->is_reference_station())
+  if (sr->isReferenceStation)
     d += "Ref";
   else
     d += "Sub";
@@ -367,7 +395,7 @@ the tide data are updated.</p>");
 }
 
 static void
-add_time_control (enum TC type, Dstr &text, Dstr &url, unsigned y, unsigned m, unsigned d) {
+add_time_control (enum TC type, Dstr &text, Dstr &url, unsigned y, unsigned m, unsigned d, int allow_forms) {
   text += "<p> <form method=get action=\"";
   text += url;
   text += "\">\nYear: <select name=\"y\">\n";
@@ -395,7 +423,7 @@ add_time_control (enum TC type, Dstr &text, Dstr &url, unsigned y, unsigned m, u
     text += "</select>\n";
   }
 
-  if(type == TC_ALL)
+  if(type == TC_DAY)
   {
     text += "Day: <select name=\"d\">\n";
     for (i=1; i<=31; i++) {
@@ -409,18 +437,25 @@ add_time_control (enum TC type, Dstr &text, Dstr &url, unsigned y, unsigned m, u
     text += "</select>\n";
   }
 
+  if (allow_forms) {
+    text += "Output: <select name=\"f\">\n";
+    text += "<option selected>Web page</option>\n";
+    text += "<option>iCalendar</option>\n";
+    text += "</select>\n";
+  }
+
   text += "<input type=submit value=\"Go\"></form></p>";
 }
 
 Timestamp
 parse_time_control (Dstr &filename, Dstr timezone, Settings *settings,
-Dstr &tc_out) {
+Dstr &tc_out, char &form_out, int s) {
   tc_out = (char *)NULL;
   Timestamp ret ((time_t)(time(NULL)));
+  form_out = 'h';
   int i;
   if ((i = filename.strrchr ('?')) != -1) {
-    unsigned y, m, d;
-    m = d = 1;
+    unsigned y, m=1, d=1;
     if ((sscanf (filename.ascharfrom(i), "?y=%u&m=%u&d=%u", &y, &m, &d)) >= 1) {
       char temp[80];
       sprintf (temp, "%4u-%02u-%02u 00:00", y, m, d);
@@ -428,6 +463,13 @@ Dstr &tc_out) {
       ret = Timestamp (in_iso, timezone, settings);
       tc_out = filename.ascharfrom(i);
     }
+    if ((i = filename.strstr ("&f=")) != -1) {
+      sscanf (filename.ascharfrom(i), "&f=%c", &form_out);
+      if (form_out != 'W' && form_out != 'i')
+        unimp_barf (s);
+      if (form_out == 'W')
+        form_out = 'h';
+    }
   }
   return ret;
 }
@@ -452,8 +494,9 @@ Content-type: text/html\n\
   top_p += " <br>\n";
 
   Dstr text_out, tc_out;
+  char form;
   Timestamp start_tm = parse_time_control (filename, st->timeZone,
-    context->settings, tc_out);
+    context->settings, tc_out, form, s);
   Dstr myurl ("/locations/");
   myurl += i;
   myurl += ".html";
@@ -486,32 +529,46 @@ for some locations. </p>\n";
   top_p += ".png";
   top_p += tc_out;
   top_p += "\" width=";
-  top_p += context->settings->gw;
+  top_p += (*(context->settings))["gw"].u;
   top_p += " height=";
-  top_p += context->settings->gh;
+  top_p += (*(context->settings))["gh"].u;
   top_p += "></p><p><pre>\n";
 
-  context->textMode (st, start_tm, text_out);
+  context->plainMode (st, start_tm, text_out, 't');
   top_p += text_out;
   top_p += "</pre></p>\n";
 
   top_p += "<h2> Time Control </h2>\n";
-  add_time_control (TC_ALL, top_p, myurl, y, m, d);
-  top_p += "<p> Note:  predictions for some locations do not cover the\n\
-full span of years.&nbsp; If your browser returns a blank page or a \"no data\"\n\
-error, then the predictions that you requested are not available. </p>\n";
+  add_time_control (TC_DAY, top_p, myurl, y, m, d, 0);
+  top_p += "<p> Note:&nbsp; If your browser returns a blank page or a \"no data\"\n\
+error, then the predictions that you requested are not available.</p>\n";
 
-  top_p += "<h2> Monthly Tide Calendars </h2>\n";
+  top_p += "<h2>Monthly Tide Calendars</h2>\n";
   Dstr monthurl ("/calendar/month/");
   monthurl += i;
-  monthurl += ".html";
-  add_time_control (TC_MONTH, top_p, monthurl, y, m, d);
+  monthurl += ".ics";
+  add_time_control (TC_MONTH, top_p, monthurl, y, m, d, 1);
 
-  top_p += "<h2> Yearly Tide Calendars </h2>\n";
+  top_p += "<h2>Yearly Tide Calendars</h2>\n";
   Dstr yearurl ("/calendar/year/");
   yearurl += i;
-  yearurl += ".html";
-  add_time_control (TC_YEAR, top_p, yearurl, y, m, d);
+  yearurl += ".ics";
+  add_time_control (TC_YEAR, top_p, yearurl, y, m, d, 1);
+
+  top_p += "<p>Selecting iCalendar output will cause supporting browsers to add events to your calendar.&nbsp; Before adding an entire month or year of events, please try this <a href=\"/calendar/day/";
+  top_p += i;
+  top_p += ".ics?y=";
+  top_p += y;
+  top_p += "&m=";
+  top_p += m;
+  top_p += "&d=";
+  top_p += d;
+  top_p += "&f=i\">one-day iCalendar test</a> to see how your calendar will react.&nbsp; The events that you get should have no duration and should not block out any time as \"busy\" time.</p>\n";
+
+  top_p += "<h2>About this station</h2>\n";
+  Dstr aboutbuf;
+  st->about (aboutbuf, 'h');
+  top_p += aboutbuf;
 
   add_disclaimer (top_p);
   write (s, top_p.aschar(), top_p.length());
@@ -600,12 +657,15 @@ location_page (int s, Dstr &filename) {
 static void
 calendar (int s, Dstr &filename) {
   unsigned long i;
-  int isyear = 0;
-  if (sscanf (filename.aschar(), "/calendar/month/%lu.html", &i) != 1) {
-    isyear = 1;
-    if (sscanf (filename.aschar(), "/calendar/year/%lu.html", &i) != 1) {
-      badclientlocation (filename);
-      notfound_barf (s);
+  TC whichcal = TC_YEAR;
+  if (sscanf (filename.aschar(), "/calendar/year/%lu.ics", &i) != 1) {
+    whichcal = TC_MONTH;
+    if (sscanf (filename.aschar(), "/calendar/month/%lu.ics", &i) != 1) {
+      whichcal = TC_DAY;
+      if (sscanf (filename.aschar(), "/calendar/day/%lu.ics", &i) != 1) {
+        badclientlocation (filename);
+        notfound_barf (s);
+      }
     }
   }
 
@@ -616,53 +676,75 @@ calendar (int s, Dstr &filename) {
   }
   Station *st = (*si)[i]->load (context);
 
-  Dstr top_p ("\
-HTTP/1.0 200 OK\n\
-MIME-version: 1.0\n\
-Content-type: text/html\n\
-\n\
-<html>\n\
-<head><title> ");
-  top_p += st->name;
-  top_p += "</title></head>\n<body bgcolor=\"FFFFFF\">\n";
-
   Dstr tc_out;
+  char form;
   Timestamp start_tm = parse_time_control (filename, st->timeZone,
-    context->settings, tc_out);
+    context->settings, tc_out, form, s);
   struct tm *starttm = start_tm.get_tm (st->timeZone, context->settings);
   unsigned y = starttm->tm_year + 1900;
   unsigned m = starttm->tm_mon + 1;
+  unsigned d = starttm->tm_mday;
+
+  switch (whichcal) {
+  case TC_YEAR:
+    assert (m == 1);
+  case TC_MONTH:
+    assert (d == 1);
+  case TC_DAY:
+    ;
+  }
 
   Timestamp end_tm;
   {
     char temp[80];
-    if (isyear)
-      sprintf (temp, "%4u-01-01 00:00", y);
-    else
-      sprintf (temp, "%4u-%02u-01 00:00", y, m);
+    sprintf (temp, "%4u-%02u-%02u 00:00", y, m, d);
     start_tm = Timestamp (temp, st->timeZone, context->settings);
-    if (isyear)
+    switch (whichcal) {
+    case TC_YEAR:
       sprintf (temp, "%4u-01-01 00:00", y+1);
-    else
+      break;
+    case TC_MONTH:
       sprintf (temp, "%4u-%02u-01 00:00", y, m+1);
+      break;
+    case TC_DAY:
+      sprintf (temp, "%4u-%02u-%02u 00:00", y, m, d+1);
+      break;
+    }
     end_tm = Timestamp (temp, st->timeZone, context->settings) - Interval(1);
   }
 
-  if (!zoneinfo_doesnt_suck) {
-    top_p += "<p> Warning:  this host machine is using an obsolete time zone\n\
-database.&nbsp; Summer Time (Daylight Savings Time) adjustments will not be done\n\
-for some locations. </p>\n";
+  Dstr top_p ("HTTP/1.0 200 OK\nMIME-version: 1.0\n");
+  if (form == 'h') {
+    top_p += "Content-type: text/html\n\n<html>\n<head><title>";
+    top_p += st->name;
+    top_p += "</title></head>\n<body bgcolor=\"FFFFFF\">\n";
+    if (!zoneinfo_doesnt_suck) {
+      top_p += "<p> Warning:  this host machine is using an obsolete time zone\n\
+  database.&nbsp; Summer Time (Daylight Savings Time) adjustments will not be done\n\
+  for some locations. </p>\n";
+    }
+    top_p += "<p>\n";
+  } else {
+    // There is no RFC defining the rules for HTML transport.
+    // Extrapolating from RFCs 2445, 2446, 2447.
+    top_p += "Content-Type:text/calendar; method=publish; charset=iso-8859-1; \n component=vevent\nContent-Transfer-Encoding: 8bit\n\n";
   }
 
-  top_p += "<p>\n";
   Dstr text_out;
-  context->calendarMode (st, start_tm, end_tm, text_out, 1, 0);
+  context->calendarMode (st, start_tm, end_tm, text_out, form, 0);
   top_p += text_out;
-  top_p += "</P>\n";
+  if (form == 'h') {
+    top_p += "</P>\n";
+    add_disclaimer (top_p);
+  }
 
-  add_disclaimer (top_p);
   write (s, top_p.aschar(), top_p.length());
-  end_page (s, 1);
+  if (form == 'h')
+    end_page (s, 1);
+  else {
+    close (s);
+    exit (0);
+  }
 }
 
 static void
@@ -691,14 +773,15 @@ graphimage (int s, Dstr &filename) {
   Station *st = (*si)[i]->load (context);
 
   Dstr tc_out;
+  char form;
   Timestamp start_tm = parse_time_control (filename, st->timeZone,
-    context->settings, tc_out);
+    context->settings, tc_out, form, s);
 
   Dstr top_p
     ("HTTP/1.0 200 OK\nMIME-version: 1.0\nPragma: no-cache\nContent-type: image/png\n\n");
   write (s, top_p.aschar(), top_p.length());
 
-  RGBGraph g (context->settings->gw, context->settings->gh, context->colors);
+  RGBGraph g ((*(context->settings))["gw"].u, (*(context->settings))["gh"].u, context->colors);
   g.drawTides (st, start_tm);
   png_socket = s;  // See externC.cc
   g.writeAsPNG (socket_write_data_fn);
@@ -799,7 +882,7 @@ demangle (Dstr &s, int sock) {
       i++;
     } else if (s[i] == '%') {
       char tiny[3];
-      int temp;
+      unsigned temp;
       i++;
       tiny[0] = s[i++];
       tiny[1] = s[i++];
@@ -932,6 +1015,10 @@ Disallow: /\n");
 
 static void
 handle_request (struct sockaddr *addr, int s) {
+  // It would have been kind of lame to do this in main and make every
+  // request get the same random sequence.
+  srand (time (NULL));
+
   char request[bufsize+1];
   Dstr buf, command, filename;
 
@@ -1013,43 +1100,33 @@ int main (int argc, char **argv)
     exit (0);
 
   int listen_socket, new_socket, ts;
-  unsigned short portnum = 80;
   fd_set rdset;
   FD_ZERO (&rdset);
 
   // The port number given to xttpd is not a "proper" command line
   // setting recognized by XTide and doesn't deserve to be, so
-  // if we get one we hide it to keep CommandLineSettings from
-  // blowing up.
+  // if we get one we hide it.
 
   int portnum_evade = 0;
-  if (argc >= 2) {
-    if (sscanf (argv[1], "%hu", &portnum) != 1)
-      portnum = 80;
-    else
-      portnum_evade = 1;
+
+  {
+    unsigned short portnum = 80;
+    unsigned long addr = INADDR_ANY;
+    if (argc >= 2)
+      portnum_evade = parse_addr (portnum, addr, argv[1]);
+    setup_socket (portnum, addr, listen_socket);
   }
 
-  setup_socket (portnum, listen_socket);
   dont_be_root();
   FD_SET (listen_socket, &rdset);
 
   Settings settings;
-  {
-    ConfigDefaults cd;
-    UserDefaults ud;
-    CommandLineSettings cls (
+  settings.applyUserDefaults();
+  settings.applyCommandLine (
       portnum_evade ? argc-1 : argc,
       portnum_evade ? &(argv[1]) : argv);
-    settings.supersedeBy (cd);
-    settings.supersedeBy (ud);
-    settings.supersedeBy (cls);
-  }
 
-  context = new TideContext (
-      portnum_evade ? argc-1 : argc,
-      portnum_evade ? &(argv[1]) : argv,
-      new Colors (settings), &settings);
+  context = new TideContext (new Colors (settings), &settings);
   // Build the indices.
   zi.add (context->stationIndex());
   // Avoid issuing the zoneinfo warning a million times.
