@@ -1,4 +1,4 @@
-// $Id: xxClock.cc,v 1.3 2003/01/17 17:31:00 flaterco Exp $
+// $Id: xxClock.cc,v 1.8 2004/10/22 17:43:51 flaterco Exp $
 /*  xxClock  Tide clock.
 
     Copyright (C) 1998  David Flater.
@@ -43,7 +43,7 @@ The options menu supplies the following commands:\n\
   Set Mark:  This is used to set the 'mark level' for tide predictions.\n\
   In clock mode, this only causes a line to be drawn at the mark level.\n\
   To see the times of mark transitions you must switch to graph or text\n\
-  mode.  Some subordinate stations don't have this option.\n\
+  mode.\n\
 \n\
   Convert ft<->m:  Convert feet to meters or vice-versa.\n\
 \n\
@@ -55,8 +55,8 @@ The options menu supplies the following commands:\n\
 \n\
   New Graph Window:  Open a graph window for the current location and time.\n\
 \n\
-  New Text Window:  Open a window with a text listing of tide predictions\n\
-  for the current location and time.\n\
+  New Plain Mode Window:  Open a window with a text listing of tide\n\
+  predictions for the current location and time.\n\
 \n\
   New Raw Mode Window:  Open a window with a text listing of unprocessed\n\
   numerical time stamps and tide levels for the current location and time.\n\
@@ -66,10 +66,12 @@ The options menu supplies the following commands:\n\
   current location and time.\n\
 \n\
   New Clock Window:  Duplicates the existing clock window.\n\
+\n\
+  About This Station:  Show station metadata.\n\
 \n\
   New Location Chooser:  Open new globe and location list windows to allow\n\
   selecting a new location.");
-  xtidecontext->root->newHelpBox (helpstring);
+  (void) xtidecontext->root->newHelpBox (helpstring);
 }
 
 void
@@ -148,11 +150,11 @@ xxDrawable (in_tidecontext, in_xxcontext, in_station, !nobuttonsflag)
   Dimension minxcwidth = minwidthfudge * 3 + mypopup->stringWidth
     (mypopup->defaultfontstruct, "OptionsDismiss?");
 
-  origgraphheight = curgraphheight = xtidecontext->settings->gh;
+  origgraphheight = curgraphheight = (*(xtidecontext->settings))["gh"].u;
   if (nobuttons)
-    origgraphwidth = curgraphwidth = xtidecontext->settings->cw;
+    origgraphwidth = curgraphwidth = (*(xtidecontext->settings))["cw"].u;
   else
-    origgraphwidth = curgraphwidth = max (minxcwidth, xtidecontext->settings->cw);
+    origgraphwidth = curgraphwidth = max (minxcwidth, (*(xtidecontext->settings))["cw"].u);
   graph = new xxPixmapGraph (origgraphwidth, origgraphheight, mypopup);
   graph->clockmode = 1;
   graph->drawTides (station, t, &analogangle);
