@@ -1,4 +1,4 @@
-// $Id: xxContext.cc,v 1.3 2002/11/20 17:43:25 flaterco Exp $
+// $Id: xxContext.cc,v 1.8 2004/11/07 22:23:01 flaterco Exp $
 /*  xxContext    X-Toolkit-on-a-steek.  Packaging of assorted XCrap.
 
     Copyright (C) 1998  David Flater.
@@ -57,12 +57,6 @@ base2 (unsigned short a) {
   }
   return temp;
 }
-
-// Straight out of O'Reilly volume 1, p. 202.
-static char *visual_class[] = {
-  "StaticGray", "GrayScale", "StaticColor", "PseudoColor",
-  "TrueColor", "DirectColor"};
-
 #endif
 
 // This converts the XVisualInfo masks to what we want.
@@ -113,16 +107,35 @@ void xxContext::convert_mask (unsigned long vinfomask, unsigned short &mask,
 #endif
 }
 
+// This function and static variable are used in the constructor below
+// to pass to Settings::applyXResources.
+
+static XrmDatabase database;
+int getResource (Dstr &name_in, Dstr &val_out) {
+  Dstr n1 ("xtide*"), n2 ("XTide*");
+  n1 += name_in;
+  n2 += name_in;
+  XrmValue xv;
+  char *str_type[20]; // Waste.
+  if (XrmGetResource (database, n1.aschar(), n2.aschar(), str_type, &xv)) {
+    val_out = (char *)xv.addr;
+    return 1;
+  }
+  return 0;
+}
+
+#ifdef DUALHEADDEBUG
+// Straight out of O'Reilly volume 1, p. 202.
+static char *visual_class[] = {
+  "StaticGray", "GrayScale", "StaticColor", "PseudoColor",
+  "TrueColor", "DirectColor"};
+#endif
+
 xxContext::xxContext (int &argc, char **argv, Colors *&colors, Settings *&settings) {
   XVisualInfo vinfo;
   is_realized = 0;
   is_popup = 0;
 
-  // XtAppInitialize is obsoleted in R6
-  // Anyhow, this doesn't give us a chance to choose the visual.
-  // manager = XtAppInitialize (&context, "XTide", NULL, 0, &argc,
-  //   argv, NULL, NULL, 0);
-
   // See if we can get a nice true color visual (code fragment 11-5,
   // X Toolkit Cookbook).  Note that some idiots will set the default
   // to 8 plane PseudoColor even when 24 bit color is supported, so
@@ -131,11 +144,13 @@ xxContext::xxContext (int &argc, char **argv, Colors *&colors, Settings *&settin
     XtToolkitInitialize();
     context = XtCreateApplicationContext();
 
-    // Kludge:  destroy -geometry, we don't want it.
+    // Special handling of -geometry.
+    // Concatenation of the argument is not supported here.
+    // It's not supported by standard X11 programs either.
     {
       for (int a=0; a<argc; a++)
-        if (!strncmp (argv[a], "-ge", 3))
-          strcpy (argv[a], "-XX");
+        if (!strncmp (argv[a], "-ge", 3) || !strcmp (argv[a], "-g"))
+          strcpy (argv[a], "-X");
     }
 
     display = XtOpenDisplay (context, NULL, "XTide", "XTide",
@@ -143,42 +158,55 @@ xxContext::xxContext (int &argc, char **argv, Colors *&colors, Settings *&settin
     if (!display)
       barf (CANTOPENDISPLAY);
 
+#ifdef DUALHEADDEBUG
+    fprintf (stderr, "DefaultScreen: %d\n", DefaultScreen(display));
+#endif
+
     // Find best available TrueColor visual
     depth = 0;
     {
       XVisualInfo vinfo_template;
       vinfo_template.c_class = 4;
+      vinfo_template.screen = DefaultScreen(display);
       int nitems;
-      XVisualInfo *vinfo_out = XGetVisualInfo (display, VisualClassMask,
-        &vinfo_template, &nitems);
-#ifdef SUPER_ULTRA_VERBOSE_DEBUGGING
-      cerr << "TrueColor visuals returned by XGetVisualInfo:" << endl;
+      XVisualInfo *vinfo_out = XGetVisualInfo (display,
+        VisualClassMask|VisualScreenMask, &vinfo_template, &nitems);
+#ifdef DUALHEADDEBUG
+      fprintf (stderr, "TrueColor visuals returned by XGetVisualInfo:\n");
 #endif
       if (nitems <= 0)
-#ifdef SUPER_ULTRA_VERBOSE_DEBUGGING
-        cerr << "  None!" << endl;
+#ifdef DUALHEADDEBUG
+        fprintf (stderr, "  None!\n");
 #else
         ;
 #endif
       else {
         for (int l = 0; l < nitems; l++) {
-#ifdef SUPER_ULTRA_VERBOSE_DEBUGGING
-          cerr << "  " << vinfo_out[l].depth << "-bit " <<
-          visual_class[vinfo_out[l].c_class];
+#ifdef DUALHEADDEBUG
+          fprintf (stderr, "  l = %d\n", l);
+          fprintf (stderr, "    visualid = %lu\n", vinfo_out[l].visualid);
+          fprintf (stderr, "    screen = %d\n", vinfo_out[l].screen);
+          fprintf (stderr, "    depth = %u\n", vinfo_out[l].depth);
+          fprintf (stderr, "    class = %s\n", visual_class[vinfo_out[l].c_class]);
+          fprintf (stderr, "      redmask = %6lx\n", vinfo_out[l].red_mask);
+          fprintf (stderr, "    greenmask = %6lx\n", vinfo_out[l].green_mask);
+          fprintf (stderr, "     bluemask = %6lx\n", vinfo_out[l].blue_mask);
+          fprintf (stderr, "    colormap size = %d\n", vinfo_out[l].colormap_size);
+          fprintf (stderr, "    bits per rgb = %d\n", vinfo_out[l].bits_per_rgb);
 #endif
           if (vinfo_out[l].depth < 15) {
-#ifdef SUPER_ULTRA_VERBOSE_DEBUGGING
-            cerr << " -- yeah, right.";
+#ifdef DUALHEADDEBUG
+            fprintf (stderr, "    ^^^ Depth too low\n");
 #else
             ;
 #endif
           } else if ((unsigned)vinfo_out[l].depth > depth) {
+#ifdef DUALHEADDEBUG
+            fprintf (stderr, "    ^^^ Selected\n");
+#endif
             depth = vinfo_out[l].depth;
             vinfo = vinfo_out[l];
           }
-#ifdef SUPER_ULTRA_VERBOSE_DEBUGGING
-          cerr << endl;
-#endif
         }
         XFree (vinfo_out);
       }
@@ -224,23 +252,17 @@ xxContext::xxContext (int &argc, char **argv, Colors *&colors, Settings *&settin
   if (!visual)
     visual = DefaultVisual (display, XScreenNumberOfScreen (XtScreen (manager)));
 
-#ifdef SUPER_ULTRA_VERBOSE_DEBUGGING
-  cerr << "XTide display:  " << depth << "-bit " <<
-    visual_class[visual->c_class] << endl;
+#ifdef DUALHEADDEBUG
+  fprintf (stderr, "XTide display:  %d-bit %s\n", depth,
+    visual_class[visual->c_class]);
 #endif
 
   // Get settings.
   settings = new Settings();
-  {
-    ConfigDefaults cd;
-    UserDefaults ud;
-    CommandLineSettings cls (argc, argv);
-    xxApplicationDefaults ad (XtScreenDatabase (XtScreen (manager)));
-    settings->supersedeBy (cd);
-    settings->supersedeBy (ad);
-    settings->supersedeBy (ud);
-    settings->supersedeBy (cls);
-  }
+  database = XtScreenDatabase (XtScreen (manager));
+  settings->applyXResources (getResource);
+  settings->applyUserDefaults();
+  settings->applyCommandLine (argc, argv);
 
   // Get colors.
   colors = new Colors (*settings);
@@ -249,7 +271,7 @@ xxContext::xxContext (int &argc, char **argv, Colors *&colors, Settings *&settin
   {
     int a;
     for (a=0; a<numcolors; a++)
-      pixels[a] = getColor (settings->colors[a].aschar());
+      pixels[a] = getColor ((*settings)[colorarg[a]].s.aschar());
   }
 
   // Get GCs.
@@ -368,10 +390,8 @@ xxContext::xxContext (int &argc, char **argv, Colors *&colors, Settings *&settin
   crab_pixmap = makePixmap (crab48, dvisual, dcolormap, ddepth);
   set_icon();
   // Get colors for clock icon.
-  iconpixels[0] = getColor (settings->colors[Colors::daytime].aschar(),
-    dvisual, dcolormap, 1);
-  iconpixels[1] = getColor (settings->colors[Colors::foreground].aschar(),
-    dvisual, dcolormap, 1);
+  iconpixels[0] = getColor ((*settings)["dc"].s.aschar(),dvisual,dcolormap,1);
+  iconpixels[1] = getColor ((*settings)["fg"].s.aschar(),dvisual,dcolormap,1);
   // Make GC for icon (must match visual and colormap).
   gcv.foreground = iconpixels[0];
   icon_gc = XCreateGC (display, icon_window, GCForeground, &gcv);
@@ -859,7 +879,7 @@ void
 xxContext::refresh() {
   if (!is_realized)
     return;
-  int width, height;
+  Dimension width, height;
   Arg args[2] = {
     {XtNwidth, (XtArgVal)(&width)},
     {XtNheight, (XtArgVal)(&height)}
@@ -962,14 +982,10 @@ void
 xxContext::change_settings (xxTideContext *xtidecontext, const Settings &ud) {
   delete xtidecontext->settings;
   xtidecontext->settings = new Settings();
-  {
-    ConfigDefaults cd;
-    xxApplicationDefaults ad (XtScreenDatabase (XtScreen (manager)));
-    xtidecontext->settings->supersedeBy (cd);
-    xtidecontext->settings->supersedeBy (ad);
-    xtidecontext->settings->supersedeBy (ud);
-    xtidecontext->settings->supersedeBy (*(xtidecontext->cls));
-  }
+  Settings &settings = *(xtidecontext->settings);
+  settings.applyXResources();
+  settings.apply (ud);
+  settings.applyCommandLine();
 
   // Get colors.
   delete xtidecontext->colors;
@@ -981,12 +997,10 @@ xxContext::change_settings (xxTideContext *xtidecontext, const Settings &ud) {
   {
     int a;
     for (a=0; a<numcolors; a++)
-      pixels[a] = getColor (xtidecontext->settings->colors[a].aschar());
+      pixels[a] = getColor (settings[colorarg[a]].s.aschar());
   }
-  iconpixels[0] = getColor (xtidecontext->settings->colors[Colors::daytime].aschar(),
-    dvisual, dcolormap, 1);
-  iconpixels[1] = getColor (xtidecontext->settings->colors[Colors::foreground].aschar(),
-    dvisual, dcolormap, 1);
+  iconpixels[0] = getColor (settings["dc"].s.aschar(),dvisual,dcolormap,1);
+  iconpixels[1] = getColor (settings["fg"].s.aschar(),dvisual,dcolormap,1);
 
   // Update GCs.
   XSetForeground (display, text_gc, pixels[Colors::foreground]);
