@@ -1,5 +1,5 @@
+// $Id: xxDisclaimer.cc,v 1.3 2003/01/17 17:31:00 flaterco Exp $
 /*  xxDisclaimer  Annoy the user.
-    Last modified 1998-05-03
 
     Copyright (C) 1998  David Flater.
 
@@ -21,11 +21,19 @@
 #include "xtide.hh"
 
 static char *disclaimer = "\
+NOTE.  The license and disclaimer appearing below applies to the XTide\n\
+program itself.  For information about permissions on the harmonic\n\
+constants, please read the relevant disclaimers provided by the\n\
+maintainer of those constants.\n\
+\n\
              XTide   Copyright (C) 1998 David Flater.\n\
 \n\
 This software is provided under the terms of the GNU General Public\n\
 License, either version 2 of the License, or (at your option) any later\n\
 version.\n\
+\n\
+Although the package as a whole is GPL, some individual source files\n\
+are public domain.  Consult their header comments for details.\n\
 \n\
                         NOT FOR NAVIGATION\n\
 \n\
@@ -33,18 +41,21 @@ This program is distributed in the hope that it will be useful, but\n\
 WITHOUT ANY WARRANTY; without even the implied warranty of\n\
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  The author\n\
 assumes no liability for damages arising from use of this program OR\n\
-of the 'harmonics data' that is distributed with it.  For details, see\n\
-the appended GNU General Public License.\n\
+of any 'harmonics data' that might be distributed with it.  For details,\n\
+see the appended GNU General Public License.\n\
 \n\
 (Accurate tide predictions can only be made if the 'harmonics data' for\n\
-the relevant location is good.  Unfortunately, the only way I have of\n\
-knowing when the data is bad is when someone with access to\n\
-authoritative tide predictions for the affected location tells me so.\n\
-That is why you should not use this program or its data files if\n\
-anyone or anything could come to harm as a result of an incorrect tide\n\
-prediction.  NOAA and similar agencies in other countries can provide\n\
+the relevant location is good.  Unfortunately, the only way the maintainer\n\
+of that data has of knowing when it is bad is when someone with access to\n\
+authoritative tide predictions or observations reports a problem.  You\n\
+should not use this program or any data files that might be distributed\n\
+with it if anyone or anything could come to harm as a result of an incorrect\n\
+tide prediction.  NOAA and similar agencies in other countries can provide\n\
 you with certified tide predictions if that is what you need.)\n\
 \n\
+XTide's predictions do not incorporate the effects of tropical storms, El\n\
+Nio, seismic events, continental drift, or changes in global sea level.\n\
+\n\
 ------------------------------------------------------------------------\n\
 \n\
                     GNU GENERAL PUBLIC LICENSE\n\
@@ -436,8 +447,8 @@ xxDisclaimer::xxDisclaimer (xxContext *context, xxTideContext *in_tideContext):
   };
 
   Widget viewportwidget = XtCreateManagedWidget ("", viewportWidgetClass,
-    box->manager, vpargs, 7);
-  viewport = new xxContext (box, viewportwidget);
+    container->manager, vpargs, 7);
+  viewport = new xxContext (container, viewportwidget);
 
   Widget scrollbarwidget;
   assert (scrollbarwidget = XtNameToWidget (viewportwidget, "vertical"));
@@ -455,12 +466,12 @@ xxDisclaimer::xxDisclaimer (xxContext *context, xxTideContext *in_tideContext):
   };
 
   Widget dontshowbuttonwidget = XtCreateManagedWidget (
-    "Don't show this again", toggleWidgetClass, box->manager,
+    "Don't show this again", toggleWidgetClass, container->manager,
     buttonargs, 2);
   dontshowbutton = new xxContext (mypopup, dontshowbuttonwidget);
 
   Widget dismissbuttonwidget = XtCreateManagedWidget ("Dismiss", commandWidgetClass,
-    box->manager, buttonargs, 2);
+    container->manager, buttonargs, 2);
   XtAddCallback (dismissbuttonwidget, XtNcallback, disclaimerdismissCallback, (XtPointer)this);
   dismissbutton = new xxContext (mypopup, dismissbuttonwidget);
 
