@@ -1,4 +1,4 @@
-// $Id: xxDrawable.cc,v 1.3 2003/02/11 21:25:10 flaterco Exp $
+// $Id: xxDrawable.cc,v 1.8 2004/10/22 17:43:51 flaterco Exp $
 /*  xxDrawable  Abstract class for all tide-predicting windows.
 
     Copyright (C) 1998  David Flater.
@@ -25,8 +25,8 @@ xxDrawable::xxDrawable (xxTideContext *in_tidecontext, xxContext
 xxWindow (in_tidecontext, in_xxcontext, needcontainer) {
   assert (station = in_station);
   savebutton = markbutton = helpbutton = dismissbutton =
-    optionsbutton = optionsmenu = graphbutton = textbutton =
-    rawbutton = mediumrarebutton =
+    optionsbutton = optionsmenu = graphbutton = plainbutton =
+    rawbutton = mediumrarebutton = aboutbutton =
     aspectbutton = clockbutton = chooserbutton = timestampbutton =
     unitsbutton = rootbutton = stepbutton = NULL;
 }
@@ -57,8 +57,8 @@ xxDrawable::~xxDrawable() {
     delete dismissbutton;
   if (graphbutton)
     delete graphbutton;
-  if (textbutton)
-    delete textbutton;
+  if (plainbutton)
+    delete plainbutton;
   if (rawbutton)
     delete rawbutton;
   if (mediumrarebutton)
@@ -69,6 +69,8 @@ xxDrawable::~xxDrawable() {
     delete stepbutton;
   if (clockbutton)
     delete clockbutton;
+  if (aboutbutton)
+    delete aboutbutton;
   if (chooserbutton)
     delete chooserbutton;
   if (rootbutton)
@@ -142,10 +144,10 @@ xxDrawableGraphCallback (Widget w, XtPointer client_data, XtPointer call_data) {
 }
 
 void
-xxDrawableTextCallback (Widget w, XtPointer client_data, XtPointer call_data) {
+xxDrawablePlainCallback (Widget w, XtPointer client_data, XtPointer call_data) {
   xxDrawable *d = ((xxDrawable*)client_data);
   if (d->station)
-    d->xtidecontext->root->newText (d->station->clone(), d->t);
+    d->xtidecontext->root->newPlain (d->station->clone(), d->t);
 }
 
 void
@@ -169,6 +171,13 @@ xxDrawableClockCallback (Widget w, XtPointer client_data, XtPointer call_data) {
     d->xtidecontext->root->newClock (d->station->clone());
 }
 
+void
+xxDrawableAboutCallback (Widget w, XtPointer client_data, XtPointer call_data) {
+  xxDrawable *d = ((xxDrawable*)client_data);
+  if (d->station)
+    d->xtidecontext->root->newAbout (d->station);
+}
+
 void
 xxDrawableChooserCallback (Widget w, XtPointer client_data, XtPointer call_data) {
   xxDrawable *d = ((xxDrawable*)client_data);
@@ -305,21 +314,10 @@ void xxDrawable::addNormalButtons (Widget below_this, Widget beside_this) {
     savebutton = new xxContext (mypopup, buttonwidget);
   }
   if (!(is_rare())) {
-    // While we're waiting for goto to come back into style...
-    while (station) {
-      // Mark doesn't work in the really hairy cases, so don't offer them
-      // a button.
-      if (!(station->is_reference_station())) {
-	SubordinateStation *sb = (SubordinateStation *)station;
-	if (sb->residualOffsets)
-	  break;
-      }
-      Widget buttonwidget = XtCreateManagedWidget ("Set Mark", smeBSBObjectClass,
-	optionsmenu->manager, boxbuttonargs, 4);
-      XtAddCallback (buttonwidget, XtNcallback, markCallback, (XtPointer)this);
-      markbutton = new xxContext (mypopup, buttonwidget);
-      break;
-    }
+    Widget buttonwidget = XtCreateManagedWidget ("Set Mark", smeBSBObjectClass,
+      optionsmenu->manager, boxbuttonargs, 4);
+    XtAddCallback (buttonwidget, XtNcallback, markCallback, (XtPointer)this);
+    markbutton = new xxContext (mypopup, buttonwidget);
   }
   if (station) {
     if (!(station->isCurrent)) {
@@ -354,10 +352,10 @@ void xxDrawable::addNormalButtons (Widget below_this, Widget beside_this) {
     graphbutton = new xxContext (mypopup, buttonwidget);
   }
   {
-    Widget buttonwidget = XtCreateManagedWidget ("New Text Window", smeBSBObjectClass,
+    Widget buttonwidget = XtCreateManagedWidget ("New Plain Mode Window", smeBSBObjectClass,
       optionsmenu->manager, boxbuttonargs, 4);
-    XtAddCallback (buttonwidget, XtNcallback, xxDrawableTextCallback, (XtPointer)this);
-    textbutton = new xxContext (mypopup, buttonwidget);
+    XtAddCallback (buttonwidget, XtNcallback, xxDrawablePlainCallback, (XtPointer)this);
+    plainbutton = new xxContext (mypopup, buttonwidget);
   }
   {
     Widget buttonwidget = XtCreateManagedWidget ("New Raw Mode Window", smeBSBObjectClass,
@@ -377,6 +375,12 @@ void xxDrawable::addNormalButtons (Widget below_this, Widget beside_this) {
     XtAddCallback (buttonwidget, XtNcallback, xxDrawableClockCallback, (XtPointer)this);
     clockbutton = new xxContext (mypopup, buttonwidget);
   }
+  {
+    Widget buttonwidget = XtCreateManagedWidget ("About This Station", smeBSBObjectClass,
+      optionsmenu->manager, boxbuttonargs, 4);
+    XtAddCallback (buttonwidget, XtNcallback, xxDrawableAboutCallback, (XtPointer)this);
+    aboutbutton = new xxContext (mypopup, buttonwidget);
+  }
   {
     Widget buttonwidget = XtCreateManagedWidget ("New Location Chooser", smeBSBObjectClass,
       optionsmenu->manager, boxbuttonargs, 4);
@@ -435,10 +439,11 @@ void xxDrawable::global_redraw() {
     station = delme->clone();
     delete delme;
 
-    if (!(xtidecontext->settings->u.isNull())) {
-      if (xtidecontext->settings->u != "x") {
-        PredictionValue::Unit u (xtidecontext->settings->u);
-        station->setUnits (u);
+    Configurable &u = (*(xtidecontext->settings))["u"];
+    if (!u.isnull) {
+      if (u.s != "x") {
+        PredictionValue::Unit p_u (u.s);
+        station->setUnits (p_u);
       }
     }
   }
@@ -465,14 +470,16 @@ void xxDrawable::global_redraw() {
     XtSetValues (stepbutton->manager, buttonargs, 2);
   if (graphbutton)
     XtSetValues (graphbutton->manager, buttonargs, 2);
-  if (textbutton)
-    XtSetValues (textbutton->manager, buttonargs, 2);
+  if (plainbutton)
+    XtSetValues (plainbutton->manager, buttonargs, 2);
   if (rawbutton)
     XtSetValues (rawbutton->manager, buttonargs, 2);
   if (mediumrarebutton)
     XtSetValues (mediumrarebutton->manager, buttonargs, 2);
   if (clockbutton)
     XtSetValues (clockbutton->manager, buttonargs, 2);
+  if (aboutbutton)
+    XtSetValues (aboutbutton->manager, buttonargs, 2);
   if (chooserbutton)
     XtSetValues (chooserbutton->manager, buttonargs, 2);
   if (rootbutton)
