@@ -1,4 +1,4 @@
-// $Id: xxGlobe.cc,v 1.4 2003/01/17 17:31:00 flaterco Exp $
+// $Id: xxGlobe.cc,v 1.13 2004/12/10 20:34:30 flaterco Exp $
 /*  xxGlobe   Location chooser using Orthographic Projection.
 
     There is some duplicated code between xxGlobe and xxMap.  However,
@@ -39,6 +39,13 @@ XEvent *event, Boolean *continue_dispatch)
     globe->update_position (globe->last_x, globe->last_y);
 }
 
+// Test for coordinates within reasonable boundaries to avoid
+// overflowing X11 (16-bit ints).
+int xxGlobe::inrange (long x, long y) {
+  return (x >= -32768 && x < 32768 &&
+          y >= -32768 && y < 32768);
+}
+
 // This draw_coastlines is based on Jan's redrawGlobe
 void xxGlobe::draw_coastlines () {
   float          *latray, *lonray;
@@ -46,6 +53,10 @@ void xxGlobe::draw_coastlines () {
   static char    files[6][12] = {"wvsfull.dat", "wvs250k.dat", "wvs1.dat", 
 				 "wvs3.dat", "wvs12.dat", "wvs43.dat"};
   int wvsrtv (char *, int, int, float **, float **, int **);
+  // Scales / resolutions as described in wvsrtv.cc (1:X)
+  // wvsfull is unknown but doesn't matter
+  static unsigned long scales[6] = {0, 250000, 1000000, 3000000, 12000000,
+				    43000000};
 
   // First figure out the worst case bounds.
   int startlat, endlat, startlon, endlon;
@@ -128,7 +139,7 @@ void xxGlobe::draw_coastlines () {
     // If a pole is showing, then forget longitude and just work on
     // latitude.  The most expedient way to find if a pole is showing is
     // to translate its coordinates.
-    int junkx, junky;
+    long junkx, junky;
     if (translateCoordinates (90.0, 0.0, &junkx, &junky)) // N
       startlat = max (startlat, minlat);
     else if (translateCoordinates (-90.0, 0.0, &junkx, &junky)) // S
@@ -165,20 +176,17 @@ void xxGlobe::draw_coastlines () {
   latray = (float *) NULL;
   lonray = (float *) NULL;
   segray = (int *) NULL;
-  int lat, lon, segx[2], segy[2];
+  int lat, lon;
+  long segx[2], segy[2];
 
   /* Get the proper coastline data set based on the zoom level. */
-  if (size <= min_globe_size)
-    coast = 5;
-  else if (size < min_globe_size * 16)
-    coast = 4;
-  else if (size < min_globe_size * 64)
-    coast = 2;
-  // Our max zoom is 32K -- jump directly to highest resolution
-  // else if (size < min_globe_size * 256)
-  //   coast = 1;
-  else
-    coast = 0;
+  // Your propriety may vary.
+  // Originally, 3 was skipped because Jan didn't like having it
+  // around because the file is physically larger than 2.
+  for (coast=5; coast > 0 &&
+	 (double)size * (double)scales[coast] > zoomresmagicnumber; coast--);
+
+  // fprintf (stderr, "%d %6lu %le\n", coast, size, (double)size * (double)scales[coast]);
 
   /*  Read and plot the coastlines. */
 
@@ -191,44 +199,39 @@ void xxGlobe::draw_coastlines () {
 	  &segray);
 
       if (nseg) {
-	/*  Get rid of single point islands that were required for 
-	    NAVO.  These were created during the decimation
-	    process.  */
-
-	if (nseg > 2 || latray[0] != latray[1] || 
-	  lonray[0] != lonray[1]) {
-	  for (k = 0 ; k < nseg ; k++) {
-	    int cnt, m;
-	    m = 0;
-	    for (cnt = 0 ; cnt < segray[k] ; cnt++) {
-              // Need to fix out-of-range longitudes here.  We get them
-              // because of the workaround discussed above.
-              double fixedlon = (double)lonray[offset+cnt];
-              if (fixedlon < -180.0)
-                fixedlon += 360.0;
-              else if (fixedlon > 180.0)
-                fixedlon -= 360.0;
-	      if (translateCoordinates (
-                (double)latray[offset+cnt], fixedlon, &segx[m], &segy[m])) {
-		/*  Check for the weird situation when
-		    west and east are at 0.0 and 360.0.  */
-		if (m && segx[1] - segx[0] < 50 && 
-		    segx[0] - segx[1] < 50) {
-                  // FIXME?  On sphere, this should be an arc.  (Nit)
-		  XDrawLine (picture->display, 
-		     globepixmap, picture->text_gc, 
-		     segx[0], segy[0], segx[1], 
-		     segy[1]);
-		  segx[0] = segx[1];
-		  segy[0] = segy[1];
-		}
-		m = 1;
-	      } else {
-		m = 0;
+	for (k = 0 ; k < nseg ; k++) {
+	  int cnt, m;
+	  m = 0;
+	  for (cnt = 0 ; cnt < segray[k] ; cnt++) {
+	    // Need to fix out-of-range longitudes here.  We get them
+	    // because of the workaround discussed above.
+	    double fixedlon = (double)lonray[offset+cnt];
+	    if (fixedlon < -180.0)
+	      fixedlon += 360.0;
+	    else if (fixedlon > 180.0)
+	      fixedlon -= 360.0;
+	    if (translateCoordinates (
+	      (double)latray[offset+cnt], fixedlon, &segx[m], &segy[m])) {
+	      // In xxMap, Jan did this:
+	      /*  Check for the weird situation when
+		  west and east are at 0.0 and 360.0.  */
+	      //		if (m && segx[1] - segx[0] < 50 && 
+	      //		    segx[0] - segx[1] < 50) {
+	      // Whatever that was supposed to do in xxMap, it wreaks
+	      // havoc here.  (Lines vanish when zoomed in.)
+	      if (m) {
+		// On sphere, this should be an arc.  (Nit)
+		XDrawLine (picture->display, globepixmap, picture->text_gc, 
+		  segx[0], segy[0], segx[1], segy[1]);
+		segx[0] = segx[1];
+		segy[0] = segy[1];
 	      }
+	      m = 1;
+	    } else {
+	      m = 0;
 	    }
-	    offset += segray[k];
 	  }
+	  offset += segray[k];
 	}
       }
     }
@@ -237,7 +240,7 @@ void xxGlobe::draw_coastlines () {
 }
 
 // Based on Jan's
-void xxGlobe::update_position (int rx, int ry)
+void xxGlobe::update_position (long rx, long ry)
 {
   char   string[50];
   double lat, lon;
@@ -257,10 +260,10 @@ void xxGlobe::update_position (int rx, int ry)
 void xxGlobe::draw_gridlines () {
 
   // Draw a circle around everything just for looks.
-  XDrawArc (picture->display, globepixmap, picture->text_gc,
-    -xorigin, -yorigin, size, size, 0, 360*64);
+  if (inrange (-xorigin, -yorigin) && inrange (size - xorigin, size - yorigin))
+    XDrawArc (picture->display, globepixmap, picture->text_gc,
+      -xorigin, -yorigin, size, size, 0, 360*64);
 
-  // FIXME (maybe?)
   // I was hoping to determine the arcs analytically but gave myself a
   // headache trying to figure out the longitude arcs.  The following
   // implementation uses brute force and ignorance to draw a faceted
@@ -268,14 +271,17 @@ void xxGlobe::draw_gridlines () {
   // enough.
 
   // Longitude
-  int lng, lat, lastx, lasty, newx, newy, lastvalid;
+  long lng, lat, lastx=0, lasty=0, newx, newy, lastvalid;
   for (lng = -180; lng <= 150; lng += 30) {
     lastvalid = 0;
     for (lat = -90; lat <= 90; lat += 5) {
       if (projectCoordinates (lat, lng, &newx, &newy)) {
+        // This draws farther outside the realm than TranslateCoordinates
+        // would allow.
         if (lastvalid)
-          XDrawLine (picture->display, globepixmap, picture->text_gc,
-            lastx-xorigin, lasty-yorigin, newx-xorigin, newy-yorigin);
+          if (inrange (lastx-xorigin, lasty-yorigin) && inrange (newx-xorigin, newy-yorigin))
+            XDrawLine (picture->display, globepixmap, picture->text_gc,
+              lastx-xorigin, lasty-yorigin, newx-xorigin, newy-yorigin);
         lastx = newx;
         lasty = newy;
         lastvalid = 1;
@@ -289,9 +295,12 @@ void xxGlobe::draw_gridlines () {
     lastvalid = 0;
     for (lng = -180; lng <= 180; lng += 5) {
       if (projectCoordinates (lat, lng, &newx, &newy)) {
+        // This draws farther outside the realm than TranslateCoordinates
+        // would allow.
         if (lastvalid)
-          XDrawLine (picture->display, globepixmap, picture->text_gc,
-            lastx-xorigin, lasty-yorigin, newx-xorigin, newy-yorigin);
+          if (inrange (lastx-xorigin, lasty-yorigin) && inrange (newx-xorigin, newy-yorigin))
+            XDrawLine (picture->display, globepixmap, picture->text_gc,
+              lastx-xorigin, lasty-yorigin, newx-xorigin, newy-yorigin);
         lastx = newx;
         lasty = newy;
         lastvalid = 1;
@@ -308,7 +317,7 @@ void xxGlobe::draw_gridlines () {
 //   1 = the point is on this side; results valid.
 //   0 = the point is on the other side; ignore results.
 
-int xxGlobe::projectCoordinates (double lat, double lng, int *x, int *y) {
+int xxGlobe::projectCoordinates (double lat, double lng, long *x, long *y) {
   // wvsrtv sent longitude -180.002060
   assert (lat >= -90 && lat <= 90 && lng >= -181 && lng <= 180);
 
@@ -351,7 +360,7 @@ int xxGlobe::projectCoordinates (double lat, double lng, int *x, int *y) {
   return 1;
 }
 
-int xxGlobe::projectCoordinates (Coordinates c_in, int *x, int *y) {
+int xxGlobe::projectCoordinates (Coordinates c_in, long *x, long *y) {
   if (c_in.isNull())
     return 0;
   return projectCoordinates (c_in.lat(), c_in.lng(), x, y);
@@ -364,19 +373,20 @@ int xxGlobe::projectCoordinates (Coordinates c_in, int *x, int *y) {
 //   1 = the point may be visible; draw it.
 //   0 = the point is invisible; ignore results.
 
-int xxGlobe::translateCoordinates (double lat, double lng, int *x, int *y) {
+int xxGlobe::translateCoordinates (double lat, double lng, long *x, long *y) {
   assert (lat >= -90 && lat <= 90 && lng >= -180 && lng <= 180);
   if (!projectCoordinates (lat, lng, x, y))
     return 0;
   *x -= xorigin;
   *y -= yorigin;
-  // We deliberately allow an extra row on each side.
-  if (*x >= -1 && *x <= min_globe_size+1 && *y >= -1 && *y <= min_globe_size+1)
+  // Allow a cushy margin so that lines won't get chopped.
+  if (*x >= -antichopmargin && *x <= min_globe_size+antichopmargin &&
+      *y >= -antichopmargin && *y <= min_globe_size+antichopmargin)
     return 1;
   return 0;
 }
 
-int xxGlobe::translateCoordinates (Coordinates c_in, int *x, int *y) {
+int xxGlobe::translateCoordinates (Coordinates c_in, long *x, long *y) {
   if (c_in.isNull())
     return 0;
   return translateCoordinates (c_in.lat(), c_in.lng(), x, y);
@@ -385,7 +395,7 @@ int xxGlobe::translateCoordinates (Coordinates c_in, int *x, int *y) {
 // Returns:
 //   1 = the point is on the globe somewhere; lat and lng valid.
 //   0 = you missed; ignore lat and lng.
-int xxGlobe::untranslateCoordinates (int x, int y, double &lat, double &lng) {
+int xxGlobe::untranslateCoordinates (long x, long y, double &lat, double &lng) {
 
   if (x < 0 || x > min_globe_size || y < 0 || y > min_globe_size)
     return 0;
@@ -474,11 +484,15 @@ xxGlobe::redrawGlobe () {
     halfspot = 2;
   }
   for (i=0; i<stationIndex->length(); i++) {
-    int x, y;
+    long x, y;
     if (translateCoordinates ((*stationIndex)[i]->coordinates, &x, &y)) {
-      XFillArc (picture->display, globepixmap, picture->mark_gc,
-        x-halfspot, y-halfspot, spot, spot, 0, 360*64);
-      si->add ((*stationIndex)[i]);
+      // Here, we *do* want to eliminate everything not strictly in
+      // the window, modulo the size of the dots.
+      if (x >= -halfspot && y >= -halfspot && x <= min_globe_size+halfspot && y <= min_globe_size+halfspot) {
+        XFillArc (picture->display, globepixmap, picture->mark_gc,
+          x-halfspot, y-halfspot, spot, spot, 0, 360*64);
+        si->add ((*stationIndex)[i]);
+      }
     }
   }
 
@@ -551,7 +565,7 @@ Buttons in the globe window:\n\
 Keyboard:\n\
 \n\
   Arrow keys:  rotate North/South/East/West.");
-  globe->xtidecontext->root->newHelpBox (helpstring);
+  (void) globe->xtidecontext->root->newHelpBox (helpstring);
 }
 
 void
@@ -581,16 +595,18 @@ xxGlobeZoomOutCallback (Widget w, XtPointer client_data, XtPointer call_data) {
 // This is done by the flat window image and not by spherical projection.
 // It's simpler this way.
 void
-xxGlobe::blast (int x, int y) {
+xxGlobe::blast (long x, long y) {
   if (blastflag)
-    XDrawArc (picture->display, globepixmap, picture->invert_gc, blastx,
-      blasty, blastradius*2, blastradius*2, 0, 360*64);
+    if (inrange (blastx, blasty) && inrange (blastx+blastradius*2, blasty+blastradius*2))
+      XDrawArc (picture->display, globepixmap, picture->invert_gc, blastx,
+        blasty, blastradius*2, blastradius*2, 0, 360*64);
 
   blastflag = 1;
   blastx = x - blastradius;
   blasty = y - blastradius;
-  XDrawArc (picture->display, globepixmap, picture->invert_gc, blastx,
-    blasty, blastradius*2, blastradius*2, 0, 360*64);
+  if (inrange (blastx, blasty) && inrange (blastx+blastradius*2, blasty+blastradius*2))
+    XDrawArc (picture->display, globepixmap, picture->invert_gc, blastx,
+      blasty, blastradius*2, blastradius*2, 0, 360*64);
 
   Arg args[1] = {
     {"bitmap", (XtArgVal)globepixmap}
@@ -599,12 +615,12 @@ xxGlobe::blast (int x, int y) {
   picture->refresh();
 
   // Make a list of all locations within the blast radius.
-  int bx = x + xorigin;
-  int by = y + yorigin;
+  long bx = x + xorigin;
+  long by = y + yorigin;
   unsigned long a;
   StationIndex *si = new StationIndex();
   for (a=0; a<stationIndex->length(); a++) {
-    int x, y;
+    long x, y;
     if (projectCoordinates ((*(stationIndex))[a]->coordinates, &x, &y)) {
       double dx = (double)x - (double)bx;
       double dy = (double)y - (double)by;
@@ -664,12 +680,12 @@ xxGlobeButtonPressEventHandler (Widget w, XtPointer client_data,
 
   // Coordinates relative to upper left corner of globe pixmap.
   // Need to subtract the internal margin of the label widget.
-  int rx = xbe->x - globe->internalWidth;
-  int ry = xbe->y - globe->internalHeight;
+  long rx = xbe->x - globe->internalWidth;
+  long ry = xbe->y - globe->internalHeight;
 
   // Relative to entire globe.
-  int bx = rx + globe->xorigin;
-  int by = ry + globe->yorigin;
+  long bx = rx + globe->xorigin;
+  long by = ry + globe->yorigin;
 
   if (xbe->button == Button3 ||
       (xbe->button == Button1 && globe->size == max_globe_size)) {
@@ -709,7 +725,7 @@ xxGlobeButtonPressEventHandler (Widget w, XtPointer client_data,
     StationRef *closestsr = NULL;
     double d = 17.0;
     for (a=0; a<globe->stationIndex->length(); a++) {
-      int x, y;
+      long x, y;
       if (globe->projectCoordinates ((*(globe->stationIndex))[a]->coordinates,
 	&x, &y)) {
 	double dx = (double)x - (double)bx;
@@ -826,11 +842,11 @@ xxTideContext *in_tidecontext): xxWindow (in_tidecontext, context, 1) {
   mypopup->realize();
   mypopup->fixSize();
 
-  Settings *settings = in_tidecontext->settings;
-  if (settings->gl_isnull != 0 || settings->gl == 360.0)
+  Configurable &gl = (*(in_tidecontext->settings))["gl"];
+  if (gl.isnull || gl.d == 360.0)
     center_longitude = stationIndex->bestCenterLongitude();
   else
-    center_longitude = settings->gl;
+    center_longitude = gl.d;
   center_latitude = 0.0;
 
   // Start with an empty list since it's going to get clobbered in
