@@ -1,4 +1,4 @@
-// $Id: xxGraphMode.cc,v 1.3 2003/01/17 17:31:00 flaterco Exp $
+// $Id: xxGraphMode.cc,v 1.8 2004/10/22 17:43:51 flaterco Exp $
 /*  xxGraphMode  Tide graphs in a window.
 
     Copyright (C) 1998  David Flater.
@@ -40,7 +40,7 @@ The options menu supplies the following commands:\n\
 \n\
   Set Mark:  This is used to set the 'mark level' for tide predictions.\n\
   The prediction window will show the times when the tide level crosses\n\
-  the mark.  Some subordinate stations don't have this option.\n\
+  the mark.\n\
 \n\
   Convert ft<->m:  Convert feet to meters or vice-versa.\n\
 \n\
@@ -56,8 +56,8 @@ The options menu supplies the following commands:\n\
 \n\
   New Graph Window:  Duplicates the existing graph window.\n\
 \n\
-  New Text Window:  Open a window with a text listing of tide predictions\n\
-  for the current location and time.\n\
+  New Plain Mode Window:  Open a window with a text listing of tide\n\
+  predictions for the current location and time.\n\
 \n\
   New Raw Mode Window:  Open a window with a text listing of unprocessed\n\
   numerical time stamps and tide levels for the current location and time.\n\
@@ -68,10 +68,12 @@ The options menu supplies the following commands:\n\
 \n\
   New Clock Window:  Open a window with a tide clock for the current\n\
   location.\n\
+\n\
+  About This Station:  Show station metadata.\n\
 \n\
   New Location Chooser:  Open new globe and location list windows to allow\n\
   selecting a new location.");
-  xtidecontext->root->newHelpBox (helpstring);
+  (void) xtidecontext->root->newHelpBox (helpstring);
 }
 
 void
@@ -167,8 +169,8 @@ xxGraphMode::construct () {
   Dimension minxgwidth = minwidthfudge * 5 + mypopup->stringWidth
     (mypopup->defaultfontstruct, "BackwardForwardOptionsDismiss?");
 
-  origgraphheight = curgraphheight = xtidecontext->settings->gh;
-  origgraphwidth = curgraphwidth = max (xtidecontext->settings->gw,
+  origgraphheight = curgraphheight = (*(xtidecontext->settings))["gh"].u;
+  origgraphwidth = curgraphwidth = max ((*(xtidecontext->settings))["gw"].u,
     minxgwidth);
   graph = new xxPixmapGraph (origgraphwidth, origgraphheight, mypopup);
   graph->drawTides (station, t);
