@@ -1,4 +1,4 @@
-// $Id: xxLocationList.cc,v 1.3 2003/01/17 19:14:15 flaterco Exp $
+// $Id: xxLocationList.cc,v 1.4 2004/09/09 13:33:02 flaterco Exp $
 /*  xxLocationList   Scrolling location chooser.
 
     Copyright (C) 1998  David Flater.
@@ -55,7 +55,7 @@ buttons to let you switch between them.\n\
 You can change the sort field by pulling down the Sort button.\n\
 \n\
 The location list is dismissed when the globe/map window is dismissed.");
-  ll->xtidecontext->root->newHelpBox (helpstring);
+  (void) ll->xtidecontext->root->newHelpBox (helpstring);
 }
 
 void
