@@ -1,4 +1,4 @@
-// $Id: xxMap.cc,v 1.4 2003/01/17 17:31:00 flaterco Exp $
+// $Id: xxMap.cc,v 1.8 2005/01/07 02:36:32 flaterco Exp $
 /*  xxMap   Location chooser using Cylindrical Equidistant projection.
 
     There is some duplicated code between xxGlobe and xxMap.  However,
@@ -60,8 +60,11 @@ int xxMap::translateCoordinates (double lat, double lon, int *x, int *y)
     if (bounds[zoom_level].wlon < -180.0 && lng > bounds[zoom_level].elon)
         lng -= 360.0;
 
-    if (lat < bounds[zoom_level].slat || lat > bounds[zoom_level].nlat ||
-        lng < bounds[zoom_level].wlon || lng > bounds[zoom_level].elon)
+    // Allow a cushy margin so that lines won't get chopped.
+    if (lat < bounds[zoom_level].slat - antichopmargin ||
+        lat > bounds[zoom_level].nlat + antichopmargin ||
+        lng < bounds[zoom_level].wlon - antichopmargin ||
+        lng > bounds[zoom_level].elon + antichopmargin)
         return (0);
 
     *x = internalWidth + (int) (((lng - bounds[zoom_level].wlon) / 
@@ -357,9 +360,13 @@ void xxMap::redrawMap ()
         if (translateCoordinates ((*stationIndex)[i]->coordinates.lat(),
             (*stationIndex)[i]->coordinates.lng(), &x, &y))
           {
-            XFillArc (picture->display, mappixmap, picture->mark_gc,
-               x - halfspot, y - halfspot, spot, spot, 0, 360*64);
-            si->add ((*stationIndex)[i]);
+            // Here, we *do* want to eliminate everything not strictly in
+            // the window, modulo the size of the dots.
+            if (x >= -halfspot && y >= -halfspot && x <= x_map_size+halfspot && y <= y_map_size+halfspot) {
+              XFillArc (picture->display, mappixmap, picture->mark_gc,
+                 x - halfspot, y - halfspot, spot, spot, 0, 360*64);
+              si->add ((*stationIndex)[i]);
+            }
           }
       }
 
@@ -401,8 +408,8 @@ xxMapRoundCallback (Widget w, XtPointer client_data, XtPointer call_data) {
 void xxMapHelpCallback (Widget w, XtPointer client_data, 
 XtPointer call_data) 
 {
-    xxMap *map = (xxMap *)client_data;
-    Dstr helpstring ("\
+  xxMap *map = (xxMap *)client_data;
+  Dstr helpstring ("\
 XTide Location Chooser -- Map Window\n\
 \n\
 The map window initially shows the entire world (cylindrical equidistant\n\
@@ -439,7 +446,7 @@ Buttons in the map window:\n\
 Keyboard:\n\
 \n\
   Arrow keys:  pan up/down/left/right.");
-    map->xtidecontext->root->newHelpBox (helpstring);
+  (void) map->xtidecontext->root->newHelpBox (helpstring);
 }
 
 
@@ -888,15 +895,11 @@ xxTideContext *in_tidecontext): xxWindow (in_tidecontext, context, 1)
     bounds[0].slat = -90.0;
     bounds[0].nlat = 90.0;
 
-    Settings *settings = in_tidecontext->settings;
-    if (settings->gl_isnull != 0 || settings->gl == 360.0)
-    {
-        center_longitude = stationIndex->bestCenterLongitude();
-    }
+    Configurable &gl = (*(in_tidecontext->settings))["gl"];
+    if (gl.isnull || gl.d == 360.0)
+      center_longitude = stationIndex->bestCenterLongitude();
     else
-    {
-        center_longitude = settings->gl;
-    }
+      center_longitude = gl.d;
 
     bounds[0].wlon = center_longitude - 180.0;
     bounds[0].elon = center_longitude + 180.0;
