@@ -1,5 +1,5 @@
-// $Id: xxTextMode.cc,v 1.4 2003/01/17 18:33:00 flaterco Exp $
-/*  xxTextMode  Plain old text mode, in a window.
+// $Id: xxTextMode.cc,v 1.13 2004/11/15 16:30:22 flaterco Exp $
+/*  xxTextMode  Raw/medium/plain modes, in a window.
 
     Copyright (C) 1998  David Flater.
 
@@ -29,9 +29,9 @@ void
 xxTextMode::help() {
   Dstr helpstring;
 
-  if (doneness == 2)
+  if (mode == 'p')
     helpstring = "\
-XTide Text Mode Window\n\
+XTide Plain Mode Window\n\
 \n\
 Use the forward and backward buttons to scroll the tide predictions\n\
 forward or backward in time.  You can resize the window vertically\n\
@@ -43,7 +43,7 @@ The options menu supplies the following commands:\n\
 \n\
   Set Mark:  This is used to set the 'mark level' for tide predictions.\n\
   The prediction window will show the times when the tide level crosses\n\
-  the mark.  Some subordinate stations don't have this option.\n\
+  the mark.\n\
 \n\
   Convert ft<->m:  Convert feet to meters or vice-versa.\n\
 \n\
@@ -54,7 +54,7 @@ The options menu supplies the following commands:\n\
   New Graph Window:  Open a window with a tide graph for the current\n\
   location and time.\n\
 \n\
-  New Text Window:  Duplicates the existing text window.\n\
+  New Plain Mode Window:  Duplicates the existing text window.\n\
 \n\
   New Raw Mode Window:  Open a window with a text listing of unprocessed\n\
   numerical time stamps and tide levels for the current location and time.\n\
@@ -70,7 +70,7 @@ The options menu supplies the following commands:\n\
   selecting a new location.";
   else {
 
-  if (doneness)
+  if (mode == 'm')
     helpstring = "XTide Medium Rare Mode Window\n";
   else
     helpstring = "XTide Raw Mode Window\n";
@@ -96,11 +96,11 @@ The options menu supplies the following commands:\n\
   New Graph Window:  Open a window with a tide graph for the current\n\
   location and time.\n\
 \n\
-  New Text Window:  Open a window with a text listing of tide predictions\n\
-  for the current location and time.\n\
+  New Plain Mode Window:  Open a window with a text listing of tide\n\
+  predictions for the current location and time.\n\
 \n";
 
-  if (doneness)
+  if (mode == 'm')
     helpstring += "\
   New Raw Mode Window:  Open a window with a text listing of unprocessed\n\
   numerical time stamps and tide levels for the current location and time.\n\
@@ -118,46 +118,30 @@ The options menu supplies the following commands:\n\
   helpstring += "\n\
   New Clock Window:  Open a window with a tide clock for the current\n\
   location.\n\
+\n\
+  About This Station:  Show station metadata.\n\
 \n\
   New Location Chooser:  Open new globe and location list windows to allow\n\
   selecting a new location.";
   }
-  xtidecontext->root->newHelpBox (helpstring);
-}
-
-void xxTextMode::predictexact (Station::Direction dir, struct tideline *tl) {
-  xtidecontext->predictText (station, dir, tl->tm, tl->line, tl->etype);
+  (void) xtidecontext->root->newHelpBox (helpstring);
 }
 
 void
 xxTextModeforwardCallback (Widget w, XtPointer client_data,
 XtPointer call_data) {
   xxTextMode *txt = (xxTextMode *)client_data;
-  Timestamp tm = txt->lastline->tm;
-  Station::EventType avoid_etype = txt->lastline->etype;
-
-  // Move the first one to the end
-  struct xxTextMode::tideline *tl = txt->firstline;
-  txt->firstline = tl->next;
-  txt->lastline->next = tl;
-  txt->lastline = tl;
-  tl->next = NULL;
-
-  // Change its contents
-
-  if (txt->doneness == 2) {
-    tl->tm = tm;
-    txt->predictexact (Station::forward, tl);
-    // Avoid repetition
-    if (tl->etype == avoid_etype)
-      txt->predictexact (Station::forward, tl);
-  } else {
-    tm += txt->station->step;
-    tl->tm = tm;
-    txt->xtidecontext->rareModes (txt->station, tm, tm, txt->station->step, tl->line, txt->doneness);
-    tl->line -= tl->line.length() - 1;
+  TideEventsIterator it = txt->organizer.upper_bound (txt->t.timet());
+  if (it == txt->organizer.end()) {
+    // This should only happen here if lines == 1; see draw().
+    if (txt->mode == 'p')
+      txt->station->extendRange (txt->organizer, forward, Interval(max(txt->lines,10)*DAYSECONDS/2));
+    else
+      txt->station->extendRange (txt->organizer, forward, 2*max(txt->lines,10));
+    xxTextModeforwardCallback (w, client_data, call_data);
+    return;
   }
-
+  txt->t = it->second.tm;
   txt->draw();
 }
 
@@ -165,35 +149,18 @@ void
 xxTextModebackwardCallback (Widget w, XtPointer client_data,
 XtPointer call_data) {
   xxTextMode *txt = (xxTextMode *)client_data;
-  Timestamp tm = txt->firstline->tm;
-  Station::EventType avoid_etype = txt->firstline->etype;
-
-  // Move the last one to the start
-  assert (txt->firstline != txt->lastline);
-  struct xxTextMode::tideline *tl = txt->lastline;
-  struct xxTextMode::tideline *temp2 = txt->firstline;
-  while (temp2->next != tl)
-    temp2 = temp2->next;
-  temp2->next = NULL;
-  txt->lastline = temp2;
-  tl->next = txt->firstline;
-  txt->firstline = tl;
-
-  // Change its contents
-
-  if (txt->doneness == 2) {
-    tl->tm = tm;
-    txt->predictexact (Station::backward, tl);
-    // Avoid repetition
-    if (tl->etype == avoid_etype)
-      txt->predictexact (Station::backward, tl);
-  } else {
-    tm -= txt->station->step;
-    tl->tm = tm;
-    txt->xtidecontext->rareModes (txt->station, tm, tm, txt->station->step, tl->line, txt->doneness);
-    tl->line -= tl->line.length() - 1;
+  TideEventsIterator it = txt->organizer.lower_bound (txt->t.timet());
+  assert (it != txt->organizer.end());
+  if (it == txt->organizer.begin()) {
+    if (txt->mode == 'p')
+      txt->station->extendRange (txt->organizer, backward, Interval(max(txt->lines,10)*DAYSECONDS/2));
+    else
+      txt->station->extendRange (txt->organizer, backward, 2*max(txt->lines,10));
+    xxTextModebackwardCallback (w, client_data, call_data);
+    return;
   }
-
+  it--;
+  txt->t = it->second.tm;
   txt->draw();
 }
 
@@ -232,8 +199,8 @@ void *in_ptr) {
   FILE *fp;
   if ((fp = fopen (filename.aschar(), "w"))) {
     Dstr text;
-    txt->xtidecontext->rareModes (txt->station,
-      start_tm, end_tm, txt->station->step, text, txt->doneness);
+    txt->xtidecontext->rareModes (txt->station, start_tm, end_tm, text,
+      txt->mode, 't');
     fprintf (fp, "%s", text.aschar());
     fclose (fp);
   } else {
@@ -251,8 +218,8 @@ xxTextModeSaveCallback (Dstr &filename, void *in_ptr) {
   FILE *fp;
   if ((fp = fopen (filename.aschar(), "w"))) {
     Dstr text;
-    txt->xtidecontext->textMode (txt->station,
-      txt->t, txt->lastline->tm, text);
+    txt->xtidecontext->plainMode (txt->station, txt->t-Interval(60),
+      txt->last_tm+Interval(60), text, 't');
     fprintf (fp, "%s", text.aschar());
     fclose (fp);
   } else {
@@ -266,24 +233,24 @@ xxTextModeSaveCallback (Dstr &filename, void *in_ptr) {
 
 void
 xxTextMode::save () {
-  if (doneness == 2)
+  if (mode == 'p')
     (void) new xxFilename (xtidecontext, mypopup, xxTextModeSaveCallback, this, "tides.txt");
   else
-    (void) new xxRareModeSavePrompts (xtidecontext, mypopup, xxRareModeSaveCallback, this, "tides.txt", firstline->tm, lastline->tm, station->timeZone);
+    (void) new xxRareModeSavePrompts (xtidecontext, mypopup, xxRareModeSaveCallback, this, "tides.txt", t, last_tm+station->step, station->timeZone);
 }
 
 xxTextMode::xxTextMode (xxTideContext *in_tidecontext,
-xxContext *in_xxcontext, Station *in_station, int doneness_in): xxDrawable (in_tidecontext,
+xxContext *in_xxcontext, Station *in_station, char mode_in): xxDrawable (in_tidecontext,
 in_xxcontext, in_station, 2) {
-  doneness = doneness_in;
+  mode = mode_in;
   t = Timestamp((time_t)(time(NULL)));
   construct();
 }
 
 xxTextMode::xxTextMode (xxTideContext *in_tidecontext,
-xxContext *in_xxcontext, Station *in_station, int doneness_in, Timestamp t_in):
+xxContext *in_xxcontext, Station *in_station, char mode_in, Timestamp t_in):
 xxDrawable (in_tidecontext, in_xxcontext, in_station, 2) {
-  doneness = doneness_in;
+  mode = mode_in;
   t = t_in;
   construct();
 }
@@ -291,14 +258,14 @@ xxDrawable (in_tidecontext, in_xxcontext, in_station, 2) {
 void xxTextMode::construct () {
   lines = origtextmodelines;
   Dstr title (station->name);
-  switch (doneness) {
-  case 0:
+  switch (mode) {
+  case 'r':
     title += " (Raw)";
     break;
-  case 1:
+  case 'm':
     title += " (Medium Rare)";
     break;
-  case 2:
+  case 'p':
     title += " (Text)";
     break;
   default:
@@ -371,7 +338,6 @@ void xxTextMode::construct () {
 
   addNormalButtons (label->manager, forwardbutton->manager);
 
-  firstline = lastline = NULL;
   redraw();
   mypopup->realize();
   mypopup->widthNoSmaller();
@@ -401,32 +367,12 @@ void xxTextMode::construct () {
 }
 
 void xxTextMode::redraw() {
-  Timestamp tm = t;
-
-  clearbuf();
-
-  unsigned a;
-  for (a=0; a<lines; a++) {
-    struct tideline *tl = new tideline;
-    tl->tm = tm;
-
-    if (doneness == 2) {
-      predictexact (Station::forward, tl);
-      tm = tl->tm;
-    } else {
-      xtidecontext->rareModes (station, tm, tm, station->step, tl->line, doneness);
-      tl->line -= tl->line.length() - 1;
-      tm += station->step;
-    }
-
-    tl->next = NULL;
-    if (!a) {
-      firstline = lastline = tl;
-    } else {
-      lastline->next = tl;
-      lastline = tl;
-    }
-  }
+  organizer.clear();
+  if (mode == 'p')
+    station->predictTideEvents (t-Interval(60), t+Interval(max(lines,10)*DAYSECONDS/2),
+				     organizer);
+  else
+    station->predictRawEvents (t, t+((station->step)*(max(lines,10)*2)), organizer);
 
   // Was having major trouble getting the label widget to resize itself
   // reliably.  This seems to help.
@@ -444,36 +390,35 @@ void xxTextMode::redraw() {
 }
 
 void xxTextMode::draw () {
-  Dstr tidetext;
-  struct tideline *tl = firstline;
-  while (tl) {
-    tidetext += tl->line;
-    tl = tl->next;
-    if (tl)
-      tidetext += "\n";
+  TideEventsIterator it = organizer.lower_bound (t.timet());
+  Dstr buf;
+  if (it != organizer.end())
+    t = it->second.tm;
+  for (unsigned a=0; a<lines; a++) {
+    if (it == organizer.end()) {
+      // You may think it's the end, but it's not.
+      if (mode == 'p')
+        station->extendRange (organizer, forward, Interval(max(lines,10)*DAYSECONDS/2));
+      else
+        station->extendRange (organizer, forward, 2*max(lines,10));
+      draw();
+      return;
+    } else {
+      Dstr line;
+      it->second.print (line, 't', mode, *station);
+      if (!(buf.isNull()))
+        buf += '\n';
+      buf += line;
+      last_tm = it->second.tm;
+      it++;
+    }
   }
   {
     Arg labelargs[1] =  {
-      {XtNlabel, (XtArgVal)tidetext.aschar()}
+      {XtNlabel, (XtArgVal)buf.aschar()}
     };
     XtSetValues (label->manager, labelargs, 1);
   }
-
-  // Keep start time updated.
-  if (doneness == 2)
-    t = firstline->tm - Interval(60);
-  else
-    t = firstline->tm;
-}
-
-void
-xxTextMode::clearbuf() {
-  while (firstline) {
-    struct tideline *tl = firstline->next;
-    delete firstline;
-    firstline = tl;
-  }
-  firstline = lastline = NULL;
 }
 
 xxTextMode::~xxTextMode() {
@@ -482,7 +427,6 @@ xxTextMode::~xxTextMode() {
   delete namelabel;
   delete backwardbutton;
   delete forwardbutton;
-  clearbuf();
 }
 
 void xxTextMode::global_redraw() {
@@ -507,7 +451,7 @@ void xxTextMode::global_redraw() {
 }
 
 int xxTextMode::is_rare() {
-  if (doneness < 2)
-    return 1;
-  return 0;
+  if (mode == 'p')
+    return 0;
+  return 1;
 }
