@@ -1,4 +1,4 @@
-// $Id: xxTimestamp.cc,v 1.2 2003/01/17 17:31:00 flaterco Exp $
+// $Id: xxTimestamp.cc,v 1.3 2004/04/19 20:16:56 flaterco Exp $
 /*  xxTimestamp  Get a Timestamp from the user.
 
     Copyright (C) 1998  David Flater.
@@ -30,7 +30,7 @@ xxTimestampCallback (Widget w, XtPointer client_data, XtPointer call_data) {
     Dstr details ("The offending input was ");
     details += time_string;
     details += "\nin the time zone ";
-    if (e->xtidecontext->settings->z == "n")
+    if ((*(e->xtidecontext->settings))["z"].c == 'n')
       details += e->timezone;
     else
       details += "UTC0";
@@ -100,7 +100,7 @@ xxWindow (in_xtidecontext, context, 1, XtGrabExclusive) {
     Dstr f;
     f = "Adjust year, month, day, hours, and minutes to desired time in time\n\
 zone ";
-    if (in_xtidecontext->settings->z == "n")
+    if ((*(in_xtidecontext->settings))["z"].c == 'n')
       f += in_timezone;
     else
       f += "UTC0";
