@@ -1,4 +1,4 @@
-// $Id: xxWindow.cc,v 1.3 2003/01/17 17:31:00 flaterco Exp $
+// $Id: xxWindow.cc,v 1.5 2004/04/08 14:25:04 flaterco Exp $
 /*  xxWindow  Abstract class for all XTide windows implementing dismiss().
 
     Copyright (C) 1998  David Flater.
@@ -68,7 +68,7 @@ void xxWindowCloseHandler (Widget w, XtPointer client_data,
       XClientMessageEvent *ev = (XClientMessageEvent *) event;
       // Window manager close.
       if (ev->message_type == f->mypopup->protocol_atom &&
-	  ev->data.l[0] == f->mypopup->kill_atom)
+	  ev->data.l[0] == (int)(f->mypopup->kill_atom))
         f->dismiss();
     }
     break;
@@ -85,6 +85,29 @@ xxWindow::~xxWindow() {
   delete mypopup;
 }
 
+void xxWindow::move (Dstr &geometry) {
+  int xoff, yoff, negx=0, negy=0;
+  if (sscanf (geometry.aschar(), "%d%d", &xoff, &yoff) == 2) {
+    if (xoff < 0 || (xoff == 0 && geometry[0] == '-'))
+      negx = 1;
+    if (yoff < 0 || (yoff == 0 && geometry.strrchr('-') > 0))
+      negy = 1;
+    if (negx || negy) {
+      Dimension width, height;
+      Arg args[2] = {
+	{XtNwidth, (XtArgVal)(&width)},
+	{XtNheight, (XtArgVal)(&height)}
+      };
+      XtGetValues (mypopup->manager, args, 2);
+      if (negx)
+	xoff += (int)(mypopup->screen_width) - width;
+      if (negy)
+	yoff += (int)(mypopup->screen_height) - height;
+    }
+    XMoveWindow (mypopup->display, XtWindow(mypopup->manager), xoff, yoff);
+  }
+}
+
 void xxWindow::global_redraw() {
   Arg args[2] =  {
     {XtNbackground, (XtArgVal)mypopup->pixels[Colors::background]},
