@@ -1,4 +1,4 @@
-// $Id: xxXTideRoot.cc,v 1.7 2003/02/04 21:26:48 flaterco Exp $
+// $Id: xxXTideRoot.cc,v 1.17 2004/09/10 15:16:31 flaterco Exp $
 /*  xxXTideRoot  XTide "root" window
 
     Copyright (C) 1998  David Flater.
@@ -31,9 +31,7 @@ applyCallback (Widget w, XtPointer client_data, XtPointer call_data) {
 }
 
 // This code is pretty much duplicated from xxAspect
-int
-xxXTideRoot::sancheckdouble (char *d) {
-  Dstr num (d);
+int xxXTideRoot::sancheckdouble (Dstr &num) {
   if (num.length() > 0) {
     if (num.strchr ('\n') != -1 ||
            num.strchr ('\r') != -1 ||
@@ -66,233 +64,143 @@ xxXTideRoot::sancheckdouble (char *d) {
   return 1;
 }
 
-void
-xxXTideRoot::apply (int saveflag) {
+void xxXTideRoot::apply (int saveflag) {
 
-  // First sanity check, then apply if OK.
+  // Build new settings for ~/.xtide.xml
+  Settings ud;
+  ud.nullify();
 
-  // Sanity check all colors
-  int a;
-  for (a=0; a<numcolors; a++) {
-    unsigned char r, g, b;
-    Dstr color (colordiags[a]->val());
-    if (color.length()) {
-      if (!(myTideContext->colors->parseColor (color, r, g, b, 0)))
-        return;
+  ConfigurablesIterator it = ud.begin();
+  while (it != ud.end()) {
+    Configurable &cfbl = it->second;
+    if (cfbl.kind == Configurable::Ksetting) {
+      void *vd = dialogs[it->first];
+      assert (vd);
+      switch (cfbl.interpretation) {
+      case Configurable::Iboolean:
+        {
+          xxMultiChoice &mc = *(xxMultiChoice*)vd;
+          switch (mc.choice()) {
+          case 0:
+            cfbl.isnull = 0;
+            cfbl.c = 'y';
+            break;
+          case 1:
+            cfbl.isnull = 0;
+            cfbl.c = 'n';
+            break;
+          case 2:
+            break;
+          default:
+            assert (0);
+          }
+        }
+	break;
+      case Configurable::Igldouble:
+        {
+          xxMultiChoice &mc = *(xxMultiChoice*)vd;
+	  switch (mc.choice()) {
+	  case 12:
+            cfbl.isnull = 0;
+            cfbl.d = 360.0;
+	    break;
+	  case 13:
+	    break;
+	  default:
+	    cfbl.isnull = 0;
+	    cfbl.d = (double)(mc.choice()) * 30.0 - 180.0;
+	    break;
+	  }
+        }
+	break;
+      case Configurable::Iunit:
+        {
+          xxMultiChoice &mc = *(xxMultiChoice*)vd;
+	  switch (mc.choice()) {
+	  case 0:
+            cfbl.isnull = 0;
+	    cfbl.s = "ft";
+	    break;
+	  case 1:
+            cfbl.isnull = 0;
+	    cfbl.s = "m";
+	    break;
+	  case 2:
+            cfbl.isnull = 0;
+	    cfbl.s = "x";
+	    break;
+	  case 3:
+	    break;
+	  default:
+	    assert (0);
+	  }
+        }
+	break;
+      case Configurable::Iposint:
+        {
+          xxUnsignedChooser &uc = *(xxUnsignedChooser*)vd;
+          if (uc.choice() != 1000000) {
+            cfbl.isnull = 0;
+            cfbl.u = uc.choice();
+          }
+        }
+	break;
+      case Configurable::Iposdouble:
+        {
+          Dstr dbl (((xxHorizDialog*)vd)->val());
+          // Sanity check double.  (A little more user-friendly than
+          // the checking in Settings)
+          if (!sancheckdouble (dbl))
+            return;
+          if (dbl.length()) {
+            cfbl.isnull = 0;
+            assert (sscanf (dbl.aschar(), "%lf", &cfbl.d) == 1);
+          }
+        }
+	break;
+      case Configurable::Icolor:
+        {
+          Dstr color (((xxHorizDialog*)vd)->val());
+          if (color.length()) {
+            // Sanity check color
+            unsigned char r, g, b;
+            if (!(myTideContext->colors->parseColor (color, r, g, b, 0)))
+              return;
+            cfbl.isnull = 0;
+            cfbl.s = color;
+          }
+        }
+        break;
+      case Configurable::Istrftime:
+        {
+          Dstr sft (((xxHorizDialog*)vd)->val());
+          if (sft.length()) {
+            // Sanity check strftime format string
+            if (sft.strchr('"') != -1 || sft.strchr('>') != -1 ||
+                sft.strchr('<') != -1) {
+                  Dstr details ("Please don't use nasty characters like \", >, and < in your date/time formats.");
+                  barf (XMLPARSE, details, 0);
+                  return;
+	    }
+            cfbl.isnull = 0;
+            cfbl.s = sft;
+          }
+        }
+	break;
+      default:
+	assert (0);
+      }
     }
-  }
-  // Sanity check date/time formats -- prevent nasty XML characters
-#define datetimebarf { \
-    Dstr details ("Please don't use nasty characters like \", >, and < \
-in your date/time formats."); \
-    barf (XMLPARSE, details, 0); \
-    return; \
-  }
-  if (strchr (dfdiag->val(), '"') ||
-      strchr (dfdiag->val(), '>') ||
-      strchr (dfdiag->val(), '<'))
-    datetimebarf;
-  if (strchr (hfdiag->val(), '"') ||
-      strchr (hfdiag->val(), '>') ||
-      strchr (hfdiag->val(), '<'))
-    datetimebarf;
-  if (strchr (tfdiag->val(), '"') ||
-      strchr (tfdiag->val(), '>') ||
-      strchr (tfdiag->val(), '<'))
-    datetimebarf;
-  // Sanity check doubles.
-  if (!(sancheckdouble (gadiag->val())))
-    return;
-  if (!(sancheckdouble (lwdiag->val())))
-    return;
-
-  // Apply changes.
-  Settings newset;
-  // Colors
-  for (a=0; a<numcolors; a++) {
-    Dstr color (colordiags[a]->val());
-    if (color.length())
-      newset.colors[a] = color;
-  }
-  // Toggles
-  switch (elchoice->choice()) {
-  case 0:
-    newset.el = "y";
-    break;
-  case 1:
-    newset.el = "n";
-    break;
-  case 2:
-    break;
-  default:
-    assert (0);
-  }
-  switch (fechoice->choice()) {
-  case 0:
-    newset.fe = "y";
-    break;
-  case 1:
-    newset.fe = "n";
-    break;
-  case 2:
-    break;
-  default:
-    assert (0);
-  }
-  switch (nfchoice->choice()) {
-  case 0:
-    newset.nf = "y";
-    break;
-  case 1:
-    newset.nf = "n";
-    break;
-  case 2:
-    break;
-  default:
-    assert (0);
-  }
-  switch (nschoice->choice()) {
-  case 0:
-    newset.ns = "y";
-    break;
-  case 1:
-    newset.ns = "n";
-    break;
-  case 2:
-    break;
-  default:
-    assert (0);
-  }
-  switch (inchoice->choice()) {
-  case 0:
-    newset.in = "y";
-    break;
-  case 1:
-    newset.in = "n";
-    break;
-  case 2:
-    break;
-  default:
-    assert (0);
-  }
-  switch (tlchoice->choice()) {
-  case 0:
-    newset.tl = "y";
-    break;
-  case 1:
-    newset.tl = "n";
-    break;
-  case 2:
-    break;
-  default:
-    assert (0);
-  }
-  switch (zchoice->choice()) {
-  case 0:
-    newset.z = "y";
-    break;
-  case 1:
-    newset.z = "n";
-    break;
-  case 2:
-    break;
-  default:
-    assert (0);
+    it++;
   }
 
-  switch (uchoice->choice()) {
-  case 0:
-    newset.u = "ft";
-    break;
-  case 1:
-    newset.u = "m";
-    break;
-  case 2:
-    newset.u = "x";
-    break;
-  case 3:
-    break;
-  default:
-    assert (0);
-  }
-
-  switch (glchoice->choice()) {
-  case 12:
-    newset.gl_isnull = 0;
-    newset.gl = 360.0;
-    break;
-  case 13:
-    break;
-  default:
-    newset.gl_isnull = 0;
-    newset.gl = (double)(glchoice->choice()) * 30.0 - 180.0;
-    break;
-  }
-
-  // Date/time formats.
-  {
-    Dstr fmt (dfdiag->val());
-    if (fmt.length())
-      newset.df = fmt;
-  }
-  {
-    Dstr fmt (hfdiag->val());
-    if (fmt.length())
-      newset.hf = fmt;
-  }
-  {
-    Dstr fmt (tfdiag->val());
-    if (fmt.length())
-      newset.tf = fmt;
-  }
-
-  // Unsigned numbers.
-  if (gwchoice->choice() != 1000000) {
-    newset.gw_isnull = 0;
-    newset.gw = gwchoice->choice();
-  }
-  if (ghchoice->choice() != 1000000) {
-    newset.gh_isnull = 0;
-    newset.gh = ghchoice->choice();
-  }
-  if (twchoice->choice() != 1000000) {
-    newset.tw_isnull = 0;
-    newset.tw = twchoice->choice();
-  }
-  if (thchoice->choice() != 1000000) {
-    newset.th_isnull = 0;
-    newset.th = thchoice->choice();
-  }
-  if (cwchoice->choice() != 1000000) {
-    newset.cw_isnull = 0;
-    newset.cw = cwchoice->choice();
-  }
-
-  // Doubles.
-  {
-    Dstr num (gadiag->val());
-    if (num.length() > 0) {
-      double d;
-      assert (sscanf (num.aschar(), "%lf", &d) == 1);
-      newset.ga_isnull = 0;
-      newset.ga = d;
-    }
-  }
-  {
-    Dstr num (lwdiag->val());
-    if (num.length() > 0) {
-      double d;
-      assert (sscanf (num.aschar(), "%lf", &d) == 1);
-      newset.lw_isnull = 0;
-      newset.lw = d;
-    }
-  }
-
-  mycontext->change_settings (myTideContext, newset);
+  mycontext->change_settings (myTideContext, ud);
   global_redraw();
 
+  // Provided that the world did not explode as a result of applying those
+  // settings, only then might they be saved.
   if (saveflag)
-    newset.save();
+    ud.save();
 }
 
 static void
@@ -339,7 +247,7 @@ of AM/PM:\n\
    Hour format  %H\n\
    Time format  %H:%M %Z");
   xxXTideRoot *f = (xxXTideRoot *)client_data;
-  f->newHelpBox (helpstring);
+  (void) f->newHelpBox (helpstring);
 }
 
 void RootCloseHandler (Widget w, XtPointer client_data,
@@ -351,7 +259,7 @@ void RootCloseHandler (Widget w, XtPointer client_data,
       XClientMessageEvent *ev = (XClientMessageEvent *) event;
       // Window manager close.
       if (ev->message_type == f->mycontext->protocol_atom &&
-	  ev->data.l[0] == f->mycontext->kill_atom)
+	  ev->data.l[0] == (int)(f->mycontext->kill_atom))
         f->hide();
     }
     break;
@@ -410,8 +318,7 @@ xxXTideRoot::xxXTideRoot (int in_argc, char **in_argv) {
   Settings *settings;
   mycontext = new xxContext (in_argc, in_argv, colors, settings);
   mycontext->setTitle ("Control Panel");
-  myTideContext = new xxTideContext (in_argc, in_argv, mycontext, this,
-    colors, settings);
+  myTideContext = new xxTideContext (mycontext, this, colors, settings);
   set_error_context (mycontext, myTideContext);
 
   Arg args[2] =  {
@@ -484,176 +391,90 @@ xxXTideRoot::xxXTideRoot (int in_argc, char **in_argv) {
     viewbox = new xxContext (viewport, boxwidget);
   }
 
-  // Get current settings.  (Another duplication...)
-  ud = new UserDefaults();
+  // Get current settings.
+  Settings ud;
+  ud.nullify();
+  ud.applyUserDefaults ();
 
-  // Add dialogs for all of the colors
-  int a;
-  for (a=0; a<numcolors; a++)
-    colordiags[a] = new xxHorizDialog (viewbox, colordesc[a],
-      ud->colors[a].aschar());
-
-  // Add toggles
   static char *togglechoices[] = {"Yes", "No", "(blank)", NULL};
-  {
-    Dstr t = ud->el;
-    unsigned firstchoice;
-    if (t.isNull())
-      firstchoice = 2;
-    else if (t == "n")
-      firstchoice = 1;
-    else
-      firstchoice = 0;
-    elchoice = new xxMultiChoice (viewbox,
-      "Draw datum and Mean Tide Level lines in tide graphs?", togglechoices, firstchoice);
-  }
-  {
-    Dstr t = ud->fe;
-    unsigned firstchoice;
-    if (t.isNull())
-      firstchoice = 2;
-    else if (t == "n")
-      firstchoice = 1;
-    else
-      firstchoice = 0;
-    fechoice = new xxMultiChoice (viewbox,
-      "Prefer flat map to round globe location chooser?", togglechoices, firstchoice);
-  }
-  {
-    Dstr t = ud->nf;
-    unsigned firstchoice;
-    if (t.isNull())
-      firstchoice = 2;
-    else if (t == "n")
-      firstchoice = 1;
-    else
-      firstchoice = 0;
-    nfchoice = new xxMultiChoice (viewbox,
-      "Draw tide graphs as line graphs?", togglechoices, firstchoice);
-  }
-  {
-    Dstr t = ud->ns;
-    unsigned firstchoice;
-    if (t.isNull())
-      firstchoice = 2;
-    else if (t == "n")
-      firstchoice = 1;
-    else
-      firstchoice = 0;
-    nschoice = new xxMultiChoice (viewbox,
-      "Suppress sun and moon events in output?", togglechoices, firstchoice);
-  }
-  {
-    Dstr t = ud->in;
-    unsigned firstchoice;
-    if (t.isNull())
-      firstchoice = 2;
-    else if (t == "n")
-      firstchoice = 1;
-    else
-      firstchoice = 0;
-    inchoice = new xxMultiChoice (viewbox,
-      "Use inferred constituents (expert only)?", togglechoices, firstchoice);
-  }
-  {
-    Dstr t = ud->tl;
-    unsigned firstchoice;
-    if (t.isNull())
-      firstchoice = 2;
-    else if (t == "n")
-      firstchoice = 1;
-    else
-      firstchoice = 0;
-    tlchoice = new xxMultiChoice (viewbox,
-      "Draw depth lines on top of tide graph?", togglechoices, firstchoice);
-  }
-  {
-    Dstr t = ud->z;
-    unsigned firstchoice;
-    if (t.isNull())
-      firstchoice = 2;
-    else if (t == "n")
-      firstchoice = 1;
-    else
-      firstchoice = 0;
-    zchoice = new xxMultiChoice (viewbox,
-      "Coerce all time zones to UTC?", togglechoices, firstchoice);
-  }
-
   static char *unitschoices[] = {"Feet", "Meters", "No preference",
     "(blank)", NULL};
-  {
-    Dstr t = ud->u;
-    unsigned firstchoice;
-    if (t.isNull())
-      firstchoice = 3;
-    else if (t == "ft")
-      firstchoice = 0;
-    else if (t == "m")
-      firstchoice = 1;
-    else
-      firstchoice = 2;
-    uchoice = new xxMultiChoice (viewbox,
-      "Preferred units of length:", unitschoices, firstchoice);
-  }
-
   static char *glchoices[] = {"180", "150 W", "120 W", "90 W", "60 W",
     "30 W", "0", "30 E", "60 E", "90 E", "120 E", "150 E",
     "Max stations", "(blank)", NULL};
-  {
-    unsigned firstchoice = 13;
-    if (!(ud->gl_isnull)) {
-      if (ud->gl == 360.0)
-        firstchoice = 12;
-      else
-        firstchoice = (unsigned) ((ud->gl + 180.0) / 30.0);
+
+  // Create all of the dialogs.
+  ConfigurablesIterator it = ud.begin();
+  while (it != ud.end()) {
+    Configurable &cfbl = it->second;
+    if (cfbl.kind == Configurable::Ksetting) {
+      switch (cfbl.interpretation) {
+      case Configurable::Iboolean:
+        {
+          unsigned firstchoice;
+          if (cfbl.isnull)
+            firstchoice = 2;
+          else if (cfbl.c == 'n')
+            firstchoice = 1;
+          else
+            firstchoice = 0;
+          dialogs[it->first] = new xxMultiChoice (viewbox,
+	    cfbl.caption.aschar(), togglechoices, firstchoice);
+        }
+        break;
+      case Configurable::Iposint:
+        dialogs[it->first] = new xxUnsignedChooser (viewbox,
+          cfbl.caption.aschar(), cfbl.u, cfbl.isnull, cfbl.min_value);
+        break;
+      case Configurable::Iposdouble:
+        {
+          char temp[80];
+	  if (cfbl.isnull)
+	    strcpy (temp, "");
+	  else
+	    sprintf (temp, "%f", cfbl.d);
+          dialogs[it->first] = new xxHorizDialog (viewbox,
+	    cfbl.caption.aschar(), temp);
+        }
+        break;
+      case Configurable::Igldouble:
+        {
+          unsigned firstchoice = 13;
+          if (!cfbl.isnull) {
+            if (cfbl.d == 360.0)
+              firstchoice = 12;
+            else
+              firstchoice = (unsigned) ((cfbl.d + 180.0) / 30.0);
+          }
+          dialogs[it->first] = new xxMultiChoice (viewbox,
+	    cfbl.caption.aschar(), glchoices, firstchoice);
+        }
+        break;
+      case Configurable::Icolor: // deliberate fall through
+      case Configurable::Istrftime:
+        dialogs[it->first] = new xxHorizDialog (viewbox,
+  	  cfbl.caption.aschar(), cfbl.isnull ? "" : cfbl.s.aschar());
+        break;
+      case Configurable::Iunit:
+        {
+          unsigned firstchoice;
+          if (cfbl.isnull)
+            firstchoice = 3;
+          else if (cfbl.s == "ft")
+            firstchoice = 0;
+          else if (cfbl.s == "m")
+            firstchoice = 1;
+          else
+            firstchoice = 2;
+          dialogs[it->first] = new xxMultiChoice (viewbox,
+            cfbl.caption.aschar(), unitschoices, firstchoice);
+        }
+        break;
+      default:
+        assert (0);
+      }
     }
-    glchoice = new xxMultiChoice (viewbox,
-      "Default center longitude for globe:", glchoices, firstchoice);
-  }
-
-  // Date/time format dialogs
-  dfdiag = new xxHorizDialog (viewbox,
-    "Strftime style format string for printing dates.", ud->df.aschar());
-  hfdiag = new xxHorizDialog (viewbox,
-    "Strftime style format string for printing hour labels on time axis.",
-    ud->hf.aschar());
-  tfdiag = new xxHorizDialog (viewbox,
-    "Strftime style format string for printing times.", ud->tf.aschar());
-
-  // Unsigned numbers.
-  gwchoice = new xxUnsignedChooser (viewbox,
-    "Default width for tide graphs (pixels):", ud->gw, ud->gw_isnull, mingwidth);
-  ghchoice = new xxUnsignedChooser (viewbox,
-    "Default height for tide graphs and clocks (pixels):", ud->gh, ud->gh_isnull,
-    mingheight);
-  cwchoice = new xxUnsignedChooser (viewbox,
-    "Default width for tide clocks (pixels):", ud->cw, ud->cw_isnull, mingwidth);
-  twchoice = new xxUnsignedChooser (viewbox,
-    "Default width of ASCII graphs and banners (characters):", ud->tw,
-     ud->tw_isnull, minttywidth);
-  thchoice = new xxUnsignedChooser (viewbox,
-    "Default height of ASCII graphs (characters):", ud->th, ud->th_isnull,
-    minttyheight);
-
-  // Doubles.
-  {
-    char temp[80];
-    if (ud->ga_isnull)
-      strcpy (temp, "");
-    else
-      sprintf (temp, "%f", ud->ga);
-    gadiag = new xxHorizDialog (viewbox,
-      "Default aspect for tide graphs.", temp);
-  }
-  {
-    char temp[80];
-    if (ud->lw_isnull)
-      strcpy (temp, "");
-    else
-      sprintf (temp, "%f", ud->lw);
-    lwdiag = new xxHorizDialog (viewbox,
-      "Width for lines in line graphs (pixels, pos. real number).", temp);
+    it++;
   }
 
   {
@@ -746,31 +567,28 @@ void xxXTideRoot::global_redraw() {
     tempc = tempc->next;
   }
 
-  int a;
-  for (a=0; a<numcolors; a++)
-    colordiags[a]->global_redraw();
-
-  elchoice->global_redraw();
-  fechoice->global_redraw();
-  nfchoice->global_redraw();
-  nschoice->global_redraw();
-  inchoice->global_redraw();
-  tlchoice->global_redraw();
-  zchoice->global_redraw();
-  uchoice->global_redraw();
-
-  dfdiag->global_redraw();
-  hfdiag->global_redraw();
-  tfdiag->global_redraw();
-  gwchoice->global_redraw();
-  glchoice->global_redraw();
-  ghchoice->global_redraw();
-  cwchoice->global_redraw();
-  twchoice->global_redraw();
-  thchoice->global_redraw();
-
-  gadiag->global_redraw();
-  lwdiag->global_redraw();
+  Settings &settings = *(myTideContext->settings);
+  std::map<std::string, void*>::iterator it = dialogs.begin();
+  while (it != dialogs.end()) {
+    switch (settings[it->first].interpretation) {
+    case Configurable::Iboolean:
+    case Configurable::Igldouble:
+    case Configurable::Iunit:
+      ((xxMultiChoice*)(it->second))->global_redraw();
+      break;
+    case Configurable::Iposint:
+      ((xxUnsignedChooser*)(it->second))->global_redraw();
+      break;
+    case Configurable::Iposdouble:
+    case Configurable::Icolor:
+    case Configurable::Istrftime:
+      ((xxHorizDialog*)(it->second))->global_redraw();
+      break;
+    default:
+      assert (0);
+    }
+    it++;
+  }
 
   Arg buttonargs[2] =  {
     {XtNbackground, (XtArgVal)mycontext->pixels[Colors::button]},
@@ -808,34 +626,35 @@ xxXTideRoot::~xxXTideRoot () {
   delete helpbutton;
   delete dismissbutton;
   delete label;
-  int a;
-  for (a=0; a<numcolors; a++)
-    delete colordiags[a];
-  delete dfdiag;
-  delete hfdiag;
-  delete tfdiag;
-  delete gadiag;
-  delete lwdiag;
-  delete gwchoice;
-  delete ghchoice;
-  delete glchoice;
-  delete twchoice;
-  delete thchoice;
-  delete cwchoice;
-  delete elchoice;
-  delete fechoice;
-  delete nfchoice;
-  delete nschoice;
-  delete inchoice;
-  delete tlchoice;
-  delete zchoice;
-  delete uchoice;
+
+  Settings &settings = *(myTideContext->settings);
+  std::map<std::string, void*>::iterator it = dialogs.begin();
+  while (it != dialogs.end()) {
+    switch (settings[it->first].interpretation) {
+    case Configurable::Iboolean:
+    case Configurable::Igldouble:
+    case Configurable::Iunit:
+      delete (xxMultiChoice*)(it->second);
+      break;
+    case Configurable::Iposint:
+      delete (xxUnsignedChooser*)(it->second);
+      break;
+    case Configurable::Iposdouble:
+    case Configurable::Icolor:
+    case Configurable::Istrftime:
+      delete (xxHorizDialog*)(it->second);
+      break;
+    default:
+      assert (0);
+    }
+    it++;
+  }
+
   delete viewbox;
   delete viewport;
   delete form;
   delete mycontext;
   delete myTideContext;
-  delete ud;
 }
 
 void xxXTideRoot::mainloop() {
@@ -845,82 +664,108 @@ void xxXTideRoot::mainloop() {
     xxMainLoop (mycontext);
   }
 
-  if (!(myTideContext->cls->l))
+  if ((*(myTideContext->settings))["l"].isnull)
     newChooser (XtGrabNone); // Let the disclaimer scroll.
   else
-    newWindows (myTideContext->cls);
+    commandLineWindows ();
 
   xxMainLoopForever (mycontext);
 }
 
-void xxXTideRoot::newHelpBox (const Dstr &help) {
-  (void) new xxHelpBox (myTideContext, mycontext, help);
+xxWindow *xxXTideRoot::newHelpBox (const Dstr &help) {
+  return new xxHelpBox (myTideContext, mycontext, help);
 }
 
-void xxXTideRoot::newGraph (Station *s, Timestamp t) {
-  (void) new xxGraphMode (myTideContext, mycontext, s, t);
+xxDrawable *xxXTideRoot::newGraph (Station *s, Timestamp t) {
+  return new xxGraphMode (myTideContext, mycontext, s, t);
 }
 
-void xxXTideRoot::newGraph (StationRef *sr) {
+xxDrawable *xxXTideRoot::newGraph (StationRef *sr) {
   Timestamp t ((time_t)(time(NULL)));
-  newGraph (sr->load (myTideContext), t);
+  return newGraph (sr->load (myTideContext), t);
 }
 
-void xxXTideRoot::newText (Station *s, Timestamp t) {
-  (void) new xxTextMode (myTideContext, mycontext, s, 2, t);
+xxDrawable *xxXTideRoot::newPlain (Station *s, Timestamp t) {
+  return new xxTextMode (myTideContext, mycontext, s, 'p', t);
 }
 
-void xxXTideRoot::newRaw (Station *s, Timestamp t) {
-  (void) new xxTextMode (myTideContext, mycontext, s, 0, t);
+xxDrawable *xxXTideRoot::newRaw (Station *s, Timestamp t) {
+  return new xxTextMode (myTideContext, mycontext, s, 'r', t);
 }
 
-void xxXTideRoot::newMediumRare (Station *s, Timestamp t) {
-  (void) new xxTextMode (myTideContext, mycontext, s, 1, t);
+xxDrawable *xxXTideRoot::newMediumRare (Station *s, Timestamp t) {
+  return new xxTextMode (myTideContext, mycontext, s, 'm', t);
 }
 
-void xxXTideRoot::newClock (Station *s, int nobuttonsflag) {
-  (void) new xxClock (myTideContext, mycontext, s, nobuttonsflag);
+xxDrawable *xxXTideRoot::newClock (Station *s, int nobuttonsflag) {
+  return new xxClock (myTideContext, mycontext, s, nobuttonsflag);
 }
 
-void xxXTideRoot::newWindows (CommandLineSettings *cls) {
+xxDrawable *xxXTideRoot::newClock (Station *s) {
+  return newClock (s, ((*(s->context->settings))["cb"].c == 'n'));
+}
+
+void xxXTideRoot::commandLineWindows () {
+  Settings &settings = *(myTideContext->settings);
   Timestamp t ((time_t)(time(NULL)));
-  assert (cls);
-  struct CommandLineSettings::lnode *l = cls->l;
-  while (l) {
-    StationRef *sr;
-    sr = (*(myTideContext->stationIndex()))[l->name];
-    if (sr) {
-      if (!(cls->b.isNull())) {
-        t = Timestamp (cls->b, sr->timezone, myTideContext->settings);
-	if (t.isNull()) {
-	  Dstr details ("The offending input was ");
-	  details += cls->b;
-	  details += "\nin the time zone ";
-	  if (myTideContext->settings->z == "n")
-	    details += sr->timezone;
-	  else
-	    details += "UTC0";
-	  barf (MKTIME_FAILED, details);
+  char mode;
+  {
+    Configurable &cfbl = settings["m"];
+    if (cfbl.isnull)
+      mode = 'x';
+    else
+      mode = cfbl.c;
+  }{
+    Configurable &l = settings["l"];
+    DvectorIterator it = l.v.begin();
+    DvectorIterator stop = l.v.end();
+    while (it != stop) {
+      StationRef *sr;
+      sr = (*(myTideContext->stationIndex()))[*it];
+      if (sr) {
+        Configurable &b = settings["b"];
+	if (!b.isnull) {
+	  t = Timestamp (b.s, sr->timezone, myTideContext->settings);
+	  if (t.isNull()) {
+	    Dstr details ("The offending input was ");
+	    details += b.s;
+	    details += "\nin the time zone ";
+	    if (settings["z"].c == 'n')
+	      details += sr->timezone;
+	    else
+	      details += "UTC0";
+	    barf (MKTIME_FAILED, details);
+	  }
 	}
+	Station *s = sr->load (myTideContext);
+	xxWindow *d = NULL;
+	if (mode == 'g')
+	  d = newGraph (s, t);
+	else if (mode == 'p')
+	  d = newPlain (s, t);
+	else if (mode == 'a') {
+          d = newAbout (s);
+          delete s;
+	} else
+	  d = newClock (s);
+        assert (d);
+
+	// Apply -geometry to first one only.
+	if (it == l.v.begin())
+	  d->move (settings["X"].s);
+
+      } else {
+	Dstr details ("Could not find: ");
+	details += *it;
+	barf (STATION_NOT_FOUND, details);
       }
-      Station *s = sr->load (myTideContext);
-      if (cls->m == "g")
-        newGraph (s, t);
-      else if (cls->m == "p")
-        newText (s, t);
-      else
-        newClock (s, 1);
-    } else {
-      Dstr details ("Could not find: ");
-      details += l->name;
-      barf (STATION_NOT_FOUND, details);
+      it++;
     }
-    l = l->next;
   }
 }
 
 void xxXTideRoot::newChooser (XtGrabKind in_grabkind) {
-  if (myTideContext->settings->fe == "n")
+  if ((*(myTideContext->settings))["fe"].c == 'n')
     (void) new xxGlobe (mycontext, *(myTideContext->stationIndex(in_grabkind)),
       myTideContext);
   else
@@ -938,6 +783,12 @@ void xxXTideRoot::newGlobe (XtGrabKind in_grabkind) {
     myTideContext);
 }
 
+xxWindow *xxXTideRoot::newAbout (const Station *s) {
+  Dstr verbiage;
+  s->about (verbiage, 't');
+  return newHelpBox (verbiage);
+}
+
 void xxXTideRoot::show() {
   if (!(mycontext->is_realized)) {
     mycontext->realize();
